/*
   This file is autogenerated. Please do not change it directly. Run instead
   ./get_references.pl
*/

/*
 Copyright (C) 2006-2007 M.A.L. Marques

 This Source Code Form is subject to the terms of the Mozilla Public
 License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

#include <xc.h>

extern func_reference_type xc_ref_Acharya1980_6978;
extern func_reference_type xc_ref_Adamo1997_242;
extern func_reference_type xc_ref_Adamo1998_664;
extern func_reference_type xc_ref_Adamo1999_6158;
extern func_reference_type xc_ref_Adamo2002_5933;
extern func_reference_type xc_ref_Adamson1998_6;
extern func_reference_type xc_ref_Ai2021_1207;
extern func_reference_type xc_ref_Akinaga2008_348;
extern func_reference_type xc_ref_Anderson2017_1656;
extern func_reference_type xc_ref_Armiento2005_085108;
extern func_reference_type xc_ref_Armiento2013_036402;
extern func_reference_type xc_ref_Aschebrock2019_033082;
extern func_reference_type xc_ref_Attaccalite2002_256601;
extern func_reference_type xc_ref_Austin2012_4989;
extern func_reference_type xc_ref_Baer2005_043002;
extern func_reference_type xc_ref_Baltin1972_1176;
extern func_reference_type xc_ref_Bartok2019_161101;
extern func_reference_type xc_ref_Becke1986_4524;
extern func_reference_type xc_ref_Becke1986_7184;
extern func_reference_type xc_ref_Becke1988_1053;
extern func_reference_type xc_ref_Becke1988_3098;
extern func_reference_type xc_ref_Becke1989_3761;
extern func_reference_type xc_ref_Becke1993_1372;
extern func_reference_type xc_ref_Becke1993_5648;
extern func_reference_type xc_ref_Becke1994_625;
extern func_reference_type xc_ref_Becke1996_1040;
extern func_reference_type xc_ref_Becke1997_8554;
extern func_reference_type xc_ref_Becke1998_2092;
extern func_reference_type xc_ref_Becke2000_4020;
extern func_reference_type xc_ref_Becke2006_221101;
extern func_reference_type xc_ref_Berland2014_035412;
extern func_reference_type xc_ref_Bernard2012_204103;
extern func_reference_type xc_ref_Besley2009_10350;
extern func_reference_type xc_ref_Bilc2008_165107;
extern func_reference_type xc_ref_Bircher2018_3184;
extern func_reference_type xc_ref_Bloch1929_545;
extern func_reference_type xc_ref_Boese2000_1670;
extern func_reference_type xc_ref_Boese2001_5497;
extern func_reference_type xc_ref_Boese2002_9559;
extern func_reference_type xc_ref_Boese2003_5965;
extern func_reference_type xc_ref_Boese2004_3405;
extern func_reference_type xc_ref_Brandenburg2016_115144;
extern func_reference_type xc_ref_Bremond2012_1184;
extern func_reference_type xc_ref_Bremond2016_1059;
extern func_reference_type xc_ref_Bremond2016_244102;
extern func_reference_type xc_ref_Brown2021_2004;
extern func_reference_type xc_ref_Brual1978_1177;
extern func_reference_type xc_ref_Burke2014_4834;
extern func_reference_type xc_ref_Caldeira2019_62;
extern func_reference_type xc_ref_Campi1978_263;
extern func_reference_type xc_ref_Campo2012_179;
extern func_reference_type xc_ref_Cancio2006_081202;
extern func_reference_type xc_ref_Cancio2016_084107;
extern func_reference_type xc_ref_Cancio2017_618;
extern func_reference_type xc_ref_Cancio2018_084116;
extern func_reference_type xc_ref_Carmona2015_054105;
extern func_reference_type xc_ref_Carmona2016_120;
extern func_reference_type xc_ref_Carmona2019_303;
extern func_reference_type xc_ref_Casula2006_245427;
extern func_reference_type xc_ref_Chachiyo2016_021101;
extern func_reference_type xc_ref_Chachiyo2020_112669;
extern func_reference_type xc_ref_Chachiyo2020_3485;
extern func_reference_type xc_ref_Chai2008_084106;
extern func_reference_type xc_ref_Chai2008_6615;
extern func_reference_type xc_ref_Chen2018_073803;
extern func_reference_type xc_ref_Chiodo2012_126402;
extern func_reference_type xc_ref_Cohen2000_160;
extern func_reference_type xc_ref_Cohen2001_607;
extern func_reference_type xc_ref_Cohen2007_191109;
extern func_reference_type xc_ref_Colle1975_329;
extern func_reference_type xc_ref_Constantin2009_035125;
extern func_reference_type xc_ref_Constantin2011_186406;
extern func_reference_type xc_ref_Constantin2011_233103;
extern func_reference_type xc_ref_Constantin2012_035130;
extern func_reference_type xc_ref_Constantin2012_194105;
extern func_reference_type xc_ref_Constantin2013_2256;
extern func_reference_type xc_ref_Constantin2016_045126;
extern func_reference_type xc_ref_Constantin2016_115127;
extern func_reference_type xc_ref_Constantin2017_115153;
extern func_reference_type xc_ref_Constantin2018_4385;
extern func_reference_type xc_ref_Cooper2010_161104;
extern func_reference_type xc_ref_Cortona2012_086101;
extern func_reference_type xc_ref_Csonka2010_3688;
extern func_reference_type xc_ref_Dahlke2005_15677;
extern func_reference_type xc_ref_Day2006_094103;
extern func_reference_type xc_ref_DePristo1987_1425;
extern func_reference_type xc_ref_DePristo1987_438;
extern func_reference_type xc_ref_Dirac1930_376;
extern func_reference_type xc_ref_Eich2014_224107;
extern func_reference_type xc_ref_Elliott2009_1485;
extern func_reference_type xc_ref_Engel1992_7;
extern func_reference_type xc_ref_Engel1993_13164;
extern func_reference_type xc_ref_Engel1995_2750;
extern func_reference_type xc_ref_Entwistle2016_205134;
extern func_reference_type xc_ref_Ernzerhof1998_3313;
extern func_reference_type xc_ref_Ernzerhof1999_5029;
extern func_reference_type xc_ref_Ernzerhof1999_911;
extern func_reference_type xc_ref_Ernzerhof2000_59;
extern func_reference_type xc_ref_EspinosaLeal2015_31463;
extern func_reference_type xc_ref_Fabiano2010_113104;
extern func_reference_type xc_ref_Fabiano2013_673;
extern func_reference_type xc_ref_Fabiano2014_2016;
extern func_reference_type xc_ref_Fabiano2015_122;
extern func_reference_type xc_ref_Fermi1927_602;
extern func_reference_type xc_ref_Filatov1997_603;
extern func_reference_type xc_ref_Filatov1997_847;
extern func_reference_type xc_ref_Filatov1998_189;
extern func_reference_type xc_ref_Finzel2017_40;
extern func_reference_type xc_ref_Fuentealba1995_31;
extern func_reference_type xc_ref_Furness2018;
extern func_reference_type xc_ref_Furness2019_041119;
extern func_reference_type xc_ref_Furness2020_8208;
extern func_reference_type xc_ref_Furness2020_9248;
extern func_reference_type xc_ref_Furness2021;
extern func_reference_type xc_ref_Gaiduk2011_012509;
extern func_reference_type xc_ref_Garza2018_3083;
extern func_reference_type xc_ref_Gazquez1982_1467;
extern func_reference_type xc_ref_GellMann1957_364;
extern func_reference_type xc_ref_Ghiringhelli2008_073104;
extern func_reference_type xc_ref_Ghiringhelli2010_014106;
extern func_reference_type xc_ref_Ghosh1985_3307;
extern func_reference_type xc_ref_Gilbert1999_511;
extern func_reference_type xc_ref_Gill1996_1005;
extern func_reference_type xc_ref_Gill1996_433;
extern func_reference_type xc_ref_Goedecker1996_1703;
extern func_reference_type xc_ref_Goerigk2010_107;
extern func_reference_type xc_ref_Golden1957_604;
extern func_reference_type xc_ref_Gombas1965_137;
extern func_reference_type xc_ref_Gordon1972_3122;
extern func_reference_type xc_ref_Grimme2005_3067;
extern func_reference_type xc_ref_Grimme2006_034108;
extern func_reference_type xc_ref_Grimme2006_1787;
extern func_reference_type xc_ref_Grimme2010_154104;
extern func_reference_type xc_ref_Groth2017_135001;
extern func_reference_type xc_ref_Gunnarsson1976_4274;
extern func_reference_type xc_ref_Haas2011_205117;
extern func_reference_type xc_ref_Haiduke2018_184106;
extern func_reference_type xc_ref_Hamada2014_121103;
extern func_reference_type xc_ref_Hammer1999_7413;
extern func_reference_type xc_ref_Hamprecht1998_6264;
extern func_reference_type xc_ref_Handy1998_707;
extern func_reference_type xc_ref_Handy2001_403;
extern func_reference_type xc_ref_Handy2002_5411;
extern func_reference_type xc_ref_Hedin1971_2064;
extern func_reference_type xc_ref_Helbig2011_032503;
extern func_reference_type xc_ref_Henderson2007_221103;
extern func_reference_type xc_ref_Henderson2008_194105;
extern func_reference_type xc_ref_Henderson2009_044108;
extern func_reference_type xc_ref_Herman1969_807;
extern func_reference_type xc_ref_Herman1969_827;
extern func_reference_type xc_ref_Heyd2003_8207;
extern func_reference_type xc_ref_Heyd2003_8207_err;
extern func_reference_type xc_ref_Heyd2004_7274;
extern func_reference_type xc_ref_Hodges1973_1428;
extern func_reference_type xc_ref_Hu1985_391;
extern func_reference_type xc_ref_Hui2016_044114;
extern func_reference_type xc_ref_Iikura2001_3540;
extern func_reference_type xc_ref_Jana2017_4804;
extern func_reference_type xc_ref_Jana2018_1;
extern func_reference_type xc_ref_Jana2018_8999;
extern func_reference_type xc_ref_Jana2019_6356;
extern func_reference_type xc_ref_Jana2021_024103;
extern func_reference_type xc_ref_Jana2021_063007;
extern func_reference_type xc_ref_Jemmer1995_3571;
extern func_reference_type xc_ref_Jin2016_034107;
extern func_reference_type xc_ref_Jin2018_064111;
extern func_reference_type xc_ref_Kang2001_11040;
extern func_reference_type xc_ref_Karasiev2006_111;
extern func_reference_type xc_ref_Karasiev2009_245120;
extern func_reference_type xc_ref_Karasiev2013_161108;
extern func_reference_type xc_ref_Karasiev2014_076403;
extern func_reference_type xc_ref_Karasiev2016_157101;
extern func_reference_type xc_ref_Karasiev2018_076401;
extern func_reference_type xc_ref_Karton2008_12868;
extern func_reference_type xc_ref_Kaupp2011_16973;
extern func_reference_type xc_ref_Keal2003_3015;
extern func_reference_type xc_ref_Keal2004_5654;
extern func_reference_type xc_ref_Keal2005_121103;
extern func_reference_type xc_ref_Kirznits1957_115;
extern func_reference_type xc_ref_Klimes2010_022201;
extern func_reference_type xc_ref_Klimes2011_195131;
extern func_reference_type xc_ref_Koehl1996_835;
extern func_reference_type xc_ref_Kohn1965_A1133;
extern func_reference_type xc_ref_Kompaneets1956_427;
extern func_reference_type xc_ref_Kraisler2010_042516;
extern func_reference_type xc_ref_Krieger1999_463;
extern func_reference_type xc_ref_Krieger2001_48;
extern func_reference_type xc_ref_Krukau2006_224106;
extern func_reference_type xc_ref_Kurth1999_889;
extern func_reference_type xc_ref_Lacks1993_4681;
extern func_reference_type xc_ref_Lacks1994_4446;
extern func_reference_type xc_ref_Langreth1981_446;
extern func_reference_type xc_ref_Laricchia2011_2439;
extern func_reference_type xc_ref_Laricchia2014_164;
extern func_reference_type xc_ref_Lee1987_2377;
extern func_reference_type xc_ref_Lee1988_785;
extern func_reference_type xc_ref_Lee1990_193;
extern func_reference_type xc_ref_Lee1991_768;
extern func_reference_type xc_ref_Lehtola2018_1;
extern func_reference_type xc_ref_Lehtola2021_943;
extern func_reference_type xc_ref_Lehtomaki2019_165111;
extern func_reference_type xc_ref_Lembarki1994_5328;
extern func_reference_type xc_ref_Lieb1981_603;
extern func_reference_type xc_ref_Lin2004_365;
extern func_reference_type xc_ref_Lin2013_263;
extern func_reference_type xc_ref_Liu1996_2211;
extern func_reference_type xc_ref_Liu2000_29;
extern func_reference_type xc_ref_Livshits2007_2932;
extern func_reference_type xc_ref_Loos2013_064108;
extern func_reference_type xc_ref_Loos2017_114108;
extern func_reference_type xc_ref_Lu2013_64;
extern func_reference_type xc_ref_Ludena1986;
extern func_reference_type xc_ref_Lundgaard2016_235162;
extern func_reference_type xc_ref_Luo2018_041111;
extern func_reference_type xc_ref_Lynch2000_4811;
extern func_reference_type xc_ref_MacDonald1979_2977;
extern func_reference_type xc_ref_Madsen2007_195108;
extern func_reference_type xc_ref_Manby2000_7002;
extern func_reference_type xc_ref_Mardirossian2014_9904;
extern func_reference_type xc_ref_Mardirossian2015_074111;
extern func_reference_type xc_ref_Mardirossian2016_214110;
extern func_reference_type xc_ref_Margraf2019_244116;
extern func_reference_type xc_ref_Marques2012_2272;
extern func_reference_type xc_ref_Mattsson2008_084714;
extern func_reference_type xc_ref_McWeeny1976_3;
extern func_reference_type xc_ref_Mejia2017_052512;
extern func_reference_type xc_ref_Mejia2018_115161;
extern func_reference_type xc_ref_Mejia2020_121109;
extern func_reference_type xc_ref_Menconi2001_3958;
extern func_reference_type xc_ref_Meyer1976_898;
extern func_reference_type xc_ref_Mezei2018_2469;
extern func_reference_type xc_ref_Miehlich1989_200;
extern func_reference_type xc_ref_Mortensen2005_216401;
extern func_reference_type xc_ref_Mosquera2016_1605;
extern func_reference_type xc_ref_Moussa2012_204117;
extern func_reference_type xc_ref_Murray2009_2754;
extern func_reference_type xc_ref_Negele1972_1472;
extern func_reference_type xc_ref_Neupane2021_063803;
extern func_reference_type xc_ref_Odashima2009_798;
extern func_reference_type xc_ref_Okuno2012_29;
extern func_reference_type xc_ref_Ortiz1994_1391;
extern func_reference_type xc_ref_Ortiz1994_1391_err;
extern func_reference_type xc_ref_OuYang1991_379;
extern func_reference_type xc_ref_PachecoKato2016_268;
extern func_reference_type xc_ref_Patra2018_8991;
extern func_reference_type xc_ref_Patra2019_045147;
extern func_reference_type xc_ref_Patra2019_155140;
extern func_reference_type xc_ref_Patra2019_19639;
extern func_reference_type xc_ref_Patra2020_184112;
extern func_reference_type xc_ref_Paziani2006_155111;
extern func_reference_type xc_ref_Pearson1983;
extern func_reference_type xc_ref_Pearson1985_881;
extern func_reference_type xc_ref_Pedroza2009_201106;
extern func_reference_type xc_ref_Peng2016_041005;
extern func_reference_type xc_ref_Perdew1981_5048;
extern func_reference_type xc_ref_Perdew1981_5048_mod;
extern func_reference_type xc_ref_Perdew1986_8800;
extern func_reference_type xc_ref_Perdew1986_8822;
extern func_reference_type xc_ref_Perdew1991;
extern func_reference_type xc_ref_Perdew1992_13244;
extern func_reference_type xc_ref_Perdew1992_13244_mod;
extern func_reference_type xc_ref_Perdew1992_6671;
extern func_reference_type xc_ref_Perdew1992_6671_err;
extern func_reference_type xc_ref_Perdew1992_79;
extern func_reference_type xc_ref_Perdew1996_3865;
extern func_reference_type xc_ref_Perdew1996_3865_err;
extern func_reference_type xc_ref_Perdew1999_2544;
extern func_reference_type xc_ref_Perdew2004_6898;
extern func_reference_type xc_ref_Perdew2007_042506;
extern func_reference_type xc_ref_Perdew2007_155109;
extern func_reference_type xc_ref_Perdew2008_136406;
extern func_reference_type xc_ref_Perdew2009_026403;
extern func_reference_type xc_ref_Perdew2009_026403_err;
extern func_reference_type xc_ref_Perez2015_3844;
extern func_reference_type xc_ref_Pernal2009_263201;
extern func_reference_type xc_ref_Peverati2011_191102;
extern func_reference_type xc_ref_Peverati2011_1991;
extern func_reference_type xc_ref_Peverati2011_2810;
extern func_reference_type xc_ref_Peverati2012_117;
extern func_reference_type xc_ref_Peverati2012_13171;
extern func_reference_type xc_ref_Peverati2012_16187;
extern func_reference_type xc_ref_Peverati2012_2310;
extern func_reference_type xc_ref_Pittalis2007_235314;
extern func_reference_type xc_ref_Pittalis2008_195322;
extern func_reference_type xc_ref_Pittalis2009_012503;
extern func_reference_type xc_ref_Pittalis2010_115108;
extern func_reference_type xc_ref_Proynov1994_7874;
extern func_reference_type xc_ref_Proynov1998_12616;
extern func_reference_type xc_ref_Proynov2000_10013;
extern func_reference_type xc_ref_Proynov2006_436;
extern func_reference_type xc_ref_Proynov2008_103;
extern func_reference_type xc_ref_Proynov2009_014103;
extern func_reference_type xc_ref_Proynov2017_059904;
extern func_reference_type xc_ref_Rae1973_574;
extern func_reference_type xc_ref_Ragot2004_7671;
extern func_reference_type xc_ref_Rajagopal1978_L943;
extern func_reference_type xc_ref_Rasanen2010_044112;
extern func_reference_type xc_ref_Reiher2001_48;
extern func_reference_type xc_ref_Renz2009_16292;
extern func_reference_type xc_ref_Rey1998_581;
extern func_reference_type xc_ref_Rinke2012_126404;
extern func_reference_type xc_ref_Rohrdanz2009_054112;
extern func_reference_type xc_ref_Ruggeri2018_161105;
extern func_reference_type xc_ref_Ruzsinszky2009_763;
extern func_reference_type xc_ref_Ruzsinszky2012_2078;
extern func_reference_type xc_ref_Savin1995_327;
extern func_reference_type xc_ref_Schimka2011_024116;
extern func_reference_type xc_ref_Schipper2000_1344;
extern func_reference_type xc_ref_Schmider1998_9624;
extern func_reference_type xc_ref_Schultz2005_11127;
extern func_reference_type xc_ref_Seth2012_901;
extern func_reference_type xc_ref_Seth2013_2286;
extern func_reference_type xc_ref_Shao2003_4807;
extern func_reference_type xc_ref_Shao2020_587;
extern func_reference_type xc_ref_Slater1951_385;
extern func_reference_type xc_ref_Song2007_154105;
extern func_reference_type xc_ref_Song2007_154109;
extern func_reference_type xc_ref_Song2008_184113;
extern func_reference_type xc_ref_Song2009_144108;
extern func_reference_type xc_ref_Sparrow2021;
extern func_reference_type xc_ref_Staroverov2003_12129;
extern func_reference_type xc_ref_Stephens1994_11623;
extern func_reference_type xc_ref_Stewart1995_4337;
extern func_reference_type xc_ref_Sun2012_051101;
extern func_reference_type xc_ref_Sun2013_044113;
extern func_reference_type xc_ref_Sun2015_036402;
extern func_reference_type xc_ref_Sun2015_685;
extern func_reference_type xc_ref_Swart2009_094103;
extern func_reference_type xc_ref_Swart2009_69;
extern func_reference_type xc_ref_Swart2011_1117;
extern func_reference_type xc_ref_Swart2013_166;
extern func_reference_type xc_ref_Tahchieva2018_4806;
extern func_reference_type xc_ref_Tao2001_3519;
extern func_reference_type xc_ref_Tao2002_2335;
extern func_reference_type xc_ref_Tao2003_146401;
extern func_reference_type xc_ref_Tao2016_073001;
extern func_reference_type xc_ref_Tawada2004_8425;
extern func_reference_type xc_ref_Thakkar1992_6920;
extern func_reference_type xc_ref_Thakkar2009_134109;
extern func_reference_type xc_ref_Thomas1927_542;
extern func_reference_type xc_ref_Tognetti2007_381;
extern func_reference_type xc_ref_Tognetti2008_034101;
extern func_reference_type xc_ref_Tognetti2008_536;
extern func_reference_type xc_ref_Tognetti2009_14415;
extern func_reference_type xc_ref_Toulouse2002_10465;
extern func_reference_type xc_ref_Toulouse2004_1047;
extern func_reference_type xc_ref_Tozer1996_9200;
extern func_reference_type xc_ref_Tozer1997_183;
extern func_reference_type xc_ref_Tozer1998_2545;
extern func_reference_type xc_ref_Tozer1998_3162;
extern func_reference_type xc_ref_Tran2002_441;
extern func_reference_type xc_ref_Tran2009_226401;
extern func_reference_type xc_ref_Trickey2011_075146;
extern func_reference_type xc_ref_Tsuneda1999_10664;
extern func_reference_type xc_ref_Tsuneda1999_5656;
extern func_reference_type xc_ref_Tsuneda2000_15527;
extern func_reference_type xc_ref_Ugalde1994_423;
extern func_reference_type xc_ref_Urso2021_1;
extern func_reference_type xc_ref_VanVoorhis1998_400;
extern func_reference_type xc_ref_Vela2009_244103;
extern func_reference_type xc_ref_Vela2012_144115;
extern func_reference_type xc_ref_Verma2014_18A534;
extern func_reference_type xc_ref_Verma2017_380;
extern func_reference_type xc_ref_Verma2017_7144;
extern func_reference_type xc_ref_Verma2019_2966;
extern func_reference_type xc_ref_Vilhena2014;
extern func_reference_type xc_ref_Vitos1998_12611;
extern func_reference_type xc_ref_Vitos2000_052511;
extern func_reference_type xc_ref_Vitos2000_10046;
extern func_reference_type xc_ref_Vosko1980_1200;
extern func_reference_type xc_ref_Vydrov2006_234109;
extern func_reference_type xc_ref_Vydrov2010_244103;
extern func_reference_type xc_ref_Wang2017_8487;
extern func_reference_type xc_ref_Wang2018_10257;
extern func_reference_type xc_ref_Wang2020_2294;
extern func_reference_type xc_ref_Weintraub2009_754;
extern func_reference_type xc_ref_Weizsacker1935_431;
extern func_reference_type xc_ref_Wellendorff2012_235149;
extern func_reference_type xc_ref_Wellendorff2014_144107;
extern func_reference_type xc_ref_Wigner1938_678;
extern func_reference_type xc_ref_Wiitala2006_1085;
extern func_reference_type xc_ref_Wilson1990_12930;
extern func_reference_type xc_ref_Wilson1994_337;
extern func_reference_type xc_ref_Wilson1998_523;
extern func_reference_type xc_ref_Wilson2001_9233;
extern func_reference_type xc_ref_Wu2006_235116;
extern func_reference_type xc_ref_Xie2021_045130;
extern func_reference_type xc_ref_Xu2004_2673;
extern func_reference_type xc_ref_Xu2004_4068;
extern func_reference_type xc_ref_Yanai2004_51;
extern func_reference_type xc_ref_Yonei1965_1051;
extern func_reference_type xc_ref_Yu2015_12146;
extern func_reference_type xc_ref_Yu2016_1280;
extern func_reference_type xc_ref_Yu2016_5032;
extern func_reference_type xc_ref_Zhang1998_890;
extern func_reference_type xc_ref_Zhao1993_918;
extern func_reference_type xc_ref_Zhao2004_2715;
extern func_reference_type xc_ref_Zhao2004_6908;
extern func_reference_type xc_ref_Zhao2005_161103;
extern func_reference_type xc_ref_Zhao2005_2012;
extern func_reference_type xc_ref_Zhao2005_415;
extern func_reference_type xc_ref_Zhao2005_43;
extern func_reference_type xc_ref_Zhao2005_5656;
extern func_reference_type xc_ref_Zhao2006_13126;
extern func_reference_type xc_ref_Zhao2006_194101;
extern func_reference_type xc_ref_Zhao2006_364;
extern func_reference_type xc_ref_Zhao2008_184109;
extern func_reference_type xc_ref_Zhao2008_1849;
extern func_reference_type xc_ref_Zhao2008_215;
extern func_reference_type xc_ref_Zheng2009_808;
extern func_reference_type xc_ref_delCampo2012_104108;
extern func_reference_type xc_ref_gaussianimplementation;
extern func_reference_type xc_ref_lda_xc_corrksdt_note;
extern func_reference_type xc_ref_nwchemimplementation;
extern func_reference_type xc_ref_vanLeeuwen1994_2421;
extern func_reference_type xc_ref_vonBarth1972_1629;
