/*
   This file is autogenerated. Please do not change it directly. Run instead
   ./get_references.pl
*/

/*
 Copyright (C) 2006-2007 M.A.L. Marques

 This Source Code Form is subject to the terms of the Mozilla Public
 License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

#include <xc.h>

func_reference_type xc_ref_Acharya1980_6978 = {
  "P. K. Acharya, L. J. Bartolotti, S. B. Sears, and R. G. Parr, Proc. Natl. Acad. Sci. U. S. A. 77, 6978 (1980)",
  "10.1073/pnas.77.12.6978",
  "@article{Acharya1980_6978,\n  title = {An atomic kinetic energy functional with full Weizsacker correction},\n  author = {P. K. Acharya and L. J. Bartolotti and S. B. Sears and R. G. Parr},\n  volume = {77},\n  number = {12},\n  pages = {6978},\n  year = {1980},\n  doi = {10.1073/pnas.77.12.6978},\n  url = {http://www.pnas.org/content/77/12/6978.abstract},\n  journal = {Proc. Natl. Acad. Sci. U. S. A.}\n}"
};


func_reference_type xc_ref_Adamo1997_242 = {
  "C. Adamo and V. Barone, Chem. Phys. Lett. 274, 242 (1997)",
  "10.1016/S0009-2614(97)00651-9",
  "@article{Adamo1997_242,\n  title = \"Toward reliable adiabatic connection models free from adjustable parameters\",\n  author = \"C. Adamo and V. Barone\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"274\",\n  number = \"1--3\",\n  pages = \"242\",\n  year = \"1997\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/S0009-2614(97)00651-9\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261497006519\",\n}"
};


func_reference_type xc_ref_Adamo1998_664 = {
  "C. Adamo and V. Barone, J. Chem. Phys. 108, 664 (1998)",
  "10.1063/1.475428",
  "@article{Adamo1998_664,\n  title = \"Exchange functionals with improved long-range behavior and adiabatic connection methods without adjustable parameters: The mPW and mPW1PW models\",\n  author = \"C. Adamo and V. Barone\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1998\",\n  volume = \"108\",\n  number = \"2\",\n  pages = \"664\",\n  doi = \"10.1063/1.475428\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/108/2/10.1063/1.475428\"\n}"
};


func_reference_type xc_ref_Adamo1999_6158 = {
  "C. Adamo and V. Barone, J. Chem. Phys. 110, 6158 (1999)",
  "10.1063/1.478522",
  "@article{Adamo1999_6158,\n  title = \"Toward reliable density functional methods without adjustable parameters: The PBE0 model\",\n  author = \"C. Adamo and V. Barone\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1999\",\n  volume = \"110\",\n  number = \"13\",\n  pages = \"6158\",\n  doi = \"10.1063/1.478522\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/110/13/10.1063/1.478522\"\n}"
};


func_reference_type xc_ref_Adamo2002_5933 = {
  "C. Adamo and V. Barone, J. Chem. Phys. 116, 5933 (2002)",
  "10.1063/1.1458927",
  "@article{Adamo2002_5933,\n  title = \"Physically motivated density functionals with improved performances: The modified Perdew--Burke--Ernzerhof model\",\n  author = \"C. Adamo and V. Barone\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2002\",\n  volume = \"116\",\n  number = \"14\",\n  pages = \"5933\",\n  doi = \"10.1063/1.1458927\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/116/14/10.1063/1.1458927\"\n}"
};


func_reference_type xc_ref_Adamson1998_6 = {
  "R. D. Adamson, P. M. W. Gill, and J. A. Pople, Chem. Phys. Lett. 284, 6 (1998)",
  "10.1016/S0009-2614(97)01282-7",
  "@article{Adamson1998_6,\n  title = \"Empirical density functionals\",\n  author = \"R. D. Adamson and P. M. W. Gill and J. A. Pople\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"284\",\n  number = \"1--2\",\n  pages = \"6\",\n  year = \"1998\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/S0009-2614(97)01282-7\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261497012827\"\n}"
};


func_reference_type xc_ref_Ai2021_1207 = {
  "W. Ai, W.-H. Fang, and N. Q. Su, J. Phys. Chem. Lett. 12, 1207 (2021)",
  "10.1021/acs.jpclett.0c03621",
  "@article{Ai2021_1207,\n  author = {Ai, Wenna and Fang, Wei-Hai and Su, Neil Qiang},\n  title = {The Role of Range-Separated Correlation in Long-Range Corrected Hybrid Functionals},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {12},\n  pages = {1207-1213},\n  year = {2021},\n  doi = {10.1021/acs.jpclett.0c03621},\n  url = {https://doi.org/10.1021/acs.jpclett.0c03621}\n}"
};


func_reference_type xc_ref_Akinaga2008_348 = {
  "Y. Akinaga and S. Ten-no, Chem. Phys. Lett. 462, 348 (2008)",
  "10.1016/j.cplett.2008.07.103",
  "@article{Akinaga2008_348,\n  title = \"Range-separation by the Yukawa potential in long-range corrected density functional theory with Gaussian-type basis functions\",\n  author = \"Y. Akinaga and S. Ten-no\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"462\",\n  number = \"4--6\",\n  pages = \"348\",\n  year = \"2008\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2008.07.103\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261408010609\"\n}"
};


func_reference_type xc_ref_Anderson2017_1656 = {
  "L. N. Anderson, M. B. Oviedo, and B. M. Wong, J. Chem. Theory Comput. 13, 1656 (2017)",
  "10.1021/acs.jctc.6b01249",
  "@article{Anderson2017_1656,\n  author = {Anderson, Lindsey N. and Oviedo, M. Bel\\'{e}n and Wong, Bryan M.},\n  title = {Accurate Electron Affinities and Orbital Energies of Anions from a Nonempirically Tuned Range-Separated Density Functional Theory Approach},\n  journal = {J. Chem. Theory Comput.},\n  volume = {13},\n  number = {4},\n  pages = {1656-1666},\n  year = {2017},\n  doi = {10.1021/acs.jctc.6b01249},\n  url = {https://doi.org/10.1021/acs.jctc.6b01249}\n}"
};


func_reference_type xc_ref_Armiento2005_085108 = {
  "R. Armiento and A. E. Mattsson, Phys. Rev. B 72, 085108 (2005)",
  "10.1103/PhysRevB.72.085108",
  "@article{Armiento2005_085108,\n  title = {Functional designed to include surface effects in self-consistent density functional theory},\n  author = {R. Armiento and A. E. Mattsson},\n  journal = {Phys. Rev. B},\n  volume = {72},\n  issue = {8},\n  pages = {085108},\n  year = {2005},\n  month = {Aug},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.72.085108},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.72.085108}\n}"
};


func_reference_type xc_ref_Armiento2013_036402 = {
  "R. Armiento and S. Kummel, Phys. Rev. Lett. 111, 036402 (2013)",
  "10.1103/PhysRevLett.111.036402",
  "@article{Armiento2013_036402,\n  title = {Orbital Localization, Charge Transfer, and Band Gaps in Semilocal Density-Functional Theory},\n  author = {R. Armiento and S. K\\\"{u}mmel},\n  journal = {Phys. Rev. Lett.},\n  volume = {111},\n  issue = {3},\n  pages = {036402},\n  year = {2013},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.111.036402},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.111.036402}\n}"
};


func_reference_type xc_ref_Aschebrock2019_033082 = {
  "T. Aschebrock and S. Kummel, Phys. Rev. Res. 1, 033082 (2019)",
  "10.1103/PhysRevResearch.1.033082",
  "@article{Aschebrock2019_033082,\n  title = {Ultranonlocality and accurate band gaps from a meta-generalized gradient approximation},\n  author = {Aschebrock, Thilo and K\\\"{u}mmel, Stephan},\n  journal = {Phys. Rev. Res.},\n  volume = {1},\n  issue = {3},\n  pages = {033082},\n  year = {2019},\n  month = {Nov},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevResearch.1.033082},\n  url = {https://link.aps.org/doi/10.1103/PhysRevResearch.1.033082}\n}"
};


func_reference_type xc_ref_Attaccalite2002_256601 = {
  "C. Attaccalite, S. Moroni, P. Gori-Giorgi, and G. B. Bachelet, Phys. Rev. Lett. 88, 256601 (2002)",
  "10.1103/PhysRevLett.88.256601",
  "@article{Attaccalite2002_256601,\n  title = {Correlation Energy and Spin Polarization in the 2D Electron Gas},\n  author = {C. Attaccalite and S. Moroni and P. Gori-Giorgi and G. B. Bachelet},\n  journal = {Phys. Rev. Lett.},\n  volume = {88},\n  issue = {25},\n  pages = {256601},\n  year = {2002},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.88.256601},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.88.256601}\n}"
};


func_reference_type xc_ref_Austin2012_4989 = {
  "A. Austin, G. A. Petersson, M. J. Frisch, F. J. Dobek, G. Scalmani, and K. Throssell, J. Chem. Theory Comput. 8, 4989 (2012)",
  "10.1021/ct300778e",
  "@article{Austin2012_4989,\n  author = {Austin, Amy and Petersson, George A. and Frisch, Michael J. and Dobek, Frank J. and Scalmani, Giovanni and Throssell, Kyle},\n  title = {A Density Functional with Spherical Atom Dispersion Terms},\n  journal = {J. Chem. Theory Comput.},\n  volume = {8},\n  number = {12},\n  pages = {4989-5007},\n  year = {2012},\n  doi = {10.1021/ct300778e},\n  url = {https://doi.org/10.1021/ct300778e}\n}"
};


func_reference_type xc_ref_Baer2005_043002 = {
  "R. Baer and D. Neuhauser, Phys. Rev. Lett. 94, 043002 (2005)",
  "10.1103/PhysRevLett.94.043002",
  "@article{Baer2005_043002,\n  title = {Density Functional Theory with Correct Long-Range Asymptotic Behavior},\n  author = {Baer, Roi and Neuhauser, Daniel},\n  journal = {Phys. Rev. Lett.},\n  volume = {94},\n  issue = {4},\n  pages = {043002},\n  year = {2005},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.94.043002},\n  url = {https://link.aps.org/doi/10.1103/PhysRevLett.94.043002}\n}"
};


func_reference_type xc_ref_Baltin1972_1176 = {
  "R. Baltin, Z. Naturforsch. A 27, 1176 (1972)",
  "10.1515/zna-1972-8-903",
  "@article{Baltin1972_1176,\n  author = \"R. Baltin\",\n  title = \"The Energy-Density Functional of an Electron Gas in Locally Linear Approximation of the One-Body Potential\",\n  journal = \"Z. Naturforsch. A\",\n  year = \"1972\",\n  publisher = \"De Gruyter\",\n  address = \"Berlin, Boston\",\n  volume = \"27\",\n  number = \"8-9\",\n  doi = \"10.1515/zna-1972-8-903\",\n  pages = \"1176 - 1186\",\n  url = \"https://www.degruyter.com/view/journals/zna/27/8-9/article-p1176.xml\"\n}"
};


func_reference_type xc_ref_Bartok2019_161101 = {
  "A. P. Bartok and J. R. Yates, J. Chem. Phys. 150, 161101 (2019)",
  "10.1063/1.5094646",
  "@article{Bartok2019_161101,\n  author = {Bart\\'{o}k, Albert P. and Yates, Jonathan R.},\n  title = {Regularized SCAN functional},\n  journal = {J. Chem. Phys.},\n  volume = {150},\n  number = {16},\n  pages = {161101},\n  year = {2019},\n  doi = {10.1063/1.5094646},\n  url = {https://doi.org/10.1063/1.5094646}\n}"
};


func_reference_type xc_ref_Becke1986_4524 = {
  "A. D. Becke, J. Chem. Phys. 84, 4524 (1986)",
  "10.1063/1.450025",
  "@article{Becke1986_4524,\n  title = \"Density functional calculations of molecular bond energies\",\n  author = \"A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1986\",\n  volume = \"84\",\n  number = \"8\",\n  pages = \"4524\",\n  doi = \"10.1063/1.450025\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/84/8/10.1063/1.450025\"\n}"
};


func_reference_type xc_ref_Becke1986_7184 = {
  "A. D. Becke, J. Chem. Phys. 85, 7184 (1986)",
  "10.1063/1.451353",
  "@article{Becke1986_7184,\n  title = \"On the large-gradient behavior of the density functional exchange energy\",\n  author = \"A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1986\",\n  volume = \"85\",\n  number = \"12\",\n  pages = \"7184\",\n  doi = \"10.1063/1.451353\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/85/12/10.1063/1.451353\"\n}"
};


func_reference_type xc_ref_Becke1988_1053 = {
  "A. D. Becke, J. Chem. Phys. 88, 1053 (1988)",
  "10.1063/1.454274",
  "@article{Becke1988_1053,\n  author = {A. D. Becke},\n  title = {Correlation energy of an inhomogeneous electron gas: A coordinate-space model},\n  journal = {J. Chem. Phys.},\n  volume = {88},\n  number = {2},\n  pages = {1053-1062},\n  year = {1988},\n  doi = {10.1063/1.454274},\n  url = {http://doi.org/10.1063/1.454274},\n}"
};


func_reference_type xc_ref_Becke1988_3098 = {
  "A. D. Becke, Phys. Rev. A 38, 3098 (1988)",
  "10.1103/PhysRevA.38.3098",
  "@article{Becke1988_3098,\n  title = {Density-functional exchange-energy approximation with correct asymptotic behavior},\n  author = {A. D. Becke},\n  journal = {Phys. Rev. A},\n  volume = {38},\n  issue = {6},\n  pages = {3098},\n  year = {1988},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.38.3098},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.38.3098}\n}"
};


func_reference_type xc_ref_Becke1989_3761 = {
  "A. D. Becke and M. R. Roussel, Phys. Rev. A 39, 3761 (1989)",
  "10.1103/PhysRevA.39.3761",
  "@article{Becke1989_3761,\n  title = {Exchange holes in inhomogeneous systems: A coordinate-space model},\n  author = {A. D. Becke and M. R. Roussel},\n  journal = {Phys. Rev. A},\n  volume = {39},\n  issue = {8},\n  pages = {3761},\n  year = {1989},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.39.3761},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.39.3761}\n}"
};


func_reference_type xc_ref_Becke1993_1372 = {
  "A. D. Becke, J. Chem. Phys. 98, 1372 (1993)",
  "10.1063/1.464304",
  "@article{Becke1993_1372,\n  title = \"A new mixing of Hartree--Fock and local density-functional theories\",\n  author = \"A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1993\",\n  volume = \"98\",\n  number = \"2\",\n  pages = \"1372\",\n  doi = \"10.1063/1.464304\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/98/2/10.1063/1.464304\"\n}"
};


func_reference_type xc_ref_Becke1993_5648 = {
  "A. D. Becke, J. Chem. Phys. 98, 5648 (1993)",
  "10.1063/1.464913",
  "@article{Becke1993_5648,\n  title = \"Density-functional thermochemistry. III. The role of exact exchange\",\n  author = \"A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1993\",\n  volume = \"98\",\n  number = \"7\",\n  pages = \"5648\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/98/7/10.1063/1.464913\",\n  doi = \"10.1063/1.464913\"\n}"
};


func_reference_type xc_ref_Becke1994_625 = {
  "A. D. Becke, Int. J. Quantum Chem. 52, 625 (1994)",
  "10.1002/qua.560520855",
  "@article{Becke1994_625,\n  author = {Becke, Axel D.},\n  title = {Thermochemical tests of a kinetic-energy dependent exchange-correlation approximation},\n  journal = {Int. J. Quantum Chem.},\n  volume = {52},\n  number = {S28},\n  pages = {625-632},\n  doi = {10.1002/qua.560520855},\n  url = {https://onlinelibrary.wiley.com/doi/abs/10.1002/qua.560520855},\n  year = {1994}\n}"
};


func_reference_type xc_ref_Becke1996_1040 = {
  "A. D. Becke, J. Chem. Phys. 104, 1040 (1996)",
  "10.1063/1.470829",
  "@article{Becke1996_1040,\n  title = \"Density-functional thermochemistry. IV. A new dynamical correlation functional and implications for exact-exchange mixing\",\n  author = \"A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1996\",\n  volume = \"104\",\n  number = \"3\",\n  pages = \"1040\",\n  doi = \"10.1063/1.470829\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/104/3/10.1063/1.470829\"\n}"
};


func_reference_type xc_ref_Becke1997_8554 = {
  "A. D. Becke, J. Chem. Phys. 107, 8554 (1997)",
  "10.1063/1.475007",
  "@article{Becke1997_8554,\n  title = \"Density-functional thermochemistry. V. Systematic optimization of exchange-correlation functionals\",\n  author = \"A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1997\",\n  volume = \"107\",\n  number = \"20\",\n  pages = \"8554\",\n  doi = \"10.1063/1.475007\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/107/20/10.1063/1.475007\"\n}"
};


func_reference_type xc_ref_Becke1998_2092 = {
  "A. D. Becke, J. Chem. Phys. 109, 2092 (1998)",
  "10.1063/1.476722",
  "@article{Becke1998_2092,\n  author = {Becke, Axel D.},\n  title = {A new inhomogeneity parameter in density-functional theory},\n  journal = {J. Chem. Phys.},\n  volume = {109},\n  number = {6},\n  pages = {2092-2098},\n  year = {1998},\n  doi = {10.1063/1.476722},\n  url = {https://doi.org/10.1063/1.476722},\n}"
};


func_reference_type xc_ref_Becke2000_4020 = {
  "A. D. Becke, J. Chem. Phys. 112, 4020 (2000)",
  "10.1063/1.480951",
  "@article{Becke2000_4020,\n  author = \"Becke, Axel D.\",\n  title = \"Simulation of delocalized exchange by local density functionals\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2000\",\n  volume = \"112\",\n  number = \"9\",\n  pages = \"4020-4026\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/112/9/10.1063/1.480951\",\n  doi = \"10.1063/1.480951\"\n}"
};


func_reference_type xc_ref_Becke2006_221101 = {
  "A. D. Becke and E. R. Johnson, J. Chem. Phys. 124, 221101 (2006)",
  "10.1063/1.2213970",
  "@article{Becke2006_221101,\n  author = \"A. D. Becke and E. R. Johnson\",\n  title = \"A simple effective potential for exchange\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2006\",\n  volume = \"124\",\n  number = \"22\",\n  pages = 221101,\n  doi = \"10.1063/1.2213970\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/124/22/10.1063/1.2213970\"\n}"
};


func_reference_type xc_ref_Berland2014_035412 = {
  "K. Berland and P. Hyldgaard, Phys. Rev. B 89, 035412 (2014)",
  "10.1103/PhysRevB.89.035412",
  "@article{Berland2014_035412,\n  title = {Exchange functional that tests the robustness of the plasmon description of the van der Waals density functional},\n  author = {K. Berland and P. Hyldgaard},\n  journal = {Phys. Rev. B},\n  volume = {89},\n  issue = {3},\n  pages = {035412},\n  year = {2014},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.89.035412},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.89.035412}\n}"
};


func_reference_type xc_ref_Bernard2012_204103 = {
  "Y. A. Bernard, Y. Shao, and A. I. Krylov, J. Chem. Phys. 136, 204103 (2012)",
  "10.1063/1.4714499",
  "@article{Bernard2012_204103,\n  author = {Yves A. Bernard and Yihan Shao and Anna I. Krylov},\n  title = {General formulation of spin-flip time-dependent density functional theory using non-collinear kernels: Theory, implementation, and benchmarks},\n  journal = {J. Chem. Phys.},\n  volume = {136},\n  number = {20},\n  pages = {204103},\n  year = {2012},\n  doi = {10.1063/1.4714499},\n  url = {http://doi.org/10.1063/1.4714499}\n}"
};


func_reference_type xc_ref_Besley2009_10350 = {
  "N. A. Besley, M. J. G. Peach, and D. J. Tozer, Phys. Chem. Chem. Phys. 11, 10350 (2009)",
  "10.1039/B912718F",
  "@Article{Besley2009_10350,\n  author = \"Besley, Nicholas A. and Peach, Michael J. G. and Tozer, David J.\",\n  title = \"Time-dependent density functional theory calculations of near-edge X-ray absorption fine structure with short-range corrected functionals\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2009\",\n  volume = \"11\",\n  issue = \"44\",\n  pages = \"10350-10358\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/B912718F\",\n  url = \"http://doi.org/10.1039/B912718F\",\n}"
};


func_reference_type xc_ref_Bilc2008_165107 = {
  "D. I. Bilc, R. Orlando, R. Shaltaf, G.-M. Rignanese, J. Iniguez, and P. Ghosez, Phys. Rev. B 77, 165107 (2008)",
  "10.1103/PhysRevB.77.165107",
  "@article{Bilc2008_165107,\n  title = {Hybrid exchange-correlation functional for accurate prediction of the electronic and structural properties of ferroelectric oxides},\n  author = {D. I. Bilc and R. Orlando and R. Shaltaf and G.-M. Rignanese and J. {\\'{I}\\~{n}iguez} and Ph. Ghosez},\n  journal = {Phys. Rev. B},\n  volume = {77},\n  issue = {16},\n  pages = {165107},\n  year = {2008},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.77.165107},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.77.165107}\n}"
};


func_reference_type xc_ref_Bircher2018_3184 = {
  "M. P. Bircher and U. Rothlisberger, J. Chem. Theory Comput. 14, 3184 (2018)",
  "10.1021/acs.jctc.8b00069",
  "@article{Bircher2018_3184,\n  author = {Bircher, Martin P. and Rothlisberger, Ursula},\n  title = {Plane-Wave Implementation and Performance of \\'{a}-la-Carte {C}oulomb-Attenuated Exchange-Correlation Functionals for Predicting Optical Excitation Energies in Some Notorious Cases},\n  journal = {J. Chem. Theory Comput.},\n  volume = {14},\n  number = {6},\n  pages = {3184-3195},\n  year = {2018},\n  doi = {10.1021/acs.jctc.8b00069},\n  url = {https://doi.org/10.1021/acs.jctc.8b00069}\n}"
};


func_reference_type xc_ref_Bloch1929_545 = {
  "F. Bloch, Z. Phys. 57, 545 (1929)",
  "10.1007/BF01340281",
  "@article{Bloch1929_545,\n  title = {Bemerkung zur Elektronentheorie des Ferromagnetismus und der elektrischen Leitf\\\"{a}higkeit},\n  author = {F. Bloch},\n  journal = {Z. Phys.},\n  volume = {57},\n  number = {7-8},\n  pages = {545},\n  year = {1929},\n  issn = {0044-3328},\n  publisher = {Springer-Verlag},\n  language = {German},\n  doi = {10.1007/BF01340281},\n  url = {http://link.springer.com/article/10.1007\\%2FBF01340281}\n}"
};


func_reference_type xc_ref_Boese2000_1670 = {
  "A. D. Boese, N. L. Doltsinis, N. C. Handy, and M. Sprik, J. Chem. Phys. 112, 1670 (2000)",
  "10.1063/1.480732",
  "@article{Boese2000_1670,\n  title = \"New generalized gradient approximation functionals\",\n  author = \"A. D. Boese and N. L. Doltsinis and N. C. Handy and M. Sprik\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2000\",\n  volume = \"112\",\n  number = \"4\",\n  pages = \"1670\",\n  doi = \"10.1063/1.480732\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/112/4/10.1063/1.480732\"\n}"
};


func_reference_type xc_ref_Boese2001_5497 = {
  "A. D. Boese and N. C. Handy, J. Chem. Phys. 114, 5497 (2001)",
  "10.1063/1.1347371",
  "@article{Boese2001_5497,\n  title = \"A new parametrization of exchange-correlation generalized gradient approximation functionals\",\n  author = \"A. D. Boese and N. C. Handy\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2001\",\n  volume = \"114\",\n  number = \"13\",\n  pages = \"5497\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/114/13/10.1063/1.1347371\",\n  doi = \"10.1063/1.1347371\"\n}"
};


func_reference_type xc_ref_Boese2002_9559 = {
  "A. D. Boese and N. C. Handy, J. Chem. Phys. 116, 9559 (2002)",
  "10.1063/1.1476309",
  "@article{Boese2002_9559,\n  title = \"New exchange-correlation density functionals: The role of the kinetic-energy density\",\n  author = \"A. D. Boese and N. C. Handy\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2002\",\n  volume = \"116\",\n  number = \"22\",\n  pages = \"9559\",\n  doi = \"10.1063/1.1476309\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/116/22/10.1063/1.1476309\"\n}"
};


func_reference_type xc_ref_Boese2003_5965 = {
  "A. D. Boese, A. Chandra, J. M. L. Martin, and D. Marx, J. Chem. Phys. 119, 5965 (2003)",
  "10.1063/1.1599338",
  "@article{Boese2003_5965,\n  title = \"From ab initio quantum chemistry to molecular dynamics: The delicate case of hydrogen bonding in ammonia\",\n  author = \"A. D. Boese and A. Chandra and J. M. L. Martin and D. Marx\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2003\",\n  volume = \"119\",\n  number = \"12\",\n  pages = \"5965\",\n  doi = \"10.1063/1.1599338\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/119/12/10.1063/1.1599338\"\n}"
};


func_reference_type xc_ref_Boese2004_3405 = {
  "A. D. Boese and J. M. L. Martin, J. Chem. Phys. 121, 3405 (2004)",
  "10.1063/1.1774975",
  "@article{Boese2004_3405,\n  title = \"Development of density functionals for thermochemical kinetics\",\n  author = \"A. D. Boese and J. M. L. Martin\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2004\",\n  volume = \"121\",\n  number = \"8\",\n  pages = \"3405\",\n  doi = \"10.1063/1.1774975\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/121/8/10.1063/1.1774975\"\n}"
};


func_reference_type xc_ref_Brandenburg2016_115144 = {
  "J. G. Brandenburg, J. E. Bates, J. Sun, and J. P. Perdew, Phys. Rev. B 94, 115144 (2016)",
  "10.1103/PhysRevB.94.115144",
  "@article{Brandenburg2016_115144,\n  title = {Benchmark tests of a strongly constrained semilocal functional with a long-range dispersion correction},\n  author = {Brandenburg, J. G. and Bates, J. E. and Sun, J. and Perdew, J. P.},\n  journal = {Phys. Rev. B},\n  volume = {94},\n  issue = {11},\n  pages = {115144},\n  year = {2016},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.94.115144},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.94.115144}\n}"
};


func_reference_type xc_ref_Bremond2012_1184 = {
  "E. Bremond, D. Pilard, I. Ciofini, H. Chermette, C. Adamo, and P. Cortona, Theor. Chem. Acc. 131, 1184 (2012)",
  "10.1007/s00214-012-1184-0",
  "@article{Bremond2012_1184,\n  title = {Generalized gradient exchange functionals based on the gradient-regulated connection: a new member of the TCA family},\n  author = {E. Br\\'{e}mond and D. Pilard and I. Ciofini and H. Chermette and C. Adamo and P. Cortona},\n  journal = {Theor. Chem. Acc.},\n  volume = {131},\n  number = {3},\n  pages = {1184},\n  year = {2012},\n  issn = {1432-881X},\n  doi = {10.1007/s00214-012-1184-0},\n  url = {http://link.springer.com/article/10.1007\\%2Fs00214-012-1184-0},\n  publisher = {Springer-Verlag},\n  language = {English}\n}"
};


func_reference_type xc_ref_Bremond2016_1059 = {
  "Eric Bremond, I. Ciofini, and C. Adamo, Mol. Phys. 114, 1059 (2016)",
  "10.1080/00268976.2015.1132788",
  "@article{Bremond2016_1059,\n  author = {\\'{E}ric Br\\'{e}mond and Ilaria Ciofini and Carlo Adamo},\n  title = {Gradient-regulated connection-based correction for the PBE exchange: the PBEtrans model},\n  journal = {Mol. Phys.},\n  volume = {114},\n  number = {7-8},\n  pages = {1059-1065},\n  year = {2016},\n  doi = {10.1080/00268976.2015.1132788},\n  url = {http://doi.org/10.1080/00268976.2015.1132788}\n}"
};


func_reference_type xc_ref_Bremond2016_244102 = {
  "Eric Bremond, J. Chem. Phys. 145, 244102 (2016)",
  "10.1063/1.4972815",
  "@article{Bremond2016_244102,\n  author = {\\'{E}ric Br\\'{e}mond},\n  title = {A power series revisit of the PBE exchange density-functional approximation: The PBEpow model},\n  journal = {J. Chem. Phys.},\n  volume = {145},\n  number = {24},\n  pages = {244102},\n  year = {2016},\n  doi = {10.1063/1.4972815},\n  url = {http://doi.org/10.1063/1.4972815}\n}"
};


func_reference_type xc_ref_Brown2021_2004 = {
  "K. Brown, Y. Maimaiti, K. Trepte, T. Bligaard, and J. Voss, J. Comput. Chem. 42, 2004 (2021)",
  "10.1002/jcc.26732",
  "@article{Brown2021_2004,\n  author = {Brown, Kristopher and Maimaiti, Yasheng and Trepte, Kai and Bligaard, Thomas and Voss, Johannes},\n  title = {{MCML}: Combining physical constraints with experimental data for a multi-purpose meta-generalized gradient approximation},\n  journal = {J. Comput. Chem.},\n  volume = {42},\n  number = {28},\n  pages = {2004--2013},\n  year = {2021},\n  doi = {10.1002/jcc.26732},\n  url = {https://onlinelibrary.wiley.com/doi/abs/10.1002/jcc.26732}\n}"
};


func_reference_type xc_ref_Brual1978_1177 = {
  "G. B. Jr. and S. M. Rothstein, J. Chem. Phys. 69, 1177 (1978)",
  "10.1063/1.436705",
  "@article{Brual1978_1177,\n  author = {Gregorio Brual Jr. and Stuart M. Rothstein},\n  title = {Rare gas interactions using an improved statistical method},\n  journal = {J. Chem. Phys.},\n  volume = {69},\n  number = {3},\n  pages = {1177-1183},\n  year = {1978},\n  doi = {10.1063/1.436705},\n  url = {http://aip.scitation.org/doi/abs/10.1063/1.436705},\n}"
};


func_reference_type xc_ref_Burke2014_4834 = {
  "K. Burke, A. Cancio, T. Gould, and S. Pittalis, ArXiv e-prints (2014), arXiv:1409.4834 [cond-mat.mtrl-sci]",
  "",
  "@article{Burke2014_4834,\n  author = {Burke, K. and Cancio, A. and Gould, T. and Pittalis, S.},\n  title = \"{Atomic correlation energies and the generalized gradient approximation}\",\n  journal = {ArXiv e-prints},\n  archivePrefix = \"arXiv\",\n  eprint = {1409.4834},\n  primaryClass = \"cond-mat.mtrl-sci\",\n  year = 2014,\n  month = sep\n}"
};


func_reference_type xc_ref_Caldeira2019_62 = {
  "M. T. Caldeira and R. Custodio, J. Mol. Model. 25, 62 (2019)",
  "10.1007/s00894-019-3952-4",
  "@article{Caldeira2019_62,\n  author = \"Caldeira, Mariana Toretti and Custodio, Rog\\'{e}rio\",\n  title = \"Partial combination of composite strategy and the B3LYP functional for the calculation of enthalpies of formation\",\n  journal = \"J. Mol. Model.\",\n  year = \"2019\",\n  month = \"Feb\",\n  day = \"12\",\n  volume = \"25\",\n  number = \"3\",\n  pages = \"62\",\n  issn = \"0948-5023\",\n  doi = \"10.1007/s00894-019-3952-4\",\n  url = \"https://doi.org/10.1007/s00894-019-3952-4\"\n}"
};


func_reference_type xc_ref_Campi1978_263 = {
  "X. Campi and A. Bouyssy, Phys. Lett. B 73, 263 (1978)",
  "10.1016/0370-2693(78)90509-9",
  "@article{Campi1978_263,\n  author = \"X. Campi and A. Bouyssy\",\n  title = \"A simple approximation for the nuclear density matrix\",\n  journal = \"Phys. Lett. B\",\n  volume = \"73\",\n  number = \"3\",\n  pages = \"263 - 266\",\n  year = \"1978\",\n  issn = \"0370-2693\",\n  doi = \"10.1016/0370-2693(78)90509-9\",\n  url = \"http://www.sciencedirect.com/science/article/pii/0370269378905099\"\n}"
};


func_reference_type xc_ref_Campo2012_179 = {
  "J. M. del Campo, J. L. Gazquez, S. Trickey, and A. Vela, Chem. Phys. Lett. 543, 179 (2012)",
  "10.1016/j.cplett.2012.06.025",
  "@article{Campo2012_179,\n  author = \"Jorge M. del Campo and Jos\\'{e} L. G\\'{a}zquez and S.B. Trickey and Alberto Vela\",\n  title = \"A new meta-GGA exchange functional based on an improved constraint-based {GGA}\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"543\",\n  number = \"\",\n  pages = \"179 - 183\",\n  year = \"2012\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2012.06.025\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261412007117\"\n}"
};


func_reference_type xc_ref_Cancio2006_081202 = {
  "A. C. Cancio and M. Y. Chou, Phys. Rev. B 74, 081202 (2006)",
  "10.1103/PhysRevB.74.081202",
  "@article{Cancio2006_081202,\n  title = {Beyond the local approximation to exchange and correlation: The role of the Laplacian of the density in the energy density of Si},\n  author = {A. C. Cancio and M. Y. Chou},\n  journal = {Phys. Rev. B},\n  volume = {74},\n  issue = {8},\n  pages = {081202},\n  year = {2006},\n  month = {Aug},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.74.081202},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.74.081202}\n}"
};


func_reference_type xc_ref_Cancio2016_084107 = {
  "A. C. Cancio, D. Stewart, and A. Kuna, J. Chem. Phys. 144, 084107 (2016)",
  "10.1063/1.4942016",
  "@article{Cancio2016_084107,\n  author = {Cancio, Antonio C. and Stewart, Dane and Kuna, Aeryk},\n  title = {Visualization and analysis of the Kohn-Sham kinetic energy density and its orbital-free description in molecules},\n  journal = {J. Chem. Phys.},\n  volume = {144},\n  number = {8},\n  pages = {084107},\n  year = {2016},\n  doi = {10.1063/1.4942016},\n  url = {https://doi.org/10.1063/1.4942016}\n}"
};


func_reference_type xc_ref_Cancio2017_618 = {
  "A. C. Cancio and J. J. Redd, Mol. Phys. 115, 618 (2017)",
  "10.1080/00268976.2016.1246757",
  "@article{Cancio2017_618,\n  author = {Antonio C. Cancio and Jeremy J. Redd},\n  title = {Visualisation and orbital-free parametrisation of the large-Z scaling of the kinetic energy density of atoms},\n  journal = {Mol. Phys.},\n  volume = {115},\n  number = {5},\n  pages = {618-635},\n  year  = {2017},\n  publisher = {Taylor & Francis},\n  doi = {10.1080/00268976.2016.1246757},\n  url = {https://doi.org/10.1080/00268976.2016.1246757}\n}"
};


func_reference_type xc_ref_Cancio2018_084116 = {
  "A. Cancio, G. P. Chen, B. T. Krull, and K. Burke, J. Chem. Phys. 149, 084116 (2018)",
  "10.1063/1.5021597",
  "@article{Cancio2018_084116,\n  author = {Cancio, Antonio and Chen, Guo P. and Krull, Brandon T. and Burke, Kieron},\n  title = {Fitting a round peg into a round hole: Asymptotically correcting the generalized gradient approximation for correlation},\n  journal = {J. Chem. Phys.},\n  volume = {149},\n  number = {8},\n  pages = {084116},\n  year = {2018},\n  doi = {10.1063/1.5021597},\n  url = {https://doi.org/10.1063/1.5021597}\n}"
};


func_reference_type xc_ref_Carmona2015_054105 = {
  "J. Carmona-Espindola, J. L. Gazquez, A. Vela, and S. B. Trickey, J. Chem. Phys. 142, 054105 (2015)",
  "10.1063/1.4906606",
  "@article{Carmona2015_054105,\n  author = \"Carmona-Esp\\'{i}ndola, Javier and G\\'{a}zquez, Jos\\'{e} L. and Vela, Alberto and Trickey, S. B.\",\n  title = {Generalized gradient approximation exchange energy functional with correct asymptotic behavior of the corresponding potential},\n  journal = {J. Chem. Phys.},\n  volume = {142},\n  number = {5},\n  pages = {054105},\n  year = {2015},\n  doi = {10.1063/1.4906606},\n  url = {https://doi.org/10.1063/1.4906606}\n}"
};


func_reference_type xc_ref_Carmona2016_120 = {
  "J. Carmona-Espindola, J. L. Gazquez, A. Vela, and S. B. Trickey, Theor. Chem. Acc. 135, 120 (2016)",
  "10.1007/s00214-016-1864-2",
  "@article{Carmona2016_120,\n  author = \"Carmona-Esp\\'{i}ndola, Javier and G\\'{a}zquez, Jos\\'{e} L. and Vela, Alberto and Trickey, S. B.\",\n  title = \"Global hybrid exchange energy functional with correct asymptotic behavior of the corresponding potential\",\n  journal = \"Theor. Chem. Acc.\",\n  year = \"2016\",\n  volume = \"135\",\n  number = \"5\",\n  pages = \"120\",\n  issn = \"1432-2234\",\n  doi = \"10.1007/s00214-016-1864-2\",\n  url = \"http://doi.org/10.1007/s00214-016-1864-2\"\n}"
};


func_reference_type xc_ref_Carmona2019_303 = {
  "J. Carmona-Espindola, J. L. Gazquez, A. Vela, and S. B. Trickey, J. Chem. Theory Comput. 15, 303 (2019)",
  "10.1021/acs.jctc.8b00998",
  "@article{Carmona2019_303,\n  author = {Carmona-Esp\\'{i}ndola, Javier and G\\'{a}zquez, Jos\\'{e} L. and Vela, Alberto and Trickey, S. B.},\n  title = {Generalized Gradient Approximation Exchange Energy Functional with Near-Best Semilocal Performance},\n  journal = {J. Chem. Theory Comput.},\n  volume = {15},\n  number = {1},\n  pages = {303-310},\n  year = {2019},\n  doi = {10.1021/acs.jctc.8b00998},\n  url = {https://doi.org/10.1021/acs.jctc.8b00998}\n}"
};


func_reference_type xc_ref_Casula2006_245427 = {
  "M. Casula, S. Sorella, and G. Senatore, Phys. Rev. B 74, 245427 (2006)",
  "10.1103/PhysRevB.74.245427",
  "@article{Casula2006_245427,\n  title = {Ground state properties of the one-dimensional Coulomb gas using the lattice regularized diffusion Monte Carlo method},\n  author = {M. Casula and S. Sorella and G. Senatore},\n  journal = {Phys. Rev. B},\n  volume = {74},\n  issue = {24},\n  pages = {245427},\n  year = {2006},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.74.245427},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.74.245427}\n}"
};


func_reference_type xc_ref_Chachiyo2016_021101 = {
  "T. Chachiyo, J. Chem. Phys. 145, 021101 (2016)",
  "10.1063/1.4958669",
  "@article{Chachiyo2016_021101,\n  author = \"Chachiyo, Teepanis\",\n  title = \"Simple and accurate uniform electron gas correlation energy for the full range of densities\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2016\",\n  volume = \"145\",\n  number = \"2\",\n  pages = \"021101\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/145/2/10.1063/1.4958669\",\n  doi = \"10.1063/1.4958669\"\n}"
};


func_reference_type xc_ref_Chachiyo2020_112669 = {
  "T. Chachiyo and H. Chachiyo, Comput. Theor. Chem. 1172, 112669 (2020)",
  "10.1016/j.comptc.2019.112669",
  "@article{Chachiyo2020_112669,\n  author = \"Teepanis Chachiyo and Hathaithip Chachiyo\",\n  title = \"Understanding electron correlation energy through density functional theory\",\n  journal = \"Comput. Theor. Chem.\",\n  volume = \"1172\",\n  pages = \"112669\",\n  year = \"2020\",\n  issn = \"2210-271X\",\n  doi = \"10.1016/j.comptc.2019.112669\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S2210271X19303652\"\n}"
};


func_reference_type xc_ref_Chachiyo2020_3485 = {
  "T. Chachiyo and H. Chachiyo, Molecules 25, 3485 (2020)",
  "10.3390/molecules25153485",
  "@article{Chachiyo2020_3485,\n  author = {Chachiyo, Teepanis and Chachiyo, Hathaithip},\n  doi = {10.3390/molecules25153485},\n  issn = {1420-3049},\n  journal = {Molecules},\n  pages = {3485},\n  title = {{Simple and Accurate Exchange Energy for Density Functional Theory}},\n  url = {https://www.mdpi.com/1420-3049/25/15/3485},\n  volume = {25},\n  year = {2020}\n}"
};


func_reference_type xc_ref_Chai2008_084106 = {
  "J.-D. Chai and M. Head-Gordon, J. Chem. Phys. 128, 084106 (2008)",
  "10.1063/1.2834918",
  "@article{Chai2008_084106,\n  author = \"Chai, Jeng-Da and Head-Gordon, Martin\",\n  title = \"Systematic optimization of long-range corrected hybrid density functionals\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2008\",\n  volume = \"128\",\n  number = \"8\",\n  eid = 084106,\n  pages = \"084106\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/128/8/10.1063/1.2834918\",\n  doi = \"10.1063/1.2834918\"\n}"
};


func_reference_type xc_ref_Chai2008_6615 = {
  "J.-D. Chai and M. Head-Gordon, Phys. Chem. Chem. Phys. 10, 6615 (2008)",
  "10.1039/B810189B",
  "@article{Chai2008_6615,\n  author = \"Chai, Jeng-Da and Head-Gordon, Martin\",\n  title = \"Long-range corrected hybrid density functionals with damped atom-atom dispersion corrections\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  publisher = \"The Royal Society of Chemistry\",\n  year = \"2008\",\n  volume = \"10\",\n  issue = \"44\",\n  pages = \"6615-6620\",\n  doi = \"10.1039/B810189B\",\n  url = \"http://doi.org/10.1039/B810189B\"\n}"
};


func_reference_type xc_ref_Chen2018_073803 = {
  "W. Chen, G. Miceli, G.-M. Rignanese, and A. Pasquarello, Phys. Rev. Mater. 2, 073803 (2018)",
  "10.1103/PhysRevMaterials.2.073803",
  "@article{Chen2018_073803,\n  title = {Nonempirical dielectric-dependent hybrid functional with range separation for semiconductors and insulators},\n  author = {Chen, Wei and Miceli, Giacomo and Rignanese, Gian-Marco and Pasquarello, Alfredo},\n  journal = {Phys. Rev. Mater.},\n  volume = {2},\n  issue = {7},\n  pages = {073803},\n  year = {2018},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevMaterials.2.073803},\n  url = {https://link.aps.org/doi/10.1103/PhysRevMaterials.2.073803}\n}"
};


func_reference_type xc_ref_Chiodo2012_126402 = {
  "L. Chiodo, L. A. Constantin, E. Fabiano, and F. Della Sala, Phys. Rev. Lett. 108, 126402 (2012)",
  "10.1103/PhysRevLett.108.126402",
  "@article{Chiodo2012_126402,\n  title = {Nonuniform Scaling Applied to Surface Energies of Transition Metals},\n  author = {L. Chiodo and L. A. Constantin and E. Fabiano and Della Sala, F.},\n  journal = {Phys. Rev. Lett.},\n  volume = {108},\n  issue = {12},\n  pages = {126402},\n  year = {2012},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.108.126402},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.108.126402}\n}"
};


func_reference_type xc_ref_Cohen2000_160 = {
  "A. J. Cohen and N. C. Handy, Chem. Phys. Lett. 316, 160 (2000)",
  "10.1016/S0009-2614(99)01273-7",
  "@article{Cohen2000_160,\n  title = \"Assessment of exchange correlation functionals\",\n  author = \"A. J. Cohen and N. C. Handy\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"316\",\n  number = \"1--2\",\n  pages = \"160\",\n  year = \"2000\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/S0009-2614(99)01273-7\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261499012737\",\n}"
};


func_reference_type xc_ref_Cohen2001_607 = {
  "A. J. Cohen and N. C. Handy, Mol. Phys. 99, 607 (2001)",
  "10.1080/00268970010023435",
  "@article{Cohen2001_607,\n  title = {Dynamic correlation},\n  author = {A. J. Cohen and N. C. Handy},\n  journal = {Mol. Phys.},\n  volume = {99},\n  number = {7},\n  pages = {607},\n  year = {2001},\n  doi = {10.1080/00268970010023435},\n  url = {http://www.tandfonline.com/doi/abs/10.1080/00268970010023435},\n}"
};


func_reference_type xc_ref_Cohen2007_191109 = {
  "A. J. Cohen, P. Mori-Sanchez, and W. Yang, J. Chem. Phys. 126, 191109 (2007)",
  "10.1063/1.2741248",
  "@article{Cohen2007_191109,\n  author = {Cohen, Aron J. and Mori-S\\'{a}nchez, Paula and Yang, Weitao},\n  title = {Development of exchange-correlation functionals with minimal many-electron self-interaction error},\n  journal = {J. Chem. Phys.},\n  volume = {126},\n  number = {19},\n  pages = {191109},\n  year = {2007},\n  doi = {10.1063/1.2741248},\n  url = {https://doi.org/10.1063/1.2741248},\n}"
};


func_reference_type xc_ref_Colle1975_329 = {
  "R. Colle and O. Salvetti, Theor. Chim. Acta 37, 329 (1975)",
  "10.1007/BF01028401",
  "@article{Colle1975_329,\n  title = {Approximate calculation of the correlation energy for the closed shells},\n  author = {R. Colle and O. Salvetti},\n  journal = {Theor. Chim. Acta},\n  volume = {37},\n  number = {4},\n  pages = {329},\n  year = {1975},\n  issn = {0040-5744},\n  publisher = {Springer-Verlag},\n  language = {English},\n  doi = {10.1007/BF01028401},\n  url = {http://link.springer.com/article/10.1007\\%2FBF01028401}\n}"
};


func_reference_type xc_ref_Constantin2009_035125 = {
  "L. A. Constantin, A. Ruzsinszky, and J. P. Perdew, Phys. Rev. B 80, 035125 (2009)",
  "10.1103/PhysRevB.80.035125",
  "@article{Constantin2009_035125,\n  title = {Exchange-correlation energy functional based on the Airy-gas reference system},\n  author = {L. A. Constantin and A. Ruzsinszky and J. P. Perdew},\n  journal = {Phys. Rev. B},\n  volume = {80},\n  issue = {3},\n  pages = {035125},\n  year = {2009},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.80.035125},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.80.035125}\n}"
};


func_reference_type xc_ref_Constantin2011_186406 = {
  "L. A. Constantin, E. Fabiano, S. Laricchia, and F. Della Sala, Phys. Rev. Lett. 106, 186406 (2011)",
  "10.1103/PhysRevLett.106.186406",
  "@article{Constantin2011_186406,\n  title = {Semiclassical Neutral Atom as a Reference System in Density Functional Theory},\n  author = {L. A. Constantin and E. Fabiano and S. Laricchia and Della Sala, F.},\n  journal = {Phys. Rev. Lett.},\n  volume = {106},\n  issue = {18},\n  pages = {186406},\n  year = {2011},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.106.186406},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.106.186406}\n}"
};


func_reference_type xc_ref_Constantin2011_233103 = {
  "L. A. Constantin, E. Fabiano, and F. Della Sala, Phys. Rev. B 84, 233103 (2011)",
  "10.1103/PhysRevB.84.233103",
  "@article{Constantin2011_233103,\n  title = {Improving atomization energies of molecules and solids with a spin-dependent gradient correction from one-electron density analysis},\n  author = {L. A. Constantin and E. Fabiano and Della Sala, F.},\n  journal = {Phys. Rev. B},\n  volume = {84},\n  issue = {23},\n  pages = {233103},\n  year = {2011},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.84.233103},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.84.233103}\n}"
};


func_reference_type xc_ref_Constantin2012_035130 = {
  "L. A. Constantin, E. Fabiano, and F. Della Sala, Phys. Rev. B 86, 035130 (2012)",
  "10.1103/PhysRevB.86.035130",
  "@article{Constantin2012_035130,\n  title = {Semilocal dynamical correlation with increased localization},\n  author = {Constantin, Lucian A. and Fabiano, Eduardo and Della Sala, Fabio},\n  journal = {Phys. Rev. B},\n  volume = {86},\n  issue = {3},\n  pages = {035130},\n  year = {2012},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.86.035130},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.86.035130}\n}"
};


func_reference_type xc_ref_Constantin2012_194105 = {
  "L. A. Constantin, E. Fabiano, and F. D. Sala, J. Chem. Phys. 137, 194105 (2012)",
  "10.1063/1.4766324",
  "@article{Constantin2012_194105,\n  author = {Lucian A. Constantin and Eduardo Fabiano and Fabio Della Sala},\n  title = {Spin-dependent gradient correction for more accurate atomization energies of molecules},\n  journal = {J. Chem. Phys.},\n  volume = {137},\n  number = {19},\n  pages = {194105},\n  year = {2012},\n  doi = {10.1063/1.4766324},\n  url = {http://doi.org/10.1063/1.4766324},\n}"
};


func_reference_type xc_ref_Constantin2013_2256 = {
  "L. A. Constantin, E. Fabiano, and F. Della Sala, J. Chem. Theory Comput. 9, 2256 (2013)",
  "10.1021/ct400148r",
  "@article{Constantin2013_2256,\n  title = {Meta-GGA Exchange-Correlation Functional with a Balanced Treatment of Nonlocality},\n  author = {L. A. Constantin and E. Fabiano and Della Sala, F.},\n  journal = {J. Chem. Theory Comput.},\n  volume = {9},\n  number = {5},\n  pages = {2256},\n  year = {2013},\n  doi = {10.1021/ct400148r},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct400148r}\n}"
};


func_reference_type xc_ref_Constantin2016_045126 = {
  "L. A. Constantin, A. Terentjevs, F. Della Sala, P. Cortona, and E. Fabiano, Phys. Rev. B 93, 045126 (2016)",
  "10.1103/PhysRevB.93.045126",
  "@article{Constantin2016_045126,\n  title = {Semiclassical atom theory applied to solid-state physics},\n  author = {Constantin, Lucian A. and Terentjevs, Aleksandrs and Della Sala, Fabio and Cortona, Pietro and Fabiano, Eduardo},\n  journal = {Phys. Rev. B},\n  volume = {93},\n  issue = {4},\n  pages = {045126},\n  year = {2016},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.93.045126},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.93.045126}\n}"
};


func_reference_type xc_ref_Constantin2016_115127 = {
  "L. A. Constantin, E. Fabiano, J. M. Pitarke, and F. Della Sala, Phys. Rev. B 93, 115127 (2016)",
  "10.1103/PhysRevB.93.115127",
  "@article{Constantin2016_115127,\n  title = {Semilocal density functional theory with correct surface asymptotics},\n  author = {Constantin, Lucian A. and Fabiano, Eduardo and Pitarke, J. M. and Della Sala, Fabio},\n  journal = {Phys. Rev. B},\n  volume = {93},\n  issue = {11},\n  pages = {115127},\n  year = {2016},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.93.115127},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.93.115127}\n}"
};


func_reference_type xc_ref_Constantin2017_115153 = {
  "L. A. Constantin, E. Fabiano, S. Smiga, and F. Della Sala, Phys. Rev. B 95, 115153 (2017)",
  "10.1103/PhysRevB.95.115153",
  "@article{Constantin2017_115153,\n  title = {Jellium-with-gap model applied to semilocal kinetic functionals},\n  author = {Constantin, Lucian A. and Fabiano, Eduardo and \\'{S}miga, Szymon and Della Sala, Fabio},\n  journal = {Phys. Rev. B},\n  volume = {95},\n  issue = {11},\n  pages = {115153},\n  year = {2017},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.95.115153},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.95.115153}\n}"
};


func_reference_type xc_ref_Constantin2018_4385 = {
  "L. A. Constantin, E. Fabiano, and F. Della Sala, J. Phys. Chem. Lett. 9, 4385 (2018)",
  "10.1021/acs.jpclett.8b01926",
  "@article{Constantin2018_4385,\n  author = {Constantin, Lucian A. and Fabiano, Eduardo and Della Sala, Fabio},\n  title = {Semilocal Pauli--Gaussian Kinetic Functionals for Orbital-Free Density Functional Theory Calculations of Solids},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {9},\n  number = {15},\n  pages = {4385-4390},\n  year = {2018},\n  doi = {10.1021/acs.jpclett.8b01926},\n  url = {https://doi.org/10.1021/acs.jpclett.8b01926}\n}"
};


func_reference_type xc_ref_Cooper2010_161104 = {
  "V. R. Cooper, Phys. Rev. B 81, 161104 (2010)",
  "10.1103/PhysRevB.81.161104",
  "@article{Cooper2010_161104,\n  title = {Van der Waals density functional: An appropriate exchange functional},\n  author = {V. R. Cooper},\n  journal = {Phys. Rev. B},\n  volume = {81},\n  issue = {16},\n  pages = {161104},\n  year = {2010},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.81.161104},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.81.161104}\n}"
};


func_reference_type xc_ref_Cortona2012_086101 = {
  "P. Cortona, J. Chem. Phys. 136, 086101 (2012)",
  "10.1063/1.3690462",
  "@article{Cortona2012_086101,\n  title = \"Note: Theoretical mixing coefficients for hybrid functionals\",\n  author = \"P. Cortona\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"136\",\n  number = \"8\",\n  pages = 086101,\n  doi = \"10.1063/1.3690462\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/136/8/10.1063/1.3690462\"\n}"
};


func_reference_type xc_ref_Csonka2010_3688 = {
  "G. I. Csonka, J. P. Perdew, and A. Ruzsinszky, J. Chem. Theory Comput. 6, 3688 (2010)",
  "10.1021/ct100488v",
  "@article{Csonka2010_3688,\n  title = {Global Hybrid Functionals: A Look at the Engine under the Hood},\n  author = {G. I. Csonka and J. P. Perdew and A. Ruzsinszky},\n  journal = {J. Chem. Theory Comput.},\n  volume = {6},\n  number = {12},\n  pages = {3688},\n  year = {2010},\n  doi = {10.1021/ct100488v},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct100488v}\n}"
};


func_reference_type xc_ref_Dahlke2005_15677 = {
  "E. E. Dahlke and D. G. Truhlar, J. Phys. Chem. B 109, 15677 (2005)",
  "10.1021/jp052436c",
  "@article{Dahlke2005_15677,\n  title = {Improved Density Functionals for Water},\n  author = {E. E. Dahlke and D. G. Truhlar},\n  journal = {J. Phys. Chem. B},\n  volume = {109},\n  number = {33},\n  pages = {15677},\n  year = {2005},\n  doi = {10.1021/jp052436c},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp052436c}\n}"
};


func_reference_type xc_ref_Day2006_094103 = {
  "P. N. Day, K. A. Nguyen, and R. Pachter, J. Chem. Phys. 125, 094103 (2006)",
  "10.1063/1.2338031",
  "@article{Day2006_094103,\n  author = {Day, Paul N. and Nguyen, Kiet A. and Pachter, Ruth},\n  title = {Calculation of two-photon absorption spectra of donor-$\\pi$-acceptor compounds in solution using quadratic response time-dependent density functional theory},\n  journal = {J. Chem. Phys.},\n  volume = {125},\n  number = {9},\n  pages = {094103},\n  year = {2006},\n  doi = {10.1063/1.2338031},\n  url = {https://doi.org/10.1063/1.2338031}\n}"
};


func_reference_type xc_ref_DePristo1987_1425 = {
  "A. E. DePristo and J. D. Kress, J. Chem. Phys. 86, 1425 (1987)",
  "10.1063/1.452230",
  "@article{DePristo1987_1425,\n  title = \"Rational function representation for accurate exchange energy functionals\",\n  author = \"A. E. DePristo and J. D. Kress\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1987\",\n  volume = \"86\",\n  number = \"3\",\n  pages = \"1425\",\n  doi = \"10.1063/1.452230\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/86/3/10.1063/1.452230\"\n}"
};


func_reference_type xc_ref_DePristo1987_438 = {
  "A. E. DePristo and J. D. Kress, Phys. Rev. A 35, 438 (1987)",
  "10.1103/PhysRevA.35.438",
  "@article{DePristo1987_438,\n  title = {Kinetic-energy functionals via Pad\\'{e} approximations},\n  author = {A. E. DePristo and J. D. Kress},\n  journal = {Phys. Rev. A},\n  volume = {35},\n  issue = {1},\n  pages = {438},\n  year = {1987},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.35.438},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.35.438}\n}"
};


func_reference_type xc_ref_Dirac1930_376 = {
  "P. A. M. Dirac, Math. Proc. Cambridge Philos. Soc. 26, 376 (1930)",
  "10.1017/S0305004100016108",
  "@article{Dirac1930_376,\n  title = {Note on Exchange Phenomena in the Thomas Atom},\n  author = {P. A. M. Dirac},\n  journal = {Math. Proc. Cambridge Philos. Soc.},\n  volume = {26},\n  issue = {03},\n  month = {7},\n  pages = {376},\n  year = {1930},\n  issn = {1469-8064},\n  doi = {10.1017/S0305004100016108},\n  url = {http://journals.cambridge.org/article_S0305004100016108}\n}"
};


func_reference_type xc_ref_Eich2014_224107 = {
  "F. G. Eich and M. Hellgren, J. Chem. Phys. 141, 224107 (2014)",
  "10.1063/1.4903273",
  "@article{Eich2014_224107,\n  author = {Eich,F. G. and Hellgren,Maria},\n  title = {Derivative discontinuity and exchange-correlation potential of meta-GGAs in density-functional theory},\n  journal = {J. Chem. Phys.},\n  volume = {141},\n  number = {22},\n  pages = {224107},\n  year = {2014},\n  doi = {10.1063/1.4903273},\n  url = {https://doi.org/10.1063/1.4903273},\n}"
};


func_reference_type xc_ref_Elliott2009_1485 = {
  "P. Elliott and K. Burke, Can. J. Chem. 87, 1485 (2009)",
  "10.1139/V09-095",
  "@article{Elliott2009_1485,\n  author = {P. Elliott and K. Burke},\n  title = {Non-empirical derivation of the parameter in the B88 exchange functional},\n  journal = {Can. J. Chem.},\n  volume = {87},\n  number = {10},\n  pages = {1485-1491},\n  year = {2009},\n  doi = {10.1139/V09-095},\n  url = {http://www.nrcresearchpress.com/doi/abs/10.1139/V09-095}\n}"
};


func_reference_type xc_ref_Engel1992_7 = {
  "E. Engel, J. A. Chevary, L. D. Macdonald, and S. H. Vosko, Z. Phys. D: At., Mol. Clusters 23, 7 (1992)",
  "10.1007/BF01436696",
  "@article{Engel1992_7,\n  author = \"Engel, E. and Chevary, J. A. and Macdonald, L. D. and Vosko, S. H.\",\n  title = \"Asymptotic properties of the exchange energy density and the exchange potential of finite systems: relevance for generalized gradient approximations\",\n  journal = \"Z. Phys. D: At., Mol. Clusters\",\n  year = \"1992\",\n  month = \"Mar\",\n  day = \"01\",\n  volume = \"23\",\n  number = \"1\",\n  pages = \"7--14\",\n  issn = \"1431-5866\",\n  doi = \"10.1007/BF01436696\",\n  url = \"https://doi.org/10.1007/BF01436696\"\n}"
};


func_reference_type xc_ref_Engel1993_13164 = {
  "E. Engel and S. H. Vosko, Phys. Rev. B 47, 13164 (1993)",
  "10.1103/PhysRevB.47.13164",
  "@article{Engel1993_13164,\n  title = {Exact exchange-only potentials and the virial relation as microscopic criteria for generalized gradient approximations},\n  author = {Engel, E. and Vosko, S. H.},\n  journal = {Phys. Rev. B},\n  volume = {47},\n  issue = {20},\n  pages = {13164--13174},\n  year = {1993},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.47.13164},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.47.13164}\n}"
};


func_reference_type xc_ref_Engel1995_2750 = {
  "E. Engel, S. Keller, A. F. Bonetti, H. Muller, and R. M. Dreizler, Phys. Rev. A 52, 2750 (1995)",
  "10.1103/PhysRevA.52.2750",
  "@article{Engel1995_2750,\n  title = {Local and nonlocal relativistic exchange-correlation energy functionals: Comparison to relativistic optimized-potential-model results},\n  author = {Engel, E. and Keller, S. and Bonetti, A. Facco and M\\\"{u}ller, H. and Dreizler, R. M.},\n  journal = {Phys. Rev. A},\n  volume = {52},\n  issue = {4},\n  pages = {2750--2764},\n  year = {1995},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.52.2750},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.52.2750}\n}"
};


func_reference_type xc_ref_Entwistle2016_205134 = {
  "M. T. Entwistle, M. J. P. Hodgson, J. Wetherell, B. Longstaff, J. D. Ramsden, and R. W. Godby, Phys. Rev. B 94, 205134 (2016)",
  "10.1103/PhysRevB.94.205134",
  "@article{Entwistle2016_205134,\n  title = {Local density approximations from finite systems},\n  author = {Entwistle, M. T. and Hodgson, M. J. P. and Wetherell, J. and Longstaff, B. and Ramsden, J. D. and Godby, R. W.},\n  journal = {Phys. Rev. B},\n  volume = {94},\n  issue = {20},\n  pages = {205134},\n  year = {2016},\n  month = {Nov},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.94.205134},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.94.205134}\n}"
};


func_reference_type xc_ref_Ernzerhof1998_3313 = {
  "M. Ernzerhof and J. P. Perdew, J. Chem. Phys. 109, 3313 (1998)",
  "10.1063/1.476928",
  "@article{Ernzerhof1998_3313,\n  title = \"Generalized gradient approximation to the angle- and system-averaged exchange hole\",\n  author = \"M. Ernzerhof and J. P. Perdew\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1998\",\n  volume = \"109\",\n  number = \"9\",\n  pages = \"3313\",\n  doi = \"10.1063/1.476928\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/109/9/10.1063/1.476928\"\n}"
};


func_reference_type xc_ref_Ernzerhof1999_5029 = {
  "M. Ernzerhof and G. E. Scuseria, J. Chem. Phys. 110, 5029 (1999)",
  "10.1063/1.478401",
  "@article{Ernzerhof1999_5029,\n  title = \"Assessment of the Perdew--Burke--Ernzerhof exchange-correlation functional\",\n  author = \"M. Ernzerhof and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1999\",\n  volume = \"110\",\n  number = \"11\",\n  pages = \"5029\",\n  doi = \"10.1063/1.478401\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/110/11/10.1063/1.478401\"\n}"
};


func_reference_type xc_ref_Ernzerhof1999_911 = {
  "M. Ernzerhof and G. E. Scuseria, J. Chem. Phys. 111, 911 (1999)",
  "10.1063/1.479374",
  "@article{Ernzerhof1999_911,\n  title = \"Kinetic energy density dependent approximations to the exchange energy\",\n  author = \"M. Ernzerhof and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1999\",\n  volume = \"111\",\n  number = \"3\",\n  pages = \"911\",\n  doi = \"10.1063/1.479374\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/111/3/10.1063/1.479374\"\n}"
};


func_reference_type xc_ref_Ernzerhof2000_59 = {
  "M. Ernzerhof, J. Mol. Struct.: THEOCHEM 501--502, 59 (2000)",
  "10.1016/S0166-1280(99)00414-5",
  "@article{Ernzerhof2000_59,\n  title = \"The role of the kinetic energy density in approximations to the exchange energy\",\n  author = \"M. Ernzerhof\",\n  journal = \"J. Mol. Struct.: THEOCHEM\",\n  volume = \"501--502\",\n  number = \"0\",\n  pages = \"59\",\n  year = \"2000\",\n  issn = \"0166-1280\",\n  doi = \"10.1016/S0166-1280(99)00414-5\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0166128099004145\"\n}"
};


func_reference_type xc_ref_EspinosaLeal2015_31463 = {
  "L. A. Espinosa Leal, A. Karpenko, M. A. Caro, and O. Lopez-Acevedo, Phys. Chem. Chem. Phys. 17, 31463 (2015)",
  "10.1039/C5CP01211B",
  "@Article{EspinosaLeal2015_31463,\n  author = {Espinosa Leal, L. A. and Karpenko, A. and Caro, M. A. and Lopez-Acevedo, O.},\n  title = {Optimizing a parametrized {Thomas--Fermi--Dirac--Weizs\\\"{a}cker} density functional for atoms},\n  journal = {Phys. Chem. Chem. Phys.},\n  year = {2015},\n  volume = {17},\n  issue = {47},\n  pages = {31463-31471},\n  publisher = {The Royal Society of Chemistry},\n  doi = {10.1039/C5CP01211B},\n  url = {http://dx.doi.org/10.1039/C5CP01211B}\n}"
};


func_reference_type xc_ref_Fabiano2010_113104 = {
  "E. Fabiano, L. A. Constantin, and F. Della Sala, Phys. Rev. B 82, 113104 (2010)",
  "10.1103/PhysRevB.82.113104",
  "@article{Fabiano2010_113104,\n  title = {Generalized gradient approximation bridging the rapidly and slowly varying density regimes: A PBE-like functional for hybrid interfaces},\n  author = {E. Fabiano and L. A. Constantin and Della Sala, F.},\n  journal = {Phys. Rev. B},\n  volume = {82},\n  issue = {11},\n  pages = {113104},\n  year = {2010},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.82.113104},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.82.113104}\n}"
};


func_reference_type xc_ref_Fabiano2013_673 = {
  "E. Fabiano, L. A. Constantin, and F. Della Sala, Int. J. Quantum Chem. 113, 673 (2013)",
  "10.1002/qua.24042",
  "@article {Fabiano2013_673,\n  author = {Fabiano, E. and Constantin, Lucian A. and Della Sala, F.},\n  title = {Testing the broad applicability of the PBEint GGA functional and its one-parameter hybrid form},\n  journal = {Int. J. Quantum Chem.},\n  volume = {113},\n  number = {5},\n  pages = {673--682},\n  year = {2013},\n  publisher = {Wiley Subscription Services, Inc., A Wiley Company},\n  issn = {1097-461X},\n  doi = {10.1002/qua.24042},\n  url = {http://doi.org/10.1002/qua.24042}\n}"
};


func_reference_type xc_ref_Fabiano2014_2016 = {
  "E. Fabiano, P. E. Trevisanutto, A. Terentjevs, and L. A. Constantin, J. Chem. Theory Comput. 10, 2016 (2014)",
  "10.1021/ct500073b",
  "@article{Fabiano2014_2016,\n  author = {Eduardo Fabiano and Paolo E. Trevisanutto and Aleksandrs Terentjevs and Lucian A. Constantin},\n  title = {Generalized Gradient Approximation Correlation Energy Functionals Based on the Uniform Electron Gas with Gap Model},\n  journal = {J. Chem. Theory Comput.},\n  volume = {10},\n  number = {5},\n  pages = {2016-2026},\n  year = {2014},\n  doi = {10.1021/ct500073b},\n  url = {http://doi.org/10.1021/ct500073b}\n}"
};


func_reference_type xc_ref_Fabiano2015_122 = {
  "E. Fabiano, L. A. Constantin, P. Cortona, and F. Della Sala, J. Chem. Theory Comput. 11, 122 (2015)",
  "10.1021/ct500902p",
  "@article{Fabiano2015_122,\n  author = {Fabiano, Eduardo and Constantin, Lucian A. and Cortona, Pietro and Della Sala, Fabio},\n  title = {Global Hybrids from the Semiclassical Atom Theory Satisfying the Local Density Linear Response},\n  journal = {J. Chem. Theory Comput.},\n  volume = {11},\n  number = {1},\n  pages = {122-131},\n  year = {2015},\n  doi = {10.1021/ct500902p},\n  url = {https://doi.org/10.1021/ct500902p},\n}"
};


func_reference_type xc_ref_Fermi1927_602 = {
  "E. Fermi, Rendiconti dell'Accademia Nazionale dei Lincei 6, 602 (1927)",
  "",
  "@article{Fermi1927_602,\n  title = \"Un metodo statistico per la determinazione di alcune propriet\\'{a} dell'atomo. ({Italian}) [{A} Statistical Method for the Determination of Some Atomic Properties]\",\n  author = \"E. Fermi\",\n  journal = {Rendiconti dell'Accademia Nazionale dei Lincei},\n  volume = \"6\",\n  number = \"6\",\n  pages = \"602\",\n  year = \"1927\",\n  CODEN = \"AANLAW\",\n  ISSN = \"0001-4435\",\n  bibsource = \"http://www.math.utah.edu/pub/bibnet/authors/f/fermi-enrico.bib\",\n}"
};


func_reference_type xc_ref_Filatov1997_603 = {
  "M. Filatov and W. Thiel, Int. J. Quantum Chem. 62, 603 (1997)",
  "10.1002/(SICI)1097-461X(1997)62:6<603::AID-QUA4>3.0.CO;2-#",
  "@article{Filatov1997_603,\n  title = {A nonlocal correlation energy density functional from a Coulomb hole model},\n  author = {M. Filatov and W. Thiel},\n  journal = {Int. J. Quantum Chem.},\n  volume = {62},\n  number = {6},\n  pages = {603},\n  year = {1997},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  doi = {10.1002/(SICI)1097-461X(1997)62:6<603::AID-QUA4>3.0.CO;2-#},\n  url = {http://onlinelibrary.wiley.com/doi/10.1002/(SICI)1097-461X(1997)62:6<603::AID-QUA4>3.0.CO;2-\\%23}\n}"
};


func_reference_type xc_ref_Filatov1997_847 = {
  "M. Filatov and W. Thiel, Mol. Phys. 91, 847 (1997)",
  "10.1080/002689797170950",
  "@article{Filatov1997_847,\n  title = {A new gradient-corrected exchange-correlation density functional},\n  author = {M. Filatov and W. Thiel},\n  journal = {Mol. Phys.},\n  volume = {91},\n  number = {5},\n  pages = {847},\n  year = {1997},\n  doi = {10.1080/002689797170950},\n  url = {http://www.tandfonline.com/doi/abs/10.1080/002689797170950}\n}"
};


func_reference_type xc_ref_Filatov1998_189 = {
  "M. Filatov and W. Thiel, Phys. Rev. A 57, 189 (1998)",
  "10.1103/PhysRevA.57.189",
  "@article{Filatov1998_189,\n  title = {Exchange-correlation density functional beyond the gradient approximation},\n  author = {Filatov, Michael and Thiel, Walter},\n  journal = {Phys. Rev. A},\n  volume = {57},\n  issue = {1},\n  pages = {189--199},\n  year = {1998},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.57.189},\n  url = {https://link.aps.org/doi/10.1103/PhysRevA.57.189}\n}"
};


func_reference_type xc_ref_Finzel2017_40 = {
  "K. Finzel and A. I. Baranov, Int. J. Quantum Chem. 117, 40 (2017)",
  "10.1002/qua.25312",
  "@article{Finzel2017_40,\n  author = {Finzel, Kati and Baranov, Alexey I.},\n  title = {A simple model for the Slater exchange potential and its performance for solids},\n  journal = {Int. J. Quantum Chem.},\n  volume = {117},\n  number = {1},\n  pages = {40-47},\n  doi = {10.1002/qua.25312},\n  url = {https://onlinelibrary.wiley.com/doi/abs/10.1002/qua.25312},\n  year = {2017}\n}"
};


func_reference_type xc_ref_Fuentealba1995_31 = {
  "P. Fuentealba and O. Reyes, Chem. Phys. Lett. 232, 31 (1995)",
  "10.1016/0009-2614(94)01321-L",
  "@article{Fuentealba1995_31,\n  title = \"Further evidence of the conjoint correction to the local kinetic and exchange energy density functionals\",\n  author = \"P. Fuentealba and O. Reyes\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"232\",\n  number = \"1--2\",\n  pages = \"31\",\n  year = \"1995\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/0009-2614(94)01321-L\",\n  url = \"http://www.sciencedirect.com/science/article/pii/000926149401321L\",\n}"
};


func_reference_type xc_ref_Furness2018 = {
  "J. W. Furness and J. Sun, ArXiv e-prints (2018), arXiv:1805.11707v1 [physics.chem-ph]",
  "",
  "@article{Furness2018,\n  author = {Furness, J. W. and Sun, J.},\n  title = \"{Enhancing the efficiency of density functionals with a novel iso-orbital indicator}\",\n  journal = {ArXiv e-prints},\n  archivePrefix = \"arXiv\",\n  eprint = {1805.11707v1},\n  primaryClass = \"physics.chem-ph\",\n  year = 2018,\n  month = may,\n}"
};


func_reference_type xc_ref_Furness2019_041119 = {
  "J. W. Furness and J. Sun, Phys. Rev. B 99, 041119 (2019)",
  "10.1103/PhysRevB.99.041119",
  "@article{Furness2019_041119,\n  title = {Enhancing the efficiency of density functionals with an improved iso-orbital indicator},\n  author = {Furness, James W. and Sun, Jianwei},\n  journal = {Phys. Rev. B},\n  volume = {99},\n  issue = {4},\n  pages = {041119},\n  year = {2019},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.99.041119},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.99.041119}\n}"
};


func_reference_type xc_ref_Furness2020_8208 = {
  "J. W. Furness, A. D. Kaplan, J. Ning, J. P. Perdew, and J. Sun, J. Phys. Chem. Lett. 11, 8208 (2020)",
  "10.1021/acs.jpclett.0c02405",
  "@article{Furness2020_8208,\n  author = {Furness, James W. and Kaplan, Aaron D. and Ning, Jinliang and Perdew, John P. and Sun, Jianwei},\n  title = {Accurate and Numerically Efficient r2SCAN Meta-Generalized Gradient Approximation},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {11},\n  number = {19},\n  pages = {8208-8215},\n  year = {2020},\n  doi = {10.1021/acs.jpclett.0c02405},\n  url = {https://doi.org/10.1021/acs.jpclett.0c02405}\n}"
};


func_reference_type xc_ref_Furness2020_9248 = {
  "J. W. Furness, A. D. Kaplan, J. Ning, J. P. Perdew, and J. Sun, J. Phys. Chem. Lett. 11, 9248 (2020)",
  "10.1021/acs.jpclett.0c03077",
  "@article{Furness2020_9248,\n  author = {Furness, James W. and Kaplan, Aaron D. and Ning, Jinliang and Perdew, John P. and Sun, Jianwei},\n  title = {Correction to \"Accurate and Numerically Efficient r2SCAN Meta-Generalized Gradient Approximation\"},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {11},\n  number = {21},\n  pages = {9248-9248},\n  year = {2020},\n  doi = {10.1021/acs.jpclett.0c03077},\n  url = {https://doi.org/10.1021/acs.jpclett.0c03077}\n}"
};


func_reference_type xc_ref_Furness2021 = {
  "J. W. Furness, A. D. Kaplan, J. Ning, J. P. Perdew, and J. Sun, (2021), arXiv:2110.00647 [cond-mat.mtrl-sci]",
  "",
  "@article{Furness2021,\n  title={Construction of meta-GGA functionals through restoration of exact constraint adherence to regularized SCAN functionals},\n  author={James W. Furness and Aaron D. Kaplan and Jinliang Ning and John P. Perdew and Jianwei Sun},\n  year={2021},\n  eprint={2110.00647},\n  archivePrefix={arXiv},\n  primaryClass={cond-mat.mtrl-sci}\n}"
};


func_reference_type xc_ref_Gaiduk2011_012509 = {
  "A. P. Gaiduk and V. N. Staroverov, Phys. Rev. A 83, 012509 (2011)",
  "10.1103/PhysRevA.83.012509",
  "@article{Gaiduk2011_012509,\n  title = {Construction of integrable model Kohn-Sham potentials by analysis of the structure of functional derivatives},\n  author = {Gaiduk, Alex P. and Staroverov, Viktor N.},\n  journal = {Phys. Rev. A},\n  volume = {83},\n  issue = {1},\n  pages = {012509},\n  year = {2011},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.83.012509},\n  url = {https://link.aps.org/doi/10.1103/PhysRevA.83.012509}\n}"
};


func_reference_type xc_ref_Garza2018_3083 = {
  "A. J. Garza, A. T. Bell, and M. Head-Gordon, J. Chem. Theory Comput. 14, 3083 (2018)",
  "10.1021/acs.jctc.8b00288",
  "@article{Garza2018_3083,\n  author = {Garza, Alejandro J. and Bell, Alexis T. and Head-Gordon, Martin},\n  title = {Nonempirical Meta-Generalized Gradient Approximations for Modeling Chemisorption at Metal Surfaces},\n  journal = {J. Chem. Theory Comput.},\n  volume = {14},\n  number = {6},\n  pages = {3083-3090},\n  year = {2018},\n  doi = {10.1021/acs.jctc.8b00288},\n  url = {https://doi.org/10.1021/acs.jctc.8b00288}\n}"
};


func_reference_type xc_ref_Gazquez1982_1467 = {
  "J. L. Gazquez and J. Robles, J. Chem. Phys. 76, 1467 (1982)",
  "10.1063/1.443107",
  "@article{Gazquez1982_1467,\n  title = \"On the atomic kinetic energy functionals with full Weizsacker correction\",\n  author = \"J. L. G\\'{a}zquez and J. Robles\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1982\",\n  volume = \"76\",\n  number = \"3\",\n  pages = \"1467\",\n  doi = \"10.1063/1.443107\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/76/3/10.1063/1.443107\"\n}"
};


func_reference_type xc_ref_GellMann1957_364 = {
  "M. Gell-Mann and K. A. Brueckner, Phys. Rev. 106, 364 (1957)",
  "10.1103/PhysRev.106.364",
  "@article{GellMann1957_364,\n  title = {Correlation Energy of an Electron Gas at High Density},\n  author = {M. Gell-Mann and K. A. Brueckner},\n  journal = {Phys. Rev.},\n  volume = {106},\n  issue = {2},\n  pages = {364},\n  year = {1957},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRev.106.364},\n  url = {http://link.aps.org/doi/10.1103/PhysRev.106.364}\n}"
};


func_reference_type xc_ref_Ghiringhelli2008_073104 = {
  "L. M. Ghiringhelli and L. Delle Site, Phys. Rev. B 77, 073104 (2008)",
  "10.1103/PhysRevB.77.073104",
  "@article{Ghiringhelli2008_073104,\n  title = {Design of kinetic functionals for many-body electron systems: Combining analytical theory with Monte Carlo sampling of electronic configurations},\n  author = {Ghiringhelli, Luca M. and Delle Site, Luigi},\n  journal = {Phys. Rev. B},\n  volume = {77},\n  issue = {7},\n  pages = {073104},\n  year = {2008},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.77.073104},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.77.073104}\n}"
};


func_reference_type xc_ref_Ghiringhelli2010_014106 = {
  "L. M. Ghiringhelli, I. P. Hamilton, and L. D. Site, J. Chem. Phys. 132, 014106 (2010)",
  "10.1063/1.3280953",
  "@article{Ghiringhelli2010_014106,\n  author = {L. M. Ghiringhelli and I. P. Hamilton and L. Delle Site},\n  title = {Interacting electrons, spin statistics, and information theory},\n  journal = {J. Chem. Phys.},\n  volume = {132},\n  number = {1},\n  pages = {014106},\n  year = {2010},\n  doi = {10.1063/1.3280953},\n  url = {https://doi.org/10.1063/1.3280953},\n}"
};


func_reference_type xc_ref_Ghosh1985_3307 = {
  "S. K. Ghosh and R. G. Parr, J. Chem. Phys. 82, 3307 (1985)",
  "10.1063/1.448229",
  "@article{Ghosh1985_3307,\n  title = \"Density-determined orthonormal orbital approach to atomic energy functionals\",\n  author = \"S. K. Ghosh and R. G. Parr\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1985\",\n  volume = \"82\",\n  number = \"7\",\n  pages = \"3307\",\n  doi = \"10.1063/1.448229\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/82/7/10.1063/1.448229\"\n}"
};


func_reference_type xc_ref_Gilbert1999_511 = {
  "A. T. Gilbert and P. M. Gill, Chem. Phys. Lett. 312, 511 (1999)",
  "10.1016/S0009-2614(99)00836-2",
  "@article{Gilbert1999_511,\n  title = \"Decomposition of exchange-correlation energies\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"312\",\n  number = \"5-6\",\n  pages = \"511 - 521\",\n  year = \"1999\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/S0009-2614(99)00836-2\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261499008362\",\n  author = \"Andrew T.B. Gilbert and Peter M.W. Gill\"\n}"
};


func_reference_type xc_ref_Gill1996_1005 = {
  "P. M. W. Gill, R. D. Adamson, and J. A. Pople, Mol. Phys. 88, 1005 (1996)",
  "10.1080/00268979609484488",
  "@article{Gill1996_1005,\n  author = { Peter M. W. Gill and Ross D. Adamson and John A. Pople},\n  title = {Coulomb-attenuated exchange energy density functionals},\n  journal = {Mol. Phys.},\n  volume = {88},\n  number = {4},\n  pages = {1005-1009},\n  year  = {1996},\n  publisher = {Taylor & Francis},\n  doi = {10.1080/00268979609484488},\n  url = {https://doi.org/10.1080/00268979609484488},\n}"
};


func_reference_type xc_ref_Gill1996_433 = {
  "P. M. W. Gill, Mol. Phys. 89, 433 (1996)",
  "10.1080/002689796173813",
  "@article{Gill1996_433,\n  title = {A new gradient-corrected exchange functional},\n  author = {P. M. W. Gill},\n  journal = {Mol. Phys.},\n  volume = {89},\n  number = {2},\n  pages = {433},\n  year = {1996},\n  doi = {10.1080/002689796173813},\n  url = {http://www.tandfonline.com/doi/abs/10.1080/002689796173813}\n}"
};


func_reference_type xc_ref_Goedecker1996_1703 = {
  "S. Goedecker, M. Teter, and J. Hutter, Phys. Rev. B 54, 1703 (1996)",
  "10.1103/PhysRevB.54.1703",
  "@article{Goedecker1996_1703,\n  title = {Separable dual-space Gaussian pseudopotentials},\n  author = {S. Goedecker and M. Teter and J. Hutter},\n  journal = {Phys. Rev. B},\n  volume = {54},\n  issue = {3},\n  pages = {1703},\n  year = {1996},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.54.1703},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.54.1703}\n}"
};


func_reference_type xc_ref_Goerigk2010_107 = {
  "L. Goerigk and S. Grimme, J. Chem. Theory Comput. 6, 107 (2010)",
  "10.1021/ct900489g",
  "@article{Goerigk2010_107,\n  title = {A General Database for Main Group Thermochemistry, Kinetics, and Noncovalent Interactions -- Assessment of Common and Reparameterized (meta-)GGA Density Functionals},\n  author = {L. Goerigk and S. Grimme},\n  journal = {J. Chem. Theory Comput.},\n  volume = {6},\n  number = {1},\n  pages = {107},\n  year = {2010},\n  doi = {10.1021/ct900489g},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct900489g}\n}"
};


func_reference_type xc_ref_Golden1957_604 = {
  "S. Golden, Phys. Rev. 105, 604 (1957)",
  "10.1103/PhysRev.105.604",
  "@article{Golden1957_604,\n  title = {Statistical Theory of Many-Electron Systems. General Considerations Pertaining to the Thomas--Fermi Theory},\n  author = {S. Golden},\n  journal = {Phys. Rev.},\n  volume = {105},\n  issue = {2},\n  pages = {604},\n  year = {1957},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRev.105.604},\n  url = {http://link.aps.org/doi/10.1103/PhysRev.105.604}\n}"
};


func_reference_type xc_ref_Gombas1965_137 = {
  "P. Gombas, Fortschr. Phys. 13, 137 (1965)",
  "10.1002/prop.19650130402",
  "@article{Gombas1965_137,\n  author = {Gomb\\'{a}s, P.},\n  title = {Pseudopotentiale},\n  journal = {Fortschr. Phys.},\n  volume = {13},\n  number = {4},\n  pages = {137-156},\n  doi = {10.1002/prop.19650130402},\n  url = {https://onlinelibrary.wiley.com/doi/abs/10.1002/prop.19650130402},\n  year = {1965}\n}"
};


func_reference_type xc_ref_Gordon1972_3122 = {
  "R. G. Gordon and Y. S. Kim, J. Chem. Phys. 56, 3122 (1972)",
  "10.1063/1.1677649",
  "@article{Gordon1972_3122,\n  author = {Roy G. Gordon and Yung Sik Kim},\n  title = {Theory for the Forces between Closed-Shell Atoms and Molecules},\n  journal = {J. Chem. Phys.},\n  volume = {56},\n  number = {6},\n  pages = {3122-3133},\n  year = {1972},\n  doi = {10.1063/1.1677649},\n  url = {https://doi.org/10.1063/1.1677649}\n}"
};


func_reference_type xc_ref_Grimme2005_3067 = {
  "S. Grimme, J. Phys. Chem. A 109, 3067 (2005)",
  "10.1021/jp050036j",
  "@article{Grimme2005_3067,\n  author = {Grimme, Stefan},\n  title = {Accurate Calculation of the Heats of Formation for Large Main Group Compounds with Spin-Component Scaled MP2 Methods},\n  journal = {J. Phys. Chem. A},\n  volume = {109},\n  number = {13},\n  pages = {3067-3077},\n  year = {2005},\n  doi = {10.1021/jp050036j},\n  url = {https://doi.org/10.1021/jp050036j}\n}"
};


func_reference_type xc_ref_Grimme2006_034108 = {
  "S. Grimme, J. Chem. Phys. 124, 034108 (2006)",
  "10.1063/1.2148954",
  "@article{Grimme2006_034108,\n  author = {Grimme,Stefan},\n  title = {Semiempirical hybrid density functional with perturbative second-order correlation},\n  journal = {J. Chem. Phys.},\n  volume = {124},\n  number = {3},\n  pages = {034108},\n  year = {2006},\n  doi = {10.1063/1.2148954},\n  url = {https://doi.org/10.1063/1.2148954},\n}"
};


func_reference_type xc_ref_Grimme2006_1787 = {
  "S. Grimme, J. Comput. Chem. 27, 1787 (2006)",
  "10.1002/jcc.20495",
  "@article{Grimme2006_1787,\n  title = {Semiempirical GGA-type density functional constructed with a long-range dispersion correction},\n  author = {S. Grimme},\n  journal = {J. Comput. Chem.},\n  volume = {27},\n  number = {15},\n  pages = {1787},\n  year = {2006},\n  publisher = {Wiley Subscription Services, Inc., A Wiley Company},\n  issn = {1096-987X},\n  doi = {10.1002/jcc.20495},\n  url = {http://onlinelibrary.wiley.com/doi/10.1002/jcc.20495/abstract}\n}"
};


func_reference_type xc_ref_Grimme2010_154104 = {
  "S. Grimme, J. Antony, S. Ehrlich, and H. Krieg, J. Chem. Phys. 132, 154104 (2010)",
  "10.1063/1.3382344",
  "@article{Grimme2010_154104,\n  author = {Grimme, Stefan and Antony, Jens and Ehrlich, Stephan and Krieg, Helge},\n  title = {A consistent and accurate ab initio parametrization of density functional dispersion correction (DFT-D) for the 94 elements H-Pu},\n  journal = {J. Chem. Phys.},\n  volume = {132},\n  number = {15},\n  pages = {154104},\n  year = {2010},\n  doi = {10.1063/1.3382344},\n  url = {https://doi.org/10.1063/1.3382344}\n}"
};


func_reference_type xc_ref_Groth2017_135001 = {
  "S. Groth, T. Dornheim, T. Sjostrom, F. D. Malone, W. M. C. Foulkes, and M. Bonitz, Phys. Rev. Lett. 119, 135001 (2017)",
  "10.1103/PhysRevLett.119.135001",
  "@article{Groth2017_135001,\n  title = {Ab initio Exchange-Correlation Free Energy of the Uniform Electron Gas at Warm Dense Matter Conditions},\n  author = {Groth, Simon and Dornheim, Tobias and Sjostrom, Travis and Malone, Fionn D. and Foulkes, W. M. C. and Bonitz, Michael},\n  journal = {Phys. Rev. Lett.},\n  volume = {119},\n  issue = {13},\n  pages = {135001},\n  year = {2017},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.119.135001},\n  url = {https://link.aps.org/doi/10.1103/PhysRevLett.119.135001}\n}"
};


func_reference_type xc_ref_Gunnarsson1976_4274 = {
  "O. Gunnarsson and B. I. Lundqvist, Phys. Rev. B 13, 4274 (1976)",
  "10.1103/PhysRevB.13.4274",
  "@article{Gunnarsson1976_4274,\n  title = {Exchange and correlation in atoms, molecules, and solids by the spin-density-functional formalism},\n  author = {O. Gunnarsson and B. I. Lundqvist},\n  journal = {Phys. Rev. B},\n  volume = {13},\n  issue = {10},\n  pages = {4274},\n  year = {1976},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.13.4274},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.13.4274}\n}"
};


func_reference_type xc_ref_Haas2011_205117 = {
  "P. Haas, F. Tran, P. Blaha, and K. Schwarz, Phys. Rev. B 83, 205117 (2011)",
  "10.1103/PhysRevB.83.205117",
  "@article{Haas2011_205117,\n  title = {Construction of an optimal GGA functional for molecules and solids},\n  author = {P. Haas and F. Tran and P. Blaha and K. Schwarz},\n  journal = {Phys. Rev. B},\n  volume = {83},\n  issue = {20},\n  pages = {205117},\n  year = {2011},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.83.205117},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.83.205117}\n}"
};


func_reference_type xc_ref_Haiduke2018_184106 = {
  "R. L. A. Haiduke and R. J. Bartlett, J. Chem. Phys. 148, 184106 (2018)",
  "10.1063/1.5025723",
  "@article{Haiduke2018_184106,\n  author = {Haiduke, Roberto Luiz A. and Bartlett, Rodney J.},\n  title = {Non-empirical exchange-correlation parameterizations based on exact conditions from correlated orbital theory},\n  journal = {J. Chem. Phys.},\n  volume = {148},\n  number = {18},\n  pages = {184106},\n  year = {2018},\n  doi = {10.1063/1.5025723},\n  url = {https://doi.org/10.1063/1.5025723}\n}"
};


func_reference_type xc_ref_Hamada2014_121103 = {
  "I. Hamada, Phys. Rev. B 89, 121103 (2014)",
  "10.1103/PhysRevB.89.121103",
  "@article{Hamada2014_121103,\n  title = {van der Waals density functional made accurate},\n  author = {Hamada, Ikaturo},\n  journal = {Phys. Rev. B},\n  volume = {89},\n  issue = {12},\n  pages = {121103},\n  year = {2014},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.89.121103},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.89.121103}\n}"
};


func_reference_type xc_ref_Hammer1999_7413 = {
  "B. Hammer, L. B. Hansen, and J. K. Norskov, Phys. Rev. B 59, 7413 (1999)",
  "10.1103/PhysRevB.59.7413",
  "@article{Hammer1999_7413,\n  title = {Improved adsorption energetics within density-functional theory using revised Perdew--Burke--Ernzerhof functionals},\n  author = {B. Hammer and L. B. Hansen and J. K. N\\o{}rskov},\n  journal = {Phys. Rev. B},\n  volume = {59},\n  issue = {11},\n  pages = {7413},\n  year = {1999},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.59.7413},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.59.7413}\n}"
};


func_reference_type xc_ref_Hamprecht1998_6264 = {
  "F. A. Hamprecht, A. J. Cohen, D. J. Tozer, and N. C. Handy, J. Chem. Phys. 109, 6264 (1998)",
  "10.1063/1.477267",
  "@article{Hamprecht1998_6264,\n  title = \"Development and assessment of new exchange-correlation functionals\",\n  author = \"F. A. Hamprecht and A. J. Cohen and D. J. Tozer and N. C. Handy\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1998\",\n  volume = \"109\",\n  number = \"15\",\n  pages = \"6264\",\n  doi = \"10.1063/1.477267\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/109/15/10.1063/1.477267\"\n}"
};


func_reference_type xc_ref_Handy1998_707 = {
  "N. C. Handy and D. J. Tozer, Mol. Phys. 94, 707 (1998)",
  "10.1080/002689798167863",
  "@article{Handy1998_707,\n  title = {The development of new exchange-correlation functionals: 3},\n  author = {N. C. Handy and D. J. Tozer},\n  journal = {Mol. Phys.},\n  volume = {94},\n  number = {4},\n  pages = {707},\n  year = {1998},\n  doi = {10.1080/002689798167863},\n  url = {http://www.tandfonline.com/doi/abs/10.1080/002689798167863}\n}"
};


func_reference_type xc_ref_Handy2001_403 = {
  "N. C. Handy and A. J. Cohen, Mol. Phys. 99, 403 (2001)",
  "10.1080/00268970010018431",
  "@article{Handy2001_403,\n  title = {Left-right correlation energy},\n  author = {N. C. Handy and A. J. Cohen},\n  journal = {Mol. Phys.},\n  volume = {99},\n  number = {5},\n  pages = {403},\n  year = {2001},\n  doi = {10.1080/00268970010018431},\n  url = {http://www.tandfonline.com/doi/abs/10.1080/00268970010018431}\n}"
};


func_reference_type xc_ref_Handy2002_5411 = {
  "N. C. Handy and A. J. Cohen, J. Chem. Phys. 116, 5411 (2002)",
  "10.1063/1.1457432",
  "@article{Handy2002_5411,\n  author = {Nicholas C. Handy and Aron J. Cohen},\n  title = {A dynamical correlation functional},\n  journal = {J. Chem. Phys.},\n  volume = {116},\n  number = {13},\n  pages = {5411-5418},\n  year = {2002},\n  doi = {10.1063/1.1457432},\n  url = {http://doi.org/10.1063/1.1457432},\n}"
};


func_reference_type xc_ref_Hedin1971_2064 = {
  "L. Hedin and B. I. Lundqvist, J. Phys. C: Solid State Phys. 4, 2064 (1971)",
  "10.1088/0022-3719/4/14/022",
  "@article{Hedin1971_2064,\n  title = {Explicit local exchange-correlation potentials},\n  author = {L. Hedin and B. I. Lundqvist},\n  journal = {J. Phys. C: Solid State Phys.},\n  volume = {4},\n  number = {14},\n  pages = {2064},\n  year = {1971},\n  doi = {10.1088/0022-3719/4/14/022},\n  url = {http://stacks.iop.org/0022-3719/4/i=14/a=022}\n}"
};


func_reference_type xc_ref_Helbig2011_032503 = {
  "N. Helbig, J. I. Fuks, M. Casula, M. J. Verstraete, M. A. L. Marques, I. V. Tokatly, and A. Rubio, Phys. Rev. A 83, 032503 (2011)",
  "10.1103/PhysRevA.83.032503",
  "@article{Helbig2011_032503,\n  title = {Density functional theory beyond the linear regime: Validating an adiabatic local density approximation},\n  author = {N. Helbig and J. I. Fuks and M. Casula and M. J. Verstraete and M. A. L. Marques and I. V. Tokatly and A. Rubio},\n  journal = {Phys. Rev. A},\n  volume = {83},\n  issue = {3},\n  pages = {032503},\n  year = {2011},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.83.032503},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.83.032503}\n}"
};


func_reference_type xc_ref_Henderson2007_221103 = {
  "T. M. Henderson, A. F. Izmaylov, G. E. Scuseria, and A. Savin, J. Chem. Phys. 127, 221103 (2007)",
  "10.1063/1.2822021",
  "@article{Henderson2007_221103,\n  author = {Henderson, Thomas M. and Izmaylov, Artur F. and Scuseria, Gustavo E. and Savin, Andreas},\n  title = {The importance of middle-range Hartree--Fock-type exchange for hybrid density functionals},\n  journal = {J. Chem. Phys.},\n  volume = {127},\n  number = {22},\n  pages = {221103},\n  year = {2007},\n  doi = {10.1063/1.2822021},\n  url = {https://doi.org/10.1063/1.2822021},\n}"
};


func_reference_type xc_ref_Henderson2008_194105 = {
  "T. M. Henderson, B. G. Janesko, and G. E. Scuseria, J. Chem. Phys. 128, 194105 (2008)",
  "10.1063/1.2921797",
  "@article{Henderson2008_194105,\n  title = \"Generalized gradient approximation model exchange holes for range-separated hybrids\",\n  author = \"T. M. Henderson and B. G. Janesko and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2008\",\n  volume = \"128\",\n  number = \"19\",\n  pages = 194105,\n  doi = \"10.1063/1.2921797\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/128/19/10.1063/1.2921797\"\n}"
};


func_reference_type xc_ref_Henderson2009_044108 = {
  "T. M. Henderson, A. F. Izmaylov, G. Scalmani, and G. E. Scuseria, J. Chem. Phys. 131, 044108 (2009)",
  "10.1063/1.3185673",
  "@article{Henderson2009_044108,\n  title = \"Can short-range hybrids describe long-range-dependent properties?\",\n  author = \"T. M. Henderson and A. F. Izmaylov and G. Scalmani and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2009\",\n  volume = \"131\",\n  number = \"4\",\n  pages = 044108,\n  doi = \"10.1063/1.3185673\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/131/4/10.1063/1.3185673\",\n}"
};


func_reference_type xc_ref_Herman1969_807 = {
  "F. Herman, J. P. V. Dyke, and I. B. Ortenburger, Phys. Rev. Lett. 22, 807 (1969)",
  "10.1103/PhysRevLett.22.807",
  "@article{Herman1969_807,\n  title = {Improved Statistical Exchange Approximation for Inhomogeneous Many-Electron Systems},\n  author = {F. Herman and J. P. Van Dyke and I. B. Ortenburger},\n  journal = {Phys. Rev. Lett.},\n  volume = {22},\n  issue = {16},\n  pages = {807},\n  year = {1969},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.22.807},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.22.807}\n}"
};


func_reference_type xc_ref_Herman1969_827 = {
  "F. Herman, I. B. Ortenburger, and J. P. V. Dyke, Int. J. Quantum Chem. 4, 827 (1969)",
  "10.1002/qua.560040746",
  "@article{Herman1969_827,\n  title = {A method for improving the physical realism of first-principles band structure calculations},\n  author = {F. Herman and I. B. Ortenburger and J. P. Van Dyke},\n  journal = {Int. J. Quantum Chem.},\n  volume = {4},\n  number = {S3B},\n  pages = {827},\n  year = {1969},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  doi = {10.1002/qua.560040746},\n  url = {10.1002/qua.560040746}\n}"
};


func_reference_type xc_ref_Heyd2003_8207 = {
  "J. Heyd, G. E. Scuseria, and M. Ernzerhof, J. Chem. Phys. 118, 8207 (2003)",
  "10.1063/1.1564060",
  "@article{Heyd2003_8207,\n  title = \"Hybrid functionals based on a screened Coulomb potential\",\n  author = \"J. Heyd and G. E. Scuseria and M. Ernzerhof\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2003\",\n  volume = \"118\",\n  number = \"18\",\n  pages = \"8207\",\n  doi = \"10.1063/1.1564060\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/118/18/10.1063/1.1564060\"\n}"
};


func_reference_type xc_ref_Heyd2003_8207_err = {
  "J. Heyd, G. E. Scuseria, and M. Ernzerhof, J. Chem. Phys. 124, 219906 (2006)",
  "10.1063/1.2204597",
  "@article{Heyd2003_8207_err,\n  title = {Erratum: \"Hybrid functionals based on a screened Coulomb potential\" [J. Chem. Phys.118, 8207 (2003)]},\n  author = \"J. Heyd and G. E. Scuseria and M. Ernzerhof\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2006\",\n  volume = \"124\",\n  number = \"21\",\n  pages = 219906,\n  doi = \"10.1063/1.2204597\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/124/21/10.1063/1.2204597\",\n}"
};


func_reference_type xc_ref_Heyd2004_7274 = {
  "J. Heyd and G. E. Scuseria, J. Chem. Phys. 120, 7274 (2004)",
  "10.1063/1.1668634",
  "@article{Heyd2004_7274,\n  title = \"Assessment and validation of a screened Coulomb hybrid density functional\",\n  author = \"J. Heyd and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2004\",\n  volume = \"120\",\n  number = \"16\",\n  pages = \"7274\",\n  doi = \"10.1063/1.1668634\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/120/16/10.1063/1.1668634\"\n}"
};


func_reference_type xc_ref_Hodges1973_1428 = {
  "C. H. Hodges, Can. J. Phys. 51, 1428 (1973)",
  "10.1139/p73-189",
  "@article{Hodges1973_1428,\n  author = {Hodges, C. H.},\n  title = {Quantum Corrections to the Thomas--Fermi Approximation---The Kirzhnits Method},\n  journal = {Can. J. Phys.},\n  volume = {51},\n  number = {13},\n  pages = {1428-1437},\n  year = {1973},\n  doi = {10.1139/p73-189},\n  url = {https://doi.org/10.1139/p73-189}\n}"
};


func_reference_type xc_ref_Hu1985_391 = {
  "C. D. Hu and D. C. Langreth, Phys. Scr. 32, 391 (1985)",
  "10.1088/0031-8949/32/4/024",
  "@article{Hu1985_391,\n  title = {A Spin Dependent Version of the Langreth--Mehl Exchange-Correlation Functional},\n  author = {C. D. Hu and D. C. Langreth},\n  journal = {Phys. Scr.},\n  volume = {32},\n  number = {4},\n  pages = {391},\n  year = {1985},\n  doi = {10.1088/0031-8949/32/4/024},\n  url = {http://stacks.iop.org/1402-4896/32/i = 4/a = 024}\n}"
};


func_reference_type xc_ref_Hui2016_044114 = {
  "K. Hui and J.-D. Chai, J. Chem. Phys. 144, 044114 (2016)",
  "10.1063/1.4940734",
  "@article{Hui2016_044114,\n  author = {Hui, Kerwin and Chai, Jeng-Da},\n  title = {SCAN-based hybrid and double-hybrid density functionals from models without fitted parameters},\n  journal = {J. Chem. Phys.},\n  volume = {144},\n  number = {4},\n  pages = {044114},\n  year = {2016},\n  doi = {10.1063/1.4940734},\n  url = {https://doi.org/10.1063/1.4940734}\n}"
};


func_reference_type xc_ref_Iikura2001_3540 = {
  "H. Iikura, T. Tsuneda, T. Yanai, and K. Hirao, J. Chem. Phys. 115, 3540 (2001)",
  "10.1063/1.1383587",
  "@article{Iikura2001_3540,\n  title = \"A long-range correction scheme for generalized-gradient-approximation exchange functionals\",\n  author = \"H. Iikura and T. Tsuneda and T. Yanai and K. Hirao\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2001\",\n  volume = \"115\",\n  number = \"8\",\n  pages = \"3540\",\n  doi = \"10.1063/1.1383587\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/115/8/10.1063/1.1383587\"\n}"
};


func_reference_type xc_ref_Jana2017_4804 = {
  "S. Jana and P. Samal, J. Phys. Chem. A 121, 4804 (2017)",
  "10.1021/acs.jpca.7b03686",
  "@article{Jana2017_4804,\n  author = {Jana, Subrata and Samal, Prasanjit},\n  title = {Semilocal Exchange Energy Functional for Two-Dimensional Quantum Systems: A Step Beyond Generalized Gradient Approximations},\n  journal = {J. Phys. Chem. A},\n  volume = {121},\n  number = {25},\n  pages = {4804-4811},\n  year = {2017},\n  doi = {10.1021/acs.jpca.7b03686},\n  url = {https://doi.org/10.1021/acs.jpca.7b03686},\n}"
};


func_reference_type xc_ref_Jana2018_1 = {
  "S. Jana, B. Patra, H. Myneni, and P. Samal, Chem. Phys. Lett. 713, 1 (2018)",
  "10.1016/j.cplett.2018.10.007",
  "@article{Jana2018_1,\n  title = \"On the many-electron self-interaction error of the semilocal exchange hole based meta-GGA level range-separated hybrid with the B88 hybrids\",\n  author = \"Subrata Jana and Bikash Patra and Hemanadhan Myneni and Prasanjit Samal\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"713\",\n  pages = \"1--9\",\n  year = \"2018\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2018.10.007\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261418308194\",\n}"
};


func_reference_type xc_ref_Jana2018_8999 = {
  "S. Jana and P. Samal, Phys. Chem. Chem. Phys. 20, 8999 (2018)",
  "10.1039/C8CP00333E",
  "@article{Jana2018_8999,\n  author = \"Jana, Subrata and Samal, Prasanjit\",\n  title = \"A meta-GGA level screened range-separated hybrid functional by employing short range Hartree--Fock with a long range semilocal functional\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2018\",\n  volume = \"20\",\n  issue = \"13\",\n  pages = \"8999-9005\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C8CP00333E\",\n  url = \"http://doi.org/10.1039/C8CP00333E\"\n}"
};


func_reference_type xc_ref_Jana2019_6356 = {
  "S. Jana, K. Sharma, and P. Samal, J. Phys. Chem. A 123, 6356 (2019)",
  "10.1021/acs.jpca.9b02921",
  "@article{Jana2019_6356,\n  author = {Jana, Subrata and Sharma, Kedar and Samal, Prasanjit},\n  title = {Improving the Performance of Tao--Mo Non-empirical Density Functional with Broader Applicability in Quantum Chemistry and Materials Science},\n  journal = {J. Phys. Chem. A},\n  volume = {123},\n  number = {29},\n  pages = {6356-6369},\n  year = {2019},\n  doi = {10.1021/acs.jpca.9b02921},\n  url = {https://doi.org/10.1021/acs.jpca.9b02921}\n}"
};


func_reference_type xc_ref_Jana2021_024103 = {
  "S. Jana, S. K. Behera, S. Smiga, L. A. Constantin, and P. Samal, J. Chem. Phys. 155, 024103 (2021)",
  "10.1063/5.0051331",
  "@article{Jana2021_024103,\n  author = {Jana, Subrata and Behera, Sushant Kumar and \\'{S}miga, Szymon and Constantin, Lucian A. and Samal, Prasanjit},\n  title = {Accurate density functional made more versatile},\n  journal = {J. Chem. Phys.},\n  volume = {155},\n  number = {2},\n  pages = {024103},\n  year = {2021},\n  doi = {10.1063/5.0051331},\n  url = {https://doi.org/10.1063/5.0051331}\n}"
};


func_reference_type xc_ref_Jana2021_063007 = {
  "S. Jana, S. K. Behera, S. Smiga, L. A. Constantin, and P. Samal, New J. Phys. 23, 063007 (2021)",
  "10.1088/1367-2630/abfd4d",
  "@article{Jana2021_063007,\n	doi = {10.1088/1367-2630/abfd4d},\n	url = {https://doi.org/10.1088/1367-2630/abfd4d},\n	year = 2021,\n	month = {apr},\n	publisher = {{IOP} Publishing},\n	author = {Subrata Jana and Sushant Kumar Behera and Szymon {\\'{S}}miga and Lucian A. Constantin and Prasanjit Samal},\n	title = {Improving the applicability of the Pauli kinetic energy density based semilocal functional for solids},\n	journal = {New J. Phys.},\n  volume = {23},\n  pages = {063007}\n}"
};


func_reference_type xc_ref_Jemmer1995_3571 = {
  "P. Jemmer and P. J. Knowles, Phys. Rev. A 51, 3571 (1995)",
  "10.1103/PhysRevA.51.3571",
  "@article{Jemmer1995_3571,\n  title = {Exchange energy in Kohn-Sham density-functional theory},\n  author = {Jemmer, Patrick and Knowles, Peter J.},\n  journal = {Phys. Rev. A},\n  volume = {51},\n  issue = {5},\n  pages = {3571--3575},\n  year = {1995},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.51.3571},\n  url = {https://link.aps.org/doi/10.1103/PhysRevA.51.3571}\n}"
};


func_reference_type xc_ref_Jin2016_034107 = {
  "Y. Jin and R. J. Bartlett, J. Chem. Phys. 145, 034107 (2016)",
  "10.1063/1.4955497",
  "@article{Jin2016_034107,\n  author = {Yifan Jin and Rodney J. Bartlett},\n  title = {The QTP family of consistent functionals and potentials in Kohn-Sham density functional theory},\n  journal = {J. Chem. Phys.},\n  volume = {145},\n  number = {3},\n  pages = {034107},\n  year = {2016},\n  doi = {10.1063/1.4955497},\n  url = {http://doi.org/10.1063/1.4955497}\n}"
};


func_reference_type xc_ref_Jin2018_064111 = {
  "Y. Jin and R. J. Bartlett, J. Chem. Phys. 149, 064111 (2018)",
  "10.1063/1.5038434",
  "@article{Jin2018_064111,\n  author = {Jin, Yifan and Bartlett, Rodney J.},\n  title = {Accurate computation of X-ray absorption spectra with ionization potential optimized global hybrid functional},\n  journal = {J. Chem. Phys.},\n  volume = {149},\n  number = {6},\n  pages = {064111},\n  year = {2018},\n  doi = {10.1063/1.5038434},\n  url = {https://doi.org/10.1063/1.5038434}\n}"
};


func_reference_type xc_ref_Kang2001_11040 = {
  "J. K. Kang and C. B. Musgrave, J. Chem. Phys. 115, 11040 (2001)",
  "10.1063/1.1415079",
  "@article{Kang2001_11040,\n  author = {Jeung Ku Kang and Charles B. Musgrave},\n  title = {Prediction of transition state barriers and enthalpies of reaction by a new hybrid density-functional approximation},\n  journal = {J. Chem. Phys.},\n  volume = {115},\n  number = {24},\n  pages = {11040-11051},\n  year = {2001},\n  doi = {10.1063/1.1415079},\n  url = {http://doi.org/10.1063/1.1415079}\n}"
};


func_reference_type xc_ref_Karasiev2006_111 = {
  "V. V. Karasiev, S. B. Trickey, and F. E. Harris, J. Comput.-Aided Mater. Des. 13, 111 (2006)",
  "10.1007/s10820-006-9019-8",
  "@article{Karasiev2006_111,\n  author = \"Karasiev, V. V. and Trickey, S. B. and Harris, Frank E.\",\n  title = \"Born--Oppenheimer Interatomic Forces from Simple, Local Kinetic Energy Density Functionals\",\n  journal = \"J. Comput.-Aided Mater. Des.\",\n  year = \"2006\",\n  month = \"Oct\",\n  day = \"01\",\n  volume = \"13\",\n  number = \"1\",\n  pages = \"111--129\",\n  issn = \"1573-4900\",\n  doi = \"10.1007/s10820-006-9019-8\",\n  url = \"https://doi.org/10.1007/s10820-006-9019-8\"\n}"
};


func_reference_type xc_ref_Karasiev2009_245120 = {
  "V. V. Karasiev, R. S. Jones, S. B. Trickey, and F. E. Harris, Phys. Rev. B 80, 245120 (2009)",
  "10.1103/PhysRevB.80.245120",
  "@article{Karasiev2009_245120,\n  title = {Properties of constraint-based single-point approximate kinetic energy functionals},\n  author = {Karasiev, V. V. and Jones, R. S. and Trickey, S. B. and Harris, Frank E.},\n  journal = {Phys. Rev. B},\n  volume = {80},\n  issue = {24},\n  pages = {245120},\n  year = {2009},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.80.245120},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.80.245120}\n}"
};


func_reference_type xc_ref_Karasiev2013_161108 = {
  "V. V. Karasiev, D. Chakraborty, O. A. Shukruto, and S. B. Trickey, Phys. Rev. B 88, 161108 (2013)",
  "10.1103/PhysRevB.88.161108",
  "@article{Karasiev2013_161108,\n  title = {Nonempirical generalized gradient approximation free-energy functional for orbital-free simulations},\n  author = {Karasiev, Valentin V. and Chakraborty, Debajit and Shukruto, Olga A. and Trickey, S. B.},\n  journal = {Phys. Rev. B},\n  volume = {88},\n  issue = {16},\n  pages = {161108},\n  year = {2013},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.88.161108},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.88.161108}\n}"
};


func_reference_type xc_ref_Karasiev2014_076403 = {
  "V. V. Karasiev, T. Sjostrom, J. Dufty, and S. B. Trickey, Phys. Rev. Lett. 112, 076403 (2014)",
  "10.1103/PhysRevLett.112.076403",
  "@article{Karasiev2014_076403,\n  title = {Accurate Homogeneous Electron Gas Exchange-Correlation Free Energy for Local Spin-Density Calculations},\n  author = {Karasiev, Valentin V. and Sjostrom, Travis and Dufty, James and Trickey, S. B.},\n  journal = {Phys. Rev. Lett.},\n  volume = {112},\n  issue = {7},\n  pages = {076403},\n  year = {2014},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.112.076403},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.112.076403}\n}"
};


func_reference_type xc_ref_Karasiev2016_157101 = {
  "V. V. Karasiev, J. Chem. Phys. 145, 157101 (2016)",
  "10.1063/1.4964758",
  "@article{Karasiev2016_157101,\n  author = {Valentin V. Karasiev},\n  title = {Comment on \"Communication: Simple and accurate uniform electron gas correlation energy for the full range of densities\" [J. Chem. Phys. 145, 021101 (2016)]},\n  journal = {J. Chem. Phys.},\n  volume = {145},\n  number = {15},\n  pages = {157101},\n  year = {2016},\n  doi = {10.1063/1.4964758},\n  url = {https://doi.org/10.1063/1.4964758}\n}"
};


func_reference_type xc_ref_Karasiev2018_076401 = {
  "V. V. Karasiev, J. W. Dufty, and S. B. Trickey, Phys. Rev. Lett. 120, 076401 (2018)",
  "10.1103/PhysRevLett.120.076401",
  "@article{Karasiev2018_076401,\n  title = {Nonempirical Semilocal Free-Energy Density Functional for Matter under Extreme Conditions},\n  author = {Karasiev, Valentin V. and Dufty, James W. and Trickey, S. B.},\n  journal = {Phys. Rev. Lett.},\n  volume = {120},\n  issue = {7},\n  pages = {076401},\n  numpages = {7},\n  year = {2018},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.120.076401},\n  url = {https://link.aps.org/doi/10.1103/PhysRevLett.120.076401}\n}"
};


func_reference_type xc_ref_Karton2008_12868 = {
  "A. Karton, A. Tarnopolsky, J.-F. Lamere, G. C. Schatz, and J. M. L. Martin, J. Phys. Chem. A 112, 12868 (2008)",
  "10.1021/jp801805p",
  "@article{Karton2008_12868,\n  author = {Karton, Amir and Tarnopolsky, Alex and Lam\\`{e}re, Jean-Fran\\c{c}ois and Schatz, George C. and Martin, Jan M. L.},\n  title = {Highly Accurate First-Principles Benchmark Data Sets for the Parametrization and Validation of Density Functional and Other Approximate Methods. Derivation of a Robust, Generally Applicable, Double-Hybrid Functional for Thermochemistry and Thermochemical Kinetics},\n  journal = {J. Phys. Chem. A},\n  volume = {112},\n  number = {50},\n  pages = {12868--12886},\n  year = {2008},\n  doi = {10.1021/jp801805p},\n  url = {https://doi.org/10.1021/jp801805p}\n}"
};


func_reference_type xc_ref_Kaupp2011_16973 = {
  "M. Kaupp, M. Renz, M. Parthey, M. Stolte, F. Wurthner, and C. Lambert, Phys. Chem. Chem. Phys. 13, 16973 (2011)",
  "10.1039/C1CP21772K",
  "@article{Kaupp2011_16973,\n  author = {Kaupp, Martin and Renz, Manuel and Parthey, Matthias and Stolte, Matthias and W\\\"{u}rthner, Frank and Lambert, Christoph},\n  title  = {Computational and spectroscopic studies of organic mixed-valence compounds: where is the charge?},\n  journal = {Phys. Chem. Chem. Phys.},\n  year = {2011},\n  volume = {13},\n  issue = {38},\n  pages = {16973-16986},\n  publisher = {The Royal Society of Chemistry},\n  doi = {10.1039/C1CP21772K},\n  url = {http://doi.org/10.1039/C1CP21772K}\n}"
};


func_reference_type xc_ref_Keal2003_3015 = {
  "T. W. Keal and D. J. Tozer, J. Chem. Phys. 119, 3015 (2003)",
  "10.1063/1.1590634",
  "@article{Keal2003_3015,\n  title = \"The exchange-correlation potential in Kohn--Sham nuclear magnetic resonance shielding calculations\",\n  author = \"T. W. Keal and D. J. Tozer\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2003\",\n  volume = \"119\",\n  number = \"6\",\n  pages = \"3015\",\n  doi = \"10.1063/1.1590634\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/119/6/10.1063/1.1590634\"\n}"
};


func_reference_type xc_ref_Keal2004_5654 = {
  "T. W. Keal and D. J. Tozer, J. Chem. Phys. 121, 5654 (2004)",
  "10.1063/1.1784777",
  "@article{Keal2004_5654,\n  author = {Thomas W. Keal and David J. Tozer},\n  title = {A semiempirical generalized gradient approximation exchange-correlation functional},\n  journal = {J. Chem. Phys.},\n  volume = {121},\n  number = {12},\n  pages = {5654-5660},\n  year = {2004},\n  doi = {10.1063/1.1784777},\n  url = {https://doi.org/10.1063/1.1784777}\n}"
};


func_reference_type xc_ref_Keal2005_121103 = {
  "T. W. Keal and D. J. Tozer, J. Chem. Phys. 123, 121103 (2005)",
  "10.1063/1.2061227",
  "@article{Keal2005_121103,\n  title = \"Semiempirical hybrid functional with improved performance in an extensive chemical assessment\",\n  author = \"T. W. Keal and D. J. Tozer\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2005\",\n  volume = \"123\",\n  number = \"12\",\n  pages = 121103,\n  doi = \"10.1063/1.2061227\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/123/12/10.1063/1.2061227\"\n}"
};


func_reference_type xc_ref_Kirznits1957_115 = {
  "D. A. Kirznits, Zh. Eksp. Teor. Fiz. 32, 115 (1957), [J. Exp. Theor. Phys. 5, 64 (1957)]",
  "",
  "@article{Kirznits1957_115,\n  title = {Quantum Corrections to the Thomas--Fermi Equation},\n  author = {D. A. Kirznits},\n  journal = {Zh. Eksp. Teor. Fiz.},\n  year = {1957},\n  volume = {32},\n  pages = {115},\n  note = {[J. Exp. Theor. Phys. 5, 64 (1957)]},\n}"
};


func_reference_type xc_ref_Klimes2010_022201 = {
  "J. Klimes, D. R. Bowler, and A. Michaelides, J. Phys.: Condens. Matter 22, 022201 (2010)",
  "10.1088/0953-8984/22/2/022201",
  "@article{Klimes2010_022201,\n  title = {Chemical accuracy for the van der Waals density functional},\n  author = {J. Klime\\v{s} and D. R. Bowler and A. Michaelides},\n  journal = {J. Phys.: Condens. Matter},\n  volume = {22},\n  number = {2},\n  pages = {022201},\n  year = {2010},\n  doi = {10.1088/0953-8984/22/2/022201},\n  url = {http://stacks.iop.org/0953-8984/22/i=2/a=022201}\n}"
};


func_reference_type xc_ref_Klimes2011_195131 = {
  "J. Klimes, D. R. Bowler, and A. Michaelides, Phys. Rev. B 83, 195131 (2011)",
  "10.1103/PhysRevB.83.195131",
  "@article{Klimes2011_195131,\n  title = {Van der Waals density functionals applied to solids},\n  author = {Klime\\v{s}, Ji\\v{r}\\'{i} and Bowler, David R. and Michaelides, Angelos},\n  journal = {Phys. Rev. B},\n  volume = {83},\n  issue = {19},\n  pages = {195131},\n  year = {2011},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.83.195131},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.83.195131}\n}"
};


func_reference_type xc_ref_Koehl1996_835 = {
  "R. M. Koehl, G. K. Odom, and G. E. Scuseria, Mol. Phys. 87, 835 (1996)",
  "10.1080/00268979600100561",
  "@article{Koehl1996_835,\n  author = {Richard M. Koehl and Gregory K. Odom and Gustavo E. Scuseria},\n  title = {The use of density matrix expansions for calculating molecular exchange energies},\n  journal = {Mol. Phys.},\n  volume = {87},\n  number = {4},\n  pages = {835-843},\n  year  = {1996},\n  publisher = {Taylor & Francis},\n  doi = {10.1080/00268979600100561},\n  url = {https://doi.org/10.1080/00268979600100561},\n}"
};


func_reference_type xc_ref_Kohn1965_A1133 = {
  "W. Kohn and L. J. Sham, Phys. Rev. 140, A1133 (1965)",
  "10.1103/PhysRev.140.A1133",
  "@article{Kohn1965_A1133,\n  title = {Self-Consistent Equations Including Exchange and Correlation Effects},\n  author = {W. Kohn and L. J. Sham},\n  journal = {Phys. Rev.},\n  volume = {140},\n  issue = {4A},\n  pages = {A1133--A1138},\n  year = {1965},\n  month = {Nov},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRev.140.A1133},\n  url = {http://link.aps.org/doi/10.1103/PhysRev.140.A1133}\n}"
};


func_reference_type xc_ref_Kompaneets1956_427 = {
  "A. S. Kompaneets and E. S. Pavlovskii, Zh. Eksp. Teor. Fiz. 31, 427 (1956), [J. Exp. Theor. Phys. 4, 328 (1957)]",
  "",
  "@article{Kompaneets1956_427,\n  title = {The Self-consistent Field Equations in an Atom},\n  author = {A. S. Kompaneets and E. S. Pavlovskii},\n  journal = {Zh. Eksp. Teor. Fiz.},\n  year = {1956},\n  volume = {31},\n  pages = {427},\n  note = {[J. Exp. Theor. Phys. 4, 328 (1957)]},\n}"
};


func_reference_type xc_ref_Kraisler2010_042516 = {
  "E. Kraisler, G. Makov, and I. Kelson, Phys. Rev. A 82, 042516 (2010)",
  "10.1103/PhysRevA.82.042516",
  "@article{Kraisler2010_042516,\n  title = {Ensemble $v$-representable ab initio density-functional calculation of energy and spin in atoms: A test of exchange-correlation approximations},\n  author = {Kraisler, Eli and Makov, Guy and Kelson, Itzhak},\n  journal = {Phys. Rev. A},\n  volume = {82},\n  issue = {4},\n  pages = {042516},\n  year = {2010},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.82.042516},\n  url = {https://link.aps.org/doi/10.1103/PhysRevA.82.042516}\n}"
};


func_reference_type xc_ref_Krieger1999_463 = {
  "J. B. Krieger, J. Chen, G. J. Iafrate, and A. Savin, Construction of an accurate self-interaction-corrected correlation energy functional based on an electron gas with a gap, in Electron Correlations and Materials Properties, edited by A. Gonis, N. Kioussis, and M. Ciftan (Springer US, Boston, MA, 1999) pp. 463--477",
  "10.1007/978-1-4615-4715-0_28",
  "@Inbook{Krieger1999_463,\n  author = \"Krieger, Joseph B. and Chen, Jiqiang and Iafrate, Gerald J. and Savin, Andreas\",\n  editor = \"Gonis, A. and Kioussis, N. and Ciftan, M.\",\n  title = \"Construction of An Accurate Self-interaction-corrected Correlation Energy Functional Based on An Electron Gas with A Gap\",\n  bookTitle = \"Electron Correlations and Materials Properties\",\n  year = \"1999\",\n  publisher = \"Springer US\",\n  address = \"Boston, MA\",\n  pages = \"463--477\",\n  isbn = \"978-1-4615-4715-0\",\n  doi = \"10.1007/978-1-4615-4715-0_28\",\n  url = \"http://doi.org/10.1007/978-1-4615-4715-0_28\"\n}"
};


func_reference_type xc_ref_Krieger2001_48 = {
  "J. B. Krieger, J. Chen, and S. Kurth, AIP Conf. Proc. 577, 48 (2001)",
  "10.1063/1.1390178",
  "@article{Krieger2001_48,\n  author = {Joseph B. Krieger and Jiqiang Chen and Stefan Kurth},\n  title = {Construction and application of an accurate self-interaction-corrected correlation energy functional based on an electron gas with a gap},\n  journal = {AIP Conf. Proc.},\n  volume = {577},\n  number = {1},\n  pages = {48-69},\n  year = {2001},\n  doi = {10.1063/1.1390178},\n  url = {http://aip.scitation.org/doi/abs/10.1063/1.1390178},\n}"
};


func_reference_type xc_ref_Krukau2006_224106 = {
  "A. V. Krukau, O. A. Vydrov, A. F. Izmaylov, and G. E. Scuseria, J. Chem. Phys. 125, 224106 (2006)",
  "10.1063/1.2404663",
  "@article{Krukau2006_224106,\n  title = \"Influence of the exchange screening parameter on the performance of screened hybrid functionals\",\n  author = \"A. V. Krukau and O. A. Vydrov and A. F. Izmaylov and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2006\",\n  volume = \"125\",\n  number = \"22\",\n  pages = 224106,\n  doi = \"10.1063/1.2404663\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/125/22/10.1063/1.2404663\"\n}"
};


func_reference_type xc_ref_Kurth1999_889 = {
  "S. Kurth, J. P. Perdew, and P. Blaha, Int. J. Quantum Chem. 75, 889 (1999)",
  "10.1002/(SICI)1097-461X(1999)75:4/5<889::AID-QUA54>3.0.CO;2-8",
  "@article{Kurth1999_889,\n  author = {Kurth, Stefan and Perdew, John P. and Blaha, Peter},\n  title = {Molecular and solid-state tests of density functional approximations: LSD, GGAs, and meta-GGAs},\n  journal = {Int. J. Quantum Chem.},\n  volume = {75},\n  number = {4-5},\n  pages = {889-909},\n  doi = {10.1002/(SICI)1097-461X(1999)75:4/5<889::AID-QUA54>3.0.CO;2-8},\n  url = {https://onlinelibrary.wiley.com/doi/abs/10.1002/%28SICI%291097-461X%281999%2975%3A4/5%3C889%3A%3AAID-QUA54%3E3.0.CO%3B2-8},\n  year = {1999}\n}"
};


func_reference_type xc_ref_Lacks1993_4681 = {
  "D. J. Lacks and R. G. Gordon, Phys. Rev. A 47, 4681 (1993)",
  "10.1103/PhysRevA.47.4681",
  "@article{Lacks1993_4681,\n  title = {Pair interactions of rare-gas atoms as a test of exchange-energy-density functionals in regions of large density gradients},\n  author = {D. J. Lacks and R. G. Gordon},\n  journal = {Phys. Rev. A},\n  volume = {47},\n  issue = {6},\n  pages = {4681},\n  year = {1993},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.47.4681},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.47.4681}\n}"
};


func_reference_type xc_ref_Lacks1994_4446 = {
  "D. J. Lacks and R. G. Gordon, J. Chem. Phys. 100, 4446 (1994)",
  "10.1063/1.466274",
  "@article{Lacks1994_4446,\n  title = \"Tests of nonlocal kinetic energy functionals\",\n  author = \"D. J. Lacks and R. G. Gordon\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1994\",\n  volume = \"100\",\n  number = \"6\",\n  pages = \"4446\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/100/6/10.1063/1.466274\",\n  doi = \"10.1063/1.466274\"\n}"
};


func_reference_type xc_ref_Langreth1981_446 = {
  "D. C. Langreth and M. J. Mehl, Phys. Rev. Lett. 47, 446 (1981)",
  "10.1103/PhysRevLett.47.446",
  "@article{Langreth1981_446,\n  title = {Easily Implementable Nonlocal Exchange-Correlation Energy Functional},\n  author = {D. C. Langreth and M. J. Mehl},\n  journal = {Phys. Rev. Lett.},\n  volume = {47},\n  issue = {6},\n  pages = {446},\n  year = {1981},\n  month = {Aug},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.47.446},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.47.446}\n}"
};


func_reference_type xc_ref_Laricchia2011_2439 = {
  "S. Laricchia, E. Fabiano, L. A. Constantin, and F. Della Sala, J. Chem. Theory Comput. 7, 2439 (2011)",
  "10.1021/ct200382w",
  "@article{Laricchia2011_2439,\n  title = {Generalized Gradient Approximations of the Noninteracting Kinetic Energy from the Semiclassical Atom Theory: Rationalization of the Accuracy of the Frozen Density Embedding Theory for Nonbonded Interactions},\n  author = {S. Laricchia and E. Fabiano and L. A. Constantin and Della Sala, F.},\n  journal = {J. Chem. Theory Comput.},\n  volume = {7},\n  number = {8},\n  pages = {2439},\n  year = {2011},\n  doi = {10.1021/ct200382w},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct200382w}\n}"
};


func_reference_type xc_ref_Laricchia2014_164 = {
  "S. Laricchia, L. A. Constantin, E. Fabiano, and F. Della Sala, J. Chem. Theory Comput. 10, 164 (2014)",
  "10.1021/ct400836s",
  "@article{Laricchia2014_164,\n  author = {Laricchia, Savio and Constantin, Lucian A. and Fabiano, Eduardo and Della Sala, Fabio},\n  title = {Laplacian-Level Kinetic Energy Approximations Based on the Fourth-Order Gradient Expansion: Global Assessment and Application to the Subsystem Formulation of Density Functional Theory},\n  journal = {J. Chem. Theory Comput.},\n  volume = {10},\n  number = {1},\n  pages = {164-179},\n  year = {2014},\n  doi = {10.1021/ct400836s},\n  url = {https://doi.org/10.1021/ct400836s}\n}"
};


func_reference_type xc_ref_Lee1987_2377 = {
  "C. Lee and R. G. Parr, Phys. Rev. A 35, 2377 (1987)",
  "10.1103/PhysRevA.35.2377",
  "@article{Lee1987_2377,\n  title = {Gaussian and other approximations to the first-order density matrix of electronic systems, and the derivation of various local-density-functional theories},\n  author = {C. Lee and R. G. Parr},\n  journal = {Phys. Rev. A},\n  volume = {35},\n  issue = {6},\n  pages = {2377},\n  year = {1987},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.35.2377},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.35.2377}\n}"
};


func_reference_type xc_ref_Lee1988_785 = {
  "C. Lee, W. Yang, and R. G. Parr, Phys. Rev. B 37, 785 (1988)",
  "10.1103/PhysRevB.37.785",
  "@article{Lee1988_785,\n  title = {Development of the Colle--Salvetti correlation-energy formula into a functional of the electron density},\n  author = {C. Lee and W. Yang and R. G. Parr},\n  journal = {Phys. Rev. B},\n  volume = {37},\n  issue = {2},\n  pages = {785},\n  year = {1988},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.37.785},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.37.785}\n}"
};


func_reference_type xc_ref_Lee1990_193 = {
  "C. Lee and R. G. Parr, Phys. Rev. A 42, 193 (1990)",
  "10.1103/PhysRevA.42.193",
  "@article{Lee1990_193,\n  title = {Exchange-correlation functional for atoms and molecules},\n  author = {Lee, Chengteh and Parr, Robert G.},\n  journal = {Phys. Rev. A},\n  volume = {42},\n  issue = {1},\n  pages = {193--200},\n  year = {1990},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.42.193},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.42.193}\n}"
};


func_reference_type xc_ref_Lee1991_768 = {
  "H. Lee, C. Lee, and R. G. Parr, Phys. Rev. A 44, 768 (1991)",
  "10.1103/PhysRevA.44.768",
  "@article{Lee1991_768,\n  title = {Conjoint gradient correction to the Hartree--Fock kinetic- and exchange-energy density functionals},\n  author = {H. Lee and C. Lee and R. G. Parr},\n  journal = {Phys. Rev. A},\n  volume = {44},\n  issue = {1},\n  pages = {768},\n  year = {1991},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.44.768},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.44.768}\n}"
};


func_reference_type xc_ref_Lehtola2018_1 = {
  "S. Lehtola, C. Steigemann, M. J. Oliveira, and M. A. Marques, SoftwareX 7, 1 (2018)",
  "10.1016/j.softx.2017.11.002",
  "@article{Lehtola2018_1,\n  title = \"Recent developments in libxc -- A comprehensive library of functionals for density functional theory\",\n  journal = \"SoftwareX\",\n  volume = \"7\",\n  pages = \"1--5\",\n  year = \"2018\",\n  issn = \"2352-7110\",\n  doi = \"10.1016/j.softx.2017.11.002\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S2352711017300602\",\n  author = \"Susi Lehtola and Conrad Steigemann and Micael J.T. Oliveira and Miguel A.L. Marques\"\n}"
};


func_reference_type xc_ref_Lehtola2021_943 = {
  "S. Lehtola and M. A. L. Marques, J. Chem. Theory Comput. 17, 943 (2021)",
  "10.1021/acs.jctc.0c01147",
  "@article{Lehtola2021_943,\n  author = {Lehtola, Susi and Marques, Miguel A. L.},\n  title = {Meta-Local Density Functionals: A New Rung on Jacob's Ladder},\n  journal = {J. Chem. Theory Comput.},\n  volume = {17},\n  number = {2},\n  pages = {943-948},\n  year = {2021},\n  doi = {10.1021/acs.jctc.0c01147},\n  url = {https://doi.org/10.1021/acs.jctc.0c01147}\n}"
};


func_reference_type xc_ref_Lehtomaki2019_165111 = {
  "J. Lehtomaki and O. Lopez-Acevedo, Phys. Rev. B 100, 165111 (2019)",
  "10.1103/PhysRevB.100.165111",
  "@article{Lehtomaki2019_165111,\n  title = {Semilocal kinetic energy functionals with parameters from neutral atoms},\n  author = {Lehtom\\\"{a}ki, Jouko and Lopez-Acevedo, Olga},\n  journal = {Phys. Rev. B},\n  volume = {100},\n  issue = {16},\n  pages = {165111},\n  numpages = {9},\n  year = {2019},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.100.165111},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.100.165111}\n}"
};


func_reference_type xc_ref_Lembarki1994_5328 = {
  "A. Lembarki and H. Chermette, Phys. Rev. A 50, 5328 (1994)",
  "10.1103/PhysRevA.50.5328",
  "@article{Lembarki1994_5328,\n  title = {Obtaining a gradient-corrected kinetic-energy functional from the Perdew--Wang exchange functional},\n  author = {A. Lembarki and H. Chermette},\n  journal = {Phys. Rev. A},\n  volume = {50},\n  issue = {6},\n  pages = {5328},\n  year = {1994},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.50.5328},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.50.5328}\n}"
};


func_reference_type xc_ref_Lieb1981_603 = {
  "E. H. Lieb, Rev. Mod. Phys. 53, 603 (1981)",
  "10.1103/RevModPhys.53.603",
  "@article{Lieb1981_603,\n  title = {Thomas--Fermi and related theories of atoms and molecules},\n  author = {E. H. Lieb},\n  journal = {Rev. Mod. Phys.},\n  volume = {53},\n  issue = {4},\n  pages = {603},\n  year = {1981},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/RevModPhys.53.603},\n  url = {http://link.aps.org/doi/10.1103/RevModPhys.53.603}\n}"
};


func_reference_type xc_ref_Lin2004_365 = {
  "C. Y. Lin, M. W. George, and P. M. W. Gill, Aust. J. Chem. 57, 365 (2004)",
  "10.1071/CH03263",
  "@article{Lin2004_365,\n  author = {Lin, Ching Yeh and George, Michael W. and Gill, Peter M. W.},\n  title = {EDF2: A Density Functional for Predicting Molecular Vibrational Frequencies},\n  volume = {57},\n  number = {4},\n  pages = {365-370},\n  year = {2004},\n  doi = {10.1071/CH03263},\n  url = {http://www.publish.csiro.au/?paper=CH03263},\n  journal = {Aust. J. Chem.}\n}"
};


func_reference_type xc_ref_Lin2013_263 = {
  "Y.-S. Lin, G.-D. Li, S.-P. Mao, and J.-D. Chai, J. Chem. Theory Comput. 9, 263 (2013)",
  "10.1021/ct300715s",
  "@article{Lin2013_263,\n  author = {Lin, You-Sheng and Li, Guan-De and Mao, Shan-Ping and Chai, Jeng-Da},\n  title = {Long-Range Corrected Hybrid Density Functionals with Improved Dispersion Corrections},\n  journal = {J. Chem. Theory Comput.},\n  volume = {9},\n  number = {1},\n  pages = {263-272},\n  year = {2013},\n  doi = {10.1021/ct300715s},\n  url = {https://doi.org/10.1021/ct300715s}\n}"
};


func_reference_type xc_ref_Liu1996_2211 = {
  "S. Liu and R. G. Parr, Phys. Rev. A 53, 2211 (1996)",
  "10.1103/PhysRevA.53.2211",
  "@article{Liu1996_2211,\n  title = {Expansions of the correlation-energy density functional ${\\mathit{E}}_{\\mathit{c}}$[\\ensuremath{\\rho}] and its kinetic-energy component ${\\mathit{T}}_{\\mathit{c}}$[\\ensuremath{\\rho}] in terms of homogeneous functionals},\n  author = {Liu, Shubin and Parr, Robert G.},\n  journal = {Phys. Rev. A},\n  volume = {53},\n  issue = {4},\n  pages = {2211--2219},\n  year = {1996},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.53.2211},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.53.2211}\n}"
};


func_reference_type xc_ref_Liu2000_29 = {
  "S. Liu and R. Parr, J. Mol. Struct.: THEOCHEM 501--502, 29 (2000)",
  "10.1016/S0166-1280(99)00410-8",
  "@article{Liu2000_29,\n  title = \"Homogeneities in density of various {LDA} energy functionals\",\n  journal = \"J. Mol. Struct.: THEOCHEM\",\n  volume = \"501--502\",\n  number = \"\",\n  pages = \"29 - 34\",\n  year = \"2000\",\n  issn = \"0166-1280\",\n  doi = \"10.1016/S0166-1280(99)00410-8\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0166128099004108\",\n  author = \"S Liu and R.G Parr\"\n}"
};


func_reference_type xc_ref_Livshits2007_2932 = {
  "E. Livshits and R. Baer, Phys. Chem. Chem. Phys. 9, 2932 (2007)",
  "10.1039/B617919C",
  "@article{Livshits2007_2932,\n  author = \"Livshits, Ester and Baer, Roi\",\n  title = \"A well-tempered density functional theory of electrons in molecules\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2007\",\n  volume = \"9\",\n  issue = \"23\",\n  pages = \"2932-2941\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/B617919C\",\n  url = \"http://doi.org/10.1039/B617919C\",\n}"
};


func_reference_type xc_ref_Loos2013_064108 = {
  "P.-F. Loos, J. Chem. Phys. 138, 064108 (2013)",
  "10.1063/1.4790613",
  "@article{Loos2013_064108,\n  title = \"High-density correlation energy expansion of the one-dimensional uniform electron gas\",\n  author = \"P.-F. Loos\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2013\",\n  volume = \"138\",\n  number = \"6\",\n  pages = 064108,\n  doi = \"10.1063/1.4790613\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/138/6/10.1063/1.4790613\"\n}"
};


func_reference_type xc_ref_Loos2017_114108 = {
  "P.-F. Loos, J. Chem. Phys. 146, 114108 (2017)",
  "10.1063/1.4978409",
  "@article{Loos2017_114108,\n  author = {Pierre-Fran\\c{c}ois Loos},\n  title = {Exchange functionals based on finite uniform electron gases},\n  journal = {J. Chem. Phys.},\n  volume = {146},\n  number = {11},\n  pages = {114108},\n  year = {2017},\n  doi = {10.1063/1.4978409},\n  url = {http://doi.org/10.1063/1.4978409},\n}"
};


func_reference_type xc_ref_Lu2013_64 = {
  "L. Lu, H. Hu, H. Hou, and B. Wang, Comput. Theor. Chem. 1015, 64 (2013)",
  "10.1016/j.comptc.2013.04.009",
  "@article{Lu2013_64,\n  title = \"An improved B3LYP method in the calculation of organic thermochemistry and reactivity\",\n  author = \"L. Lu and H. Hu and H. Hou and B. Wang\",\n  journal = \"Comput. Theor. Chem.\",\n  volume = \"1015\",\n  pages = \"64\",\n  year = \"2013\",\n  issn = \"2210-271X\",\n  doi = \"10.1016/j.comptc.2013.04.009\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S2210271X13001576\"\n}"
};


func_reference_type xc_ref_Ludena1986 = {
  "E. V. Ludena, in Cond. Matt. Theor., Vol. 1, edited by F. B. Malik (Plenum, New York, 1986) p. 183",
  "",
  "@inproceedings{Ludena1986,\n  author = {E. V. {Lude\\~{n}a}},\n  title = {},\n  booktitle = {Cond. Matt. Theor.},\n  pages = {183},\n  year = {1986},\n  editor = {F. B. Malik},\n  volume = {1},\n  address = {New York},\n  publisher =   {Plenum},\n}"
};


func_reference_type xc_ref_Lundgaard2016_235162 = {
  "K. T. Lundgaard, J. Wellendorff, J. Voss, K. W. Jacobsen, and T. Bligaard, Phys. Rev. B 93, 235162 (2016)",
  "10.1103/PhysRevB.93.235162",
  "@article{Lundgaard2016_235162,\n  title = {mBEEF-vdW: Robust fitting of error estimation density functionals},\n  author = {Lundgaard, Keld T. and Wellendorff, Jess and Voss, Johannes and Jacobsen, Karsten W. and Bligaard, Thomas},\n  journal = {Phys. Rev. B},\n  volume = {93},\n  issue = {23},\n  pages = {235162},\n  year = {2016},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.93.235162},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.93.235162}\n}"
};


func_reference_type xc_ref_Luo2018_041111 = {
  "K. Luo, V. V. Karasiev, and S. B. Trickey, Phys. Rev. B 98, 041111 (2018)",
  "10.1103/PhysRevB.98.041111",
  "@article{Luo2018_041111,\n  title = {A simple generalized gradient approximation for the noninteracting kinetic energy density functional},\n  author = {Luo, Kai and Karasiev, Valentin V. and Trickey, S. B.},\n  journal = {Phys. Rev. B},\n  volume = {98},\n  issue = {4},\n  pages = {041111},\n  year = {2018},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.98.041111},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.98.041111}\n}"
};


func_reference_type xc_ref_Lynch2000_4811 = {
  "B. J. Lynch, P. L. Fast, M. Harris, and D. G. Truhlar, J. Phys. Chem. A 104, 4811 (2000)",
  "10.1021/jp000497z",
  "@article{Lynch2000_4811,\n  title = {Adiabatic Connection for Kinetics},\n  author = {B. J. Lynch and P. L. Fast and M. Harris and D. G. Truhlar},\n  journal = {J. Phys. Chem. A},\n  volume = {104},\n  number = {21},\n  pages = {4811},\n  year = {2000},\n  doi = {10.1021/jp000497z},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp000497z}\n}"
};


func_reference_type xc_ref_MacDonald1979_2977 = {
  "A. H. MacDonald and S. H. Vosko, J. Phys. C: Solid State Phys. 12, 2977 (1979)",
  "10.1088/0022-3719/12/15/007",
  "@article{MacDonald1979_2977,\n  author = {A H MacDonald and S H Vosko},\n  title = {A relativistic density functional formalism},\n  journal = {J. Phys. C: Solid State Phys.},\n  volume = {12},\n  number = {15},\n  pages = {2977},\n  url = {http://stacks.iop.org/0022-3719/12/i = 15/a = 007},\n  year = {1979},\n  doi = {10.1088/0022-3719/12/15/007}\n}"
};


func_reference_type xc_ref_Madsen2007_195108 = {
  "G. K. H. Madsen, Phys. Rev. B 75, 195108 (2007)",
  "10.1103/PhysRevB.75.195108",
  "@article{Madsen2007_195108,\n  title = {Functional form of the generalized gradient approximation for exchange: The PBE$\\alpha$ functional},\n  author = {G. K. H. Madsen},\n  journal = {Phys. Rev. B},\n  volume = {75},\n  issue = {19},\n  pages = {195108},\n  year = {2007},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.75.195108},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.75.195108}\n}"
};


func_reference_type xc_ref_Manby2000_7002 = {
  "F. R. Manby and P. J. Knowles, J. Chem. Phys. 112, 7002 (2000)",
  "10.1063/1.481298",
  "@article{Manby2000_7002,\n  title = \"An exchange functional for accurate virtual orbital energies\",\n  author = \"F. R. Manby and P. J. Knowles\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2000\",\n  volume = \"112\",\n  number = \"16\",\n  pages = \"7002\",\n  doi = \"10.1063/1.481298\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/112/16/10.1063/1.481298\"\n}"
};


func_reference_type xc_ref_Mardirossian2014_9904 = {
  "N. Mardirossian and M. Head-Gordon, Phys. Chem. Chem. Phys. 16, 9904 (2014)",
  "10.1039/C3CP54374A",
  "@article{Mardirossian2014_9904,\n  author = \"Mardirossian, Narbe and Head-Gordon, Martin\",\n  title = \"$\\omega$B97X-V: A 10-parameter, range-separated hybrid,\n            generalized gradient approximation density\n            functional with nonlocal correlation, designed by a\n            survival-of-the-fittest strategy\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2014\",\n  volume = \"16\",\n  issue = \"21\",\n  pages = \"9904-9924\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C3CP54374A\",\n  url = \"http://doi.org/10.1039/C3CP54374A\"\n}"
};


func_reference_type xc_ref_Mardirossian2015_074111 = {
  "N. Mardirossian and M. Head-Gordon, J. Chem. Phys. 142, 074111 (2015)",
  "10.1063/1.4907719",
  "@article{Mardirossian2015_074111,\n  author = {Mardirossian, Narbe and Head-Gordon, Martin},\n  title = {Mapping the genome of meta-generalized gradient approximation density functionals: The search for B97M-V},\n  journal = {J. Chem. Phys.},\n  volume = {142},\n  number = {7},\n  pages = {074111},\n  year = {2015},\n  doi = {10.1063/1.4907719},\n  url = {https://doi.org/10.1063/1.4907719}\n}"
};


func_reference_type xc_ref_Mardirossian2016_214110 = {
  "N. Mardirossian and M. Head-Gordon, J. Chem. Phys. 144, 214110 (2016)",
  "10.1063/1.4952647",
  "@article{Mardirossian2016_214110,\n  author = \"Mardirossian, Narbe and Head-Gordon, Martin\",\n  title = \"$\\omega$B97M-V: A combinatorially optimized, range-separated hybrid, meta-GGA density functional with VV10 nonlocal correlation\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2016\",\n  volume = \"144\",\n  number = \"21\",\n  pages = \"214110\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/144/21/10.1063/1.4952647\",\n  doi = \"10.1063/1.4952647\"\n}"
};


func_reference_type xc_ref_Margraf2019_244116 = {
  "J. T. Margraf, C. Kunkel, and K. Reuter, J. Chem. Phys. 150, 244116 (2019)",
  "10.1063/1.5094788",
  "@article{Margraf2019_244116,\n  author = {Margraf, Johannes T. and Kunkel, Christian and Reuter, Karsten},\n  title = {Towards density functional approximations from coupled cluster correlation energy densities},\n  journal = {J. Chem. Phys.},\n  volume = {150},\n  number = {24},\n  pages = {244116},\n  year = {2019},\n  doi = {10.1063/1.5094788},\n  url = {https://doi.org/10.1063/1.5094788}\n}"
};


func_reference_type xc_ref_Marques2012_2272 = {
  "M. A. Marques, M. J. Oliveira, and T. Burnus, Comput. Phys. Commun. 183, 2272 (2012)",
  "10.1016/j.cpc.2012.05.007",
  "@article{Marques2012_2272,\n  title = \"Libxc: A library of exchange and correlation functionals for density functional theory\",\n  journal = \"Comput. Phys. Commun.\",\n  volume = \"183\",\n  number = \"10\",\n  pages = \"2272--2281\",\n  year = \"2012\",\n  issn = \"0010-4655\",\n  doi = \"10.1016/j.cpc.2012.05.007\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0010465512001750\",\n  author = \"Miguel A.L. Marques and Micael J.T. Oliveira and Tobias Burnus\"\n}"
};


func_reference_type xc_ref_Mattsson2008_084714 = {
  "A. E. Mattsson, R. Armiento, J. Paier, G. Kresse, J. M. Wills, and T. R. Mattsson, J. Chem. Phys. 128, 084714 (2008)",
  "10.1063/1.2835596",
  "@article{Mattsson2008_084714,\n  title = \"The AM05 density functional applied to solids\",\n  author = \"A. E. Mattsson and R. Armiento and J. Paier and G. Kresse and J. M. Wills and T. R. Mattsson\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2008\",\n  volume = \"128\",\n  number = \"8\",\n  pages = 084714,\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/128/8/10.1063/1.2835596\",\n  doi = \"10.1063/1.2835596\"\n}"
};


func_reference_type xc_ref_McWeeny1976_3 = {
  "R. McWeeny, in The New World of Quantum Chemistry, edited by B. Pullman and R. Parr (Reidel, Boston, 1976) pp. 3--31",
  "10.1007/978-94-010-1523-3_1",
  "@inproceedings{McWeeny1976_3,\n  author = {R. McWeeny},\n  title = {Present Status of the Correlation Problem},\n  booktitle = {The New World of Quantum Chemistry},\n  year = {1976},\n  editor = {B. Pullman and R. Parr},\n  pages = {3--31},\n  address = {Boston},\n  publisher = {Reidel},\n  doi = {10.1007/978-94-010-1523-3_1}\n}"
};


func_reference_type xc_ref_Mejia2017_052512 = {
  "D. Mejia-Rodriguez and S. B. Trickey, Phys. Rev. A 96, 052512 (2017)",
  "10.1103/PhysRevA.96.052512",
  "@article{Mejia2017_052512,\n  title = {Deorbitalization strategies for meta-generalized-gradient-approximation exchange-correlation functionals},\n  author = {Mejia-Rodriguez, Daniel and Trickey, S. B.},\n  journal = {Phys. Rev. A},\n  volume = {96},\n  issue = {5},\n  pages = {052512},\n  year = {2017},\n  month = {Nov},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.96.052512},\n  url = {https://link.aps.org/doi/10.1103/PhysRevA.96.052512}\n}"
};


func_reference_type xc_ref_Mejia2018_115161 = {
  "D. Mejia-Rodriguez and S. B. Trickey, Phys. Rev. B 98, 115161 (2018)",
  "10.1103/PhysRevB.98.115161",
  "@article{Mejia2018_115161,\n  title = {Deorbitalized meta-GGA exchange-correlation functionals in solids},\n  author = {Mejia-Rodriguez, Daniel and Trickey, S. B.},\n  journal = {Phys. Rev. B},\n  volume = {98},\n  issue = {11},\n  pages = {115161},\n  year = {2018},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.98.115161},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.98.115161}\n}"
};


func_reference_type xc_ref_Mejia2020_121109 = {
  "D. Mejia-Rodriguez and S. B. Trickey, Phys. Rev. B 102, 121109 (2020)",
  "10.1103/PhysRevB.102.121109",
  "@article{Mejia2020_121109,\n  title = {Meta-GGA performance in solids at almost GGA cost},\n  author = {Mej\\'{i}a-Rodr\\'{i}guez, Daniel and Trickey, S. B.},\n  journal = {Phys. Rev. B},\n  volume = {102},\n  issue = {12},\n  pages = {121109},\n  numpages = {4},\n  year = {2020},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.102.121109},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.102.121109}\n}"
};


func_reference_type xc_ref_Menconi2001_3958 = {
  "G. Menconi, P. J. Wilson, and D. J. Tozer, J. Chem. Phys. 114, 3958 (2001)",
  "10.1063/1.1342776",
  "@article{Menconi2001_3958,\n  title = \"Emphasizing the exchange-correlation potential in functional development\",\n  author = \"G. Menconi and P. J. Wilson and D. J. Tozer\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2001\",\n  volume = \"114\",\n  number = \"9\",\n  pages = \"3958\",\n  doi = \"10.1063/1.1342776\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/114/9/10.1063/1.1342776\"\n}"
};


func_reference_type xc_ref_Meyer1976_898 = {
  "A. Meyer, G. C. Wang, and W. H. Young, Z. Naturforsch. A 31, 898 (1976)",
  "10.1515/zna-1976-0804",
  "@article{Meyer1976_898,\n  title = {The von Weizs\\\"acker Coefficient in Density Functional Theory},\n  author = {A. Meyer and G. C. Wang and W. H. Young},\n  journal = {Z. Naturforsch. A},\n  year = {1976},\n  volume =  {31},\n  pages = {898},\n  url = {http://doi.org/10.1515/zna-1976-0804},\n  doi = {10.1515/zna-1976-0804}\n}"
};


func_reference_type xc_ref_Mezei2018_2469 = {
  "P. D. Mezei, G. I. Csonka, and M. Kallay, J. Chem. Theory Comput. 14, 2469 (2018)",
  "10.1021/acs.jctc.8b00072",
  "@article{Mezei2018_2469,\n  author = {Mezei, P\\'{a}l D. and Csonka, G\\'{a}bor I. and K\\'{a}llay, Mih\\'{a}ly},\n  title = {Simple Modifications of the SCAN Meta-Generalized Gradient Approximation Functional},\n  journal = {J. Chem. Theory Comput.},\n  volume = {14},\n  number = {5},\n  pages = {2469-2479},\n  year = {2018},\n  doi = {10.1021/acs.jctc.8b00072},\n  url = {https://doi.org/10.1021/acs.jctc.8b00072}\n}"
};


func_reference_type xc_ref_Miehlich1989_200 = {
  "B. Miehlich, A. Savin, H. Stoll, and H. Preuss, Chem. Phys. Lett. 157, 200 (1989)",
  "10.1016/0009-2614(89)87234-3",
  "@article{Miehlich1989_200,\n  title = \"Results obtained with the correlation energy density functionals of becke and Lee, Yang and Parr\",\n  author = \"B. Miehlich and A. Savin and H. Stoll and H. Preuss\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"157\",\n  number = \"3\",\n  pages = \"200\",\n  year = \"1989\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/0009-2614(89)87234-3\",\n  url = \"http://www.sciencedirect.com/science/article/pii/0009261489872343\"\n}"
};


func_reference_type xc_ref_Mortensen2005_216401 = {
  "J. J. Mortensen, K. Kaasbjerg, S. L. Frederiksen, J. K. Norskov, J. P. Sethna, and K. W. Jacobsen, Phys. Rev. Lett. 95, 216401 (2005)",
  "10.1103/PhysRevLett.95.216401",
  "@article{Mortensen2005_216401,\n  title = {Bayesian Error Estimation in Density-Functional Theory},\n  author = {J. J. Mortensen and K. Kaasbjerg and S. L. Frederiksen and J. K. N\\o{}rskov and J. P. Sethna and K. W. Jacobsen},\n  journal = {Phys. Rev. Lett.},\n  volume = {95},\n  issue = {21},\n  pages = {216401},\n  year = {2005},\n  month = {Nov},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.95.216401},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.95.216401}\n}"
};


func_reference_type xc_ref_Mosquera2016_1605 = {
  "M. A. Mosquera, C. H. Borca, M. A. Ratner, and G. C. Schatz, J. Phys. Chem. A 120, 1605 (2016)",
  "10.1021/acs.jpca.5b10864",
  "@article{Mosquera2016_1605,\n  author = {Mosquera, Mart\\'{i}n A. and Borca, Carlos H. and Ratner, Mark A. and Schatz, George C.},\n  title = {Connection between Hybrid Functionals and Importance of the Local Density Approximation},\n  journal = {J. Phys. Chem. A},\n  volume = {120},\n  number = {9},\n  pages = {1605-1612},\n  year = {2016},\n  doi = {10.1021/acs.jpca.5b10864},\n  url = {https://doi.org/10.1021/acs.jpca.5b10864}\n}"
};


func_reference_type xc_ref_Moussa2012_204117 = {
  "J. E. Moussa, P. A. Schultz, and J. R. Chelikowsky, J. Chem. Phys. 136, 204117 (2012)",
  "10.1063/1.4722993",
  "@article{Moussa2012_204117,\n  author = \"Moussa, Jonathan E. and Schultz, Peter A. and Chelikowsky, James R.\",\n  title = \"Analysis of the Heyd-Scuseria-Ernzerhof density functional parameter space\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"136\",\n  number = \"20\",\n  pages = \"204117\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/136/20/10.1063/1.4722993\",\n  doi = \"10.1063/1.4722993\"\n}"
};


func_reference_type xc_ref_Murray2009_2754 = {
  "E. D. Murray, K. Lee, and D. C. Langreth, J. Chem. Theory Comput. 5, 2754 (2009)",
  "10.1021/ct900365q",
  "@article{Murray2009_2754,\n  title = {Investigation of Exchange Energy Density Functional Accuracy for Interacting Molecules},\n  author = {\\'{E}. D. Murray and K. Lee and D. C. Langreth},\n  journal = {J. Chem. Theory Comput.},\n  volume = {5},\n  number = {10},\n  pages = {2754},\n  year = {2009},\n  doi = {10.1021/ct900365q},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct900365q},\n}"
};


func_reference_type xc_ref_Negele1972_1472 = {
  "J. W. Negele and D. Vautherin, Phys. Rev. C 5, 1472 (1972)",
  "10.1103/PhysRevC.5.1472",
  "@article{Negele1972_1472,\n  title = {Density-Matrix Expansion for an Effective Nuclear Hamiltonian},\n  author = {Negele, J. W. and Vautherin, D.},\n  journal = {Phys. Rev. C},\n  volume = {5},\n  issue = {5},\n  pages = {1472--1493},\n  year = {1972},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevC.5.1472},\n  url = {https://link.aps.org/doi/10.1103/PhysRevC.5.1472}\n}"
};


func_reference_type xc_ref_Neupane2021_063803 = {
  "B. Neupane, H. Tang, N. K. Nepal, S. Adhikari, and A. Ruzsinszky, Phys. Rev. Materials 5, 063803 (2021)",
  "10.1103/PhysRevMaterials.5.063803",
  "@article{Neupane2021_063803,\n  title = {Opening band gaps of low-dimensional materials at the meta-{GGA} level of density functional approximations},\n  author = {Neupane, Bimal and Tang, Hong and Nepal, Niraj K. and Adhikari, Santosh and Ruzsinszky, Adrienn},\n  journal = {Phys. Rev. Materials},\n  volume = {5},\n  issue = {6},\n  pages = {063803},\n  numpages = {12},\n  year = {2021},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevMaterials.5.063803},\n  url = {https://link.aps.org/doi/10.1103/PhysRevMaterials.5.063803}\n}"
};


func_reference_type xc_ref_Odashima2009_798 = {
  "M. M. Odashima, K. Capelle, and S. B. Trickey, J. Chem. Theory Comput. 5, 798 (2009)",
  "10.1021/ct8005634",
  "@article{Odashima2009_798,\n  author = {Odashima, Mariana M. and Capelle, K. and Trickey, S. B.},\n  title = {Tightened Lieb-Oxford Bound for Systems of Fixed Particle Number},\n  journal = {J. Chem. Theory Comput.},\n  volume = {5},\n  number = {4},\n  pages = {798-807},\n  year = {2009},\n  doi = {10.1021/ct8005634},\n  url = {http://doi.org/10.1021/ct8005634},\n}"
};


func_reference_type xc_ref_Okuno2012_29 = {
  "K. Okuno, Y. Shigeta, R. Kishi, H. Miyasaka, and M. Nakano, J. Photochem. Photobiol., A 235, 29 (2012)",
  "10.1016/j.jphotochem.2012.03.003",
  "@article{Okuno2012_29,\n  title = \"Tuned CAM-B3LYP functional in the time-dependent density functional theory scheme for excitation energies and properties of diarylethene derivatives\",\n  author = \"K. Okuno and Y. Shigeta and R. Kishi and H. Miyasaka and M. Nakano\",\n  journal = \"J. Photochem. Photobiol., A\",\n  volume = \"235\",\n  pages = \"29\",\n  year = \"2012\",\n  issn = \"1010-6030\",\n  doi = \"10.1016/j.jphotochem.2012.03.003\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S101060301200130X\"\n}"
};


func_reference_type xc_ref_Ortiz1994_1391 = {
  "G. Ortiz and P. Ballone, Phys. Rev. B 50, 1391 (1994)",
  "10.1103/PhysRevB.50.1391",
  "@article{Ortiz1994_1391,\n  title = {Correlation energy, structure factor, radial distribution function, and momentum distribution of the spin-polarized uniform electron gas},\n  author = {G. Ortiz and P. Ballone},\n  journal = {Phys. Rev. B},\n  volume = {50},\n  issue = {3},\n  pages = {1391},\n  year = {1994},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.50.1391},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.50.1391}\n}"
};


func_reference_type xc_ref_Ortiz1994_1391_err = {
  "G. Ortiz and P. Ballone, Phys. Rev. B 56, 9970 (1997)",
  "10.1103/PhysRevB.56.9970",
  "@article{Ortiz1994_1391_err,\n  title = {Erratum: Correlation energy, structure factor, radial distribution function, and momentum distribution of the spin-polarized uniform electron gas [Phys. Rev. B <b>50</b>, 1391 (1994)]},\n  author = {G. Ortiz and P. Ballone},\n  journal = {Phys. Rev. B},\n  volume = {56},\n  issue = {15},\n  pages = {9970},\n  year = {1997},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.56.9970},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.56.9970}\n}"
};


func_reference_type xc_ref_OuYang1991_379 = {
  "H. Ou-Yang and M. Levy, Int. J. Quantum Chem. 40, 379 (1991)",
  "10.1002/qua.560400309",
  "@article{OuYang1991_379,\n  title = {Approximate noninteracting kinetic energy functionals from a nonuniform scaling requirement},\n  author = {H. Ou-Yang and M. Levy},\n  journal = {Int. J. Quantum Chem.},\n  volume = {40},\n  number = {3},\n  pages = {379},\n  year = {1991},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  doi = {10.1002/qua.560400309},\n  url = {http://onlinelibrary.wiley.com/doi/10.1002/qua.560400309/abstract}\n}"
};


func_reference_type xc_ref_PachecoKato2016_268 = {
  "J. C. Pacheco-Kato, J. M. del Campo, J. L. Gazquez, S. Trickey, and A. Vela, Chem. Phys. Lett. 651, 268 (2016)",
  "10.1016/j.cplett.2016.03.028",
  "@article{PachecoKato2016_268,\n  title = \"A PW91-like exchange with a simple analytical form\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"651\",\n  pages = \"268-273\",\n  year = \"2016\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2016.03.028\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261416301373\",\n  author = \"Juan C. Pacheco-Kato and Jorge M. del Campo and Jos\\'{e} L. G\\'{a}zquez and S.B. Trickey and Alberto Vela\"\n}"
};


func_reference_type xc_ref_Patra2018_8991 = {
  "B. Patra, S. Jana, and P. Samal, Phys. Chem. Chem. Phys. 20, 8991 (2018)",
  "10.1039/C8CP00717A",
  "@article{Patra2018_8991,\n  author = \"Patra, Bikash and Jana, Subrata and Samal, Prasanjit\",\n  title = \"Long-range corrected density functional through the density matrix expansion based semilocal exchange hole\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2018\",\n  volume = \"20\",\n  issue = \"13\",\n  pages = \"8991-8998\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C8CP00717A\",\n  url = \"http://doi.org/10.1039/C8CP00717A\",\n}"
};


func_reference_type xc_ref_Patra2019_045147 = {
  "B. Patra, S. Jana, L. A. Constantin, and P. Samal, Phys. Rev. B 100, 045147 (2019)",
  "10.1103/PhysRevB.100.045147",
  "@article{Patra2019_045147,\n  title = {Efficient band gap prediction of semiconductors and insulators from a semilocal exchange-correlation functional},\n  author = {Patra, Bikash and Jana, Subrata and Constantin, Lucian A. and Samal, Prasanjit},\n  journal = {Phys. Rev. B},\n  volume = {100},\n  issue = {4},\n  pages = {045147},\n  year = {2019},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.100.045147},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.100.045147}\n}"
};


func_reference_type xc_ref_Patra2019_155140 = {
  "B. Patra, S. Jana, L. A. Constantin, and P. Samal, Phys. Rev. B 100, 155140 (2019)",
  "10.1103/PhysRevB.100.155140",
  "@article{Patra2019_155140,\n  title = {Relevance of the Pauli kinetic energy density for semilocal functionals},\n  author = {Patra, Bikash and Jana, Subrata and Constantin, Lucian A. and Samal, Prasanjit},\n  journal = {Phys. Rev. B},\n  volume = {100},\n  issue = {15},\n  pages = {155140},\n  year = {2019},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.100.155140},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.100.155140}\n}"
};


func_reference_type xc_ref_Patra2019_19639 = {
  "A. Patra, S. Jana, H. Myneni, and P. Samal, Phys. Chem. Chem. Phys. 21, 19639 (2019)",
  "10.1039/C9CP03356D",
  "@Article{Patra2019_19639,\n  author = \"Patra, Abhilash and Jana, Subrata and Myneni, Hemanadhan and Samal, Prasanjit\",\n  title = \"Laplacian free and asymptotic corrected semilocal exchange potential applied to the band gap of solids\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2019\",\n  volume = \"21\",\n  issue = \"35\",\n  pages = \"19639-19650\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C9CP03356D\",\n  url = \"http://doi.org/10.1039/C9CP03356D\",\n}"
};


func_reference_type xc_ref_Patra2020_184112 = {
  "A. Patra, S. Jana, and P. Samal, J. Chem. Phys. 153, 184112 (2020)",
  "10.1063/5.0025173",
  "@article{Patra2020_184112,\n  author = {Patra, Abhilash and Jana, Subrata and Samal, Prasanjit },\n  title = {A way of resolving the order-of-limit problem of Tao--Mo semilocal functional},\n  journal = {J. Chem. Phys.},\n  volume = {153},\n  number = {18},\n  pages = {184112},\n  year = {2020},\n  doi = {10.1063/5.0025173},\n  url = {https://doi.org/10.1063/5.0025173}\n}"
};


func_reference_type xc_ref_Paziani2006_155111 = {
  "S. Paziani, S. Moroni, P. Gori-Giorgi, and G. B. Bachelet, Phys. Rev. B 73, 155111 (2006)",
  "10.1103/PhysRevB.73.155111",
  "@article{Paziani2006_155111,\n  title = {Local-spin-density functional for multideterminant density functional theory},\n  author = {Paziani, Simone and Moroni, Saverio and Gori-Giorgi, Paola and Bachelet, Giovanni B.},\n  journal = {Phys. Rev. B},\n  volume = {73},\n  issue = {15},\n  pages = {155111},\n  year = {2006},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.73.155111},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.73.155111}\n}"
};


func_reference_type xc_ref_Pearson1983 = {
  "E. W. Pearson, Theory and application of the electron gas model, Ph.D. thesis, Harvard University (1983)",
  "",
  "@PhdThesis{Pearson1983,\n  title = 	 {Theory and application of the electron gas model},\n  author = 	 {E. W. Pearson},\n  school = 	 {Harvard University},\n  year = 	 {1983},\n  url  =      {http://discovery.lib.harvard.edu/?itemid=|library/m/aleph|001176470}\n}"
};


func_reference_type xc_ref_Pearson1985_881 = {
  "E. W. Pearson and R. G. Gordon, J. Chem. Phys. 82, 881 (1985)",
  "10.1063/1.448516",
  "@article{Pearson1985_881,\n  title = \"Local asymptotic gradient corrections to the energy functional of an electron gas\",\n  author = \"E. W. Pearson and R. G. Gordon\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1985\",\n  volume = \"82\",\n  number = \"2\",\n  pages = \"881\",\n  doi = \"10.1063/1.448516\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/82/2/10.1063/1.448516\"\n}"
};


func_reference_type xc_ref_Pedroza2009_201106 = {
  "L. S. Pedroza, A. J. R. da Silva, and K. Capelle, Phys. Rev. B 79, 201106 (2009)",
  "10.1103/PhysRevB.79.201106",
  "@article{Pedroza2009_201106,\n  title = {Gradient-dependent density functionals of the Perdew--Burke--Ernzerhof type for atoms, molecules, and solids},\n  author = {L. S. Pedroza and A. J. R. da Silva and K. Capelle},\n  journal = {Phys. Rev. B},\n  volume = {79},\n  issue = {20},\n  pages = {201106},\n  year = {2009},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.79.201106},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.79.201106}\n}"
};


func_reference_type xc_ref_Peng2016_041005 = {
  "H. Peng, Z.-H. Yang, J. P. Perdew, and J. Sun, Phys. Rev. X 6, 041005 (2016)",
  "10.1103/PhysRevX.6.041005",
  "@article{Peng2016_041005,\n  title = {Versatile van der Waals Density Functional Based on a Meta-Generalized Gradient Approximation},\n  author = {Peng, Haowei and Yang, Zeng-Hui and Perdew, John P. and Sun, Jianwei},\n  journal = {Phys. Rev. X},\n  volume = {6},\n  issue = {4},\n  pages = {041005},\n  year = {2016},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevX.6.041005},\n  url = {https://link.aps.org/doi/10.1103/PhysRevX.6.041005}\n}"
};


func_reference_type xc_ref_Perdew1981_5048 = {
  "J. P. Perdew and A. Zunger, Phys. Rev. B 23, 5048 (1981)",
  "10.1103/PhysRevB.23.5048",
  "@article{Perdew1981_5048,\n  title = {Self-interaction correction to density-functional approximations for many-electron systems},\n  author = {J. P. Perdew and A. Zunger},\n  journal = {Phys. Rev. B},\n  volume = {23},\n  issue = {10},\n  pages = {5048},\n  year = {1981},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.23.5048},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.23.5048}\n}"
};


func_reference_type xc_ref_Perdew1981_5048_mod = {
  "J. P. Perdew and A. Zunger, Phys. Rev. B 23, 5048 (1981), modified to improve the matching between the low- and high-rs parts",
  "10.1103/PhysRevB.23.5048",
  "@article{Perdew1981_5048_mod,\n  title = {Self-interaction correction to density-functional approximations for many-electron systems},\n  author = {J. P. Perdew and A. Zunger},\n  journal = {Phys. Rev. B},\n  volume = {23},\n  issue = {10},\n  pages = {5048},\n  year = {1981},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.23.5048},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.23.5048},\n  note = {Modified to improve the matching between the low- and high-rs parts}\n}"
};


func_reference_type xc_ref_Perdew1986_8800 = {
  "J. P. Perdew and W. Yue, Phys. Rev. B 33, 8800 (1986)",
  "10.1103/PhysRevB.33.8800",
  "@article{Perdew1986_8800,\n  title = {Accurate and simple density functional for the electronic exchange energy: Generalized gradient approximation},\n  author = {J. P. Perdew and W. Yue},\n  journal = {Phys. Rev. B},\n  volume = {33},\n  issue = {12},\n  pages = {8800},\n  year = {1986},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.33.8800},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.33.8800}\n}"
};


func_reference_type xc_ref_Perdew1986_8822 = {
  "J. P. Perdew, Phys. Rev. B 33, 8822 (1986)",
  "10.1103/PhysRevB.33.8822",
  "@article{Perdew1986_8822,\n  title = {Density-functional approximation for the correlation energy of the inhomogeneous electron gas},\n  author = {J. P. Perdew},\n  journal = {Phys. Rev. B},\n  volume = {33},\n  issue = {12},\n  pages = {8822},\n  year = {1986},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.33.8822},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.33.8822}\n}"
};


func_reference_type xc_ref_Perdew1991 = {
  "J. P. Perdew, in Proceedings of the 75. WE-Heraeus-Seminar and 21st Annual International Symposium on Electronic Structure of Solids, edited by P. Ziesche and H. Eschrig (Akademie Verlag, Berlin, 1991) p. 11",
  "",
  "@inproceedings{Perdew1991,\n  title = {},\n  author = {J. P. Perdew},\n  booktitle = {Proceedings of the 75. WE-Heraeus-Seminar and 21st Annual International Symposium on Electronic Structure of Solids},\n  pages = {11},\n  year = {1991},\n  editor = {P. Ziesche and H. Eschrig},\n  address = {Berlin},\n  publisher = {Akademie Verlag},\n}"
};


func_reference_type xc_ref_Perdew1992_13244 = {
  "J. P. Perdew and Y. Wang, Phys. Rev. B 45, 13244 (1992)",
  "10.1103/PhysRevB.45.13244",
  "@article{Perdew1992_13244,\n  title = {Accurate and simple analytic representation of the electron-gas correlation energy},\n  author = {J. P. Perdew and Y. Wang},\n  journal = {Phys. Rev. B},\n  volume = {45},\n  issue = {23},\n  pages = {13244},\n  year = {1992},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.45.13244},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.45.13244}\n}"
};


func_reference_type xc_ref_Perdew1992_13244_mod = {
  "J. P. Perdew and Y. Wang, Phys. Rev. B 45, 13244 (1992), added extra digits to some constants as in the PBE routine (http://dft.rutgers.edu/pubs/PBE.asc)",
  "10.1103/PhysRevB.45.13244",
  "@article{Perdew1992_13244_mod,\n  title = {Accurate and simple analytic representation of the electron-gas correlation energy},\n  author = {J. P. Perdew and Y. Wang},\n  journal = {Phys. Rev. B},\n  volume = {45},\n  issue = {23},\n  pages = {13244},\n  year = {1992},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.45.13244},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.45.13244},\n  note = {Added extra digits to some constants as in the PBE routine (http://dft.rutgers.edu/pubs/PBE.asc)}\n}"
};


func_reference_type xc_ref_Perdew1992_6671 = {
  "J. P. Perdew, J. A. Chevary, S. H. Vosko, K. A. Jackson, M. R. Pederson, D. J. Singh, and C. Fiolhais, Phys. Rev. B 46, 6671 (1992)",
  "10.1103/PhysRevB.46.6671",
  "@article{Perdew1992_6671,\n  title = {Atoms, molecules, solids, and surfaces: Applications of the generalized gradient approximation for exchange and correlation},\n  author = {J. P. Perdew and J. A. Chevary and S. H. Vosko and K. A. Jackson and M. R. Pederson and D. J. Singh and C. Fiolhais},\n  journal = {Phys. Rev. B},\n  volume = {46},\n  issue = {11},\n  pages = {6671},\n  year = {1992},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.46.6671},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.46.6671}\n}"
};


func_reference_type xc_ref_Perdew1992_6671_err = {
  "J. P. Perdew, J. A. Chevary, S. H. Vosko, K. A. Jackson, M. R. Pederson, D. J. Singh, and C. Fiolhais, Phys. Rev. B 48, 4978 (1993)",
  "10.1103/PhysRevB.48.4978.2",
  "@article{Perdew1992_6671_err,\n  title = {Erratum: Atoms, molecules, solids, and surfaces: Applications of the generalized gradient approximation for exchange and correlation},\n  author = {J. P. Perdew and J. A. Chevary and S. H. Vosko and K. A. Jackson and M. R. Pederson and D. J. Singh and C. Fiolhais},\n  journal = {Phys. Rev. B},\n  volume = {48},\n  issue = {7},\n  pages = {4978},\n  year = {1993},\n  month = {Aug},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.48.4978.2},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.48.4978.2}\n}"
};


func_reference_type xc_ref_Perdew1992_79 = {
  "J. P. Perdew, Phys. Lett. A 165, 79 (1992)",
  "10.1016/0375-9601(92)91058-Y",
  "@article{Perdew1992_79,\n  title = \"Generalized gradient approximation for the fermion kinetic energy as a functional of the density\",\n  author = \"J. P. Perdew\",\n  journal = \"Phys. Lett. A\",\n  volume = \"165\",\n  number = \"1\",\n  pages = \"79\",\n  year = \"1992\",\n  issn = \"0375-9601\",\n  doi = \"10.1016/0375-9601(92)91058-Y\",\n  url = \"http://www.sciencedirect.com/science/article/pii/037596019291058Y\",\n}"
};


func_reference_type xc_ref_Perdew1996_3865 = {
  "J. P. Perdew, K. Burke, and M. Ernzerhof, Phys. Rev. Lett. 77, 3865 (1996)",
  "10.1103/PhysRevLett.77.3865",
  "@article{Perdew1996_3865,\n  title = {Generalized Gradient Approximation Made Simple},\n  author = {J. P. Perdew and K. Burke and M. Ernzerhof},\n  journal = {Phys. Rev. Lett.},\n  volume = {77},\n  issue = {18},\n  pages = {3865},\n  year = {1996},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.77.3865},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.77.3865}\n}"
};


func_reference_type xc_ref_Perdew1996_3865_err = {
  "J. P. Perdew, K. Burke, and M. Ernzerhof, Phys. Rev. Lett. 78, 1396 (1997)",
  "10.1103/PhysRevLett.78.1396",
  "@article{Perdew1996_3865_err,\n  title = {Errata: Generalized Gradient Approximation Made Simple [Phys. Rev. Lett. 77, 3865 (1996)]},\n  author = {J. P. Perdew and K. Burke and M. Ernzerhof},\n  journal = {Phys. Rev. Lett.},\n  volume = {78},\n  issue = {7},\n  pages = {1396},\n  year = {1997},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.78.1396},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.78.1396}\n}"
};


func_reference_type xc_ref_Perdew1999_2544 = {
  "J. P. Perdew, S. Kurth, A. Zupan, and P. Blaha, Phys. Rev. Lett. 82, 2544 (1999)",
  "10.1103/PhysRevLett.82.2544",
  "@article{Perdew1999_2544,\n  title = {Accurate Density Functional with Correct Formal Properties: A Step Beyond the Generalized Gradient Approximation},\n  author = {J. P. Perdew and S. Kurth and A. Zupan and P. Blaha},\n  journal = {Phys. Rev. Lett.},\n  volume = {82},\n  issue = {12},\n  pages = {2544},\n  year = {1999},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.82.2544},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.82.2544}\n}"
};


func_reference_type xc_ref_Perdew2004_6898 = {
  "J. P. Perdew, J. Tao, V. N. Staroverov, and G. E. Scuseria, J. Chem. Phys. 120, 6898 (2004)",
  "10.1063/1.1665298",
  "@article{Perdew2004_6898,\n  title = \"Meta-generalized gradient approximation: Explanation of a realistic nonempirical density functional\",\n  author = \"J. P. Perdew and J. Tao and V. N. Staroverov and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2004\",\n  volume = \"120\",\n  number = \"15\",\n  pages = \"6898\",\n  doi = \"10.1063/1.1665298\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/120/15/10.1063/1.1665298\"\n}"
};


func_reference_type xc_ref_Perdew2007_042506 = {
  "J. P. Perdew, A. Ruzsinszky, J. Tao, G. I. Csonka, and G. E. Scuseria, Phys. Rev. A 76, 042506 (2007)",
  "10.1103/PhysRevA.76.042506",
  "@article{Perdew2007_042506,\n  title = {One-parameter optimization of a nonempirical meta-generalized-gradient-approximation for the exchange-correlation energy},\n  author = {J. P. Perdew and A. Ruzsinszky and J. Tao and G. I. Csonka and G. E. Scuseria},\n  journal = {Phys. Rev. A},\n  volume = {76},\n  issue = {4},\n  pages = {042506},\n  year = {2007},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.76.042506},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.76.042506}\n}"
};


func_reference_type xc_ref_Perdew2007_155109 = {
  "J. P. Perdew and L. A. Constantin, Phys. Rev. B 75, 155109 (2007)",
  "10.1103/PhysRevB.75.155109",
  "@article{Perdew2007_155109,\n  title = {Laplacian-level density functionals for the kinetic energy density and exchange-correlation energy},\n  author = {Perdew, John P. and Constantin, Lucian A.},\n  journal = {Phys. Rev. B},\n  volume = {75},\n  issue = {15},\n  pages = {155109},\n  year = {2007},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.75.155109},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.75.155109}\n}"
};


func_reference_type xc_ref_Perdew2008_136406 = {
  "J. P. Perdew, A. Ruzsinszky, G. I. Csonka, O. A. Vydrov, G. E. Scuseria, L. A. Constantin, X. Zhou, and K. Burke, Phys. Rev. Lett. 100, 136406 (2008)",
  "10.1103/PhysRevLett.100.136406",
  "@article{Perdew2008_136406,\n  title = {Restoring the Density-Gradient Expansion for Exchange in Solids and Surfaces},\n  author = {J. P. Perdew and A. Ruzsinszky and G. I. Csonka and O. A. Vydrov and G. E. Scuseria and L. A. Constantin and X. Zhou and K. Burke},\n  journal = {Phys. Rev. Lett.},\n  volume = {100},\n  issue = {13},\n  pages = {136406},\n  year = {2008},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.100.136406},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.100.136406}\n}"
};


func_reference_type xc_ref_Perdew2009_026403 = {
  "J. P. Perdew, A. Ruzsinszky, G. I. Csonka, L. A. Constantin, and J. Sun, Phys. Rev. Lett. 103, 026403 (2009)",
  "10.1103/PhysRevLett.103.026403",
  "@article{Perdew2009_026403,\n  title = {Workhorse Semilocal Density Functional for Condensed Matter Physics and Quantum Chemistry},\n  author = {J. P. Perdew and A. Ruzsinszky and G. I. Csonka and L. A. Constantin and J. Sun},\n  journal = {Phys. Rev. Lett.},\n  volume = {103},\n  issue = {2},\n  pages = {026403},\n  year = {2009},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.103.026403},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.103.026403}\n}"
};


func_reference_type xc_ref_Perdew2009_026403_err = {
  "J. P. Perdew, A. Ruzsinszky, G. I. Csonka, L. A. Constantin, and J. Sun, Phys. Rev. Lett. 106, 179902 (2011)",
  "10.1103/PhysRevLett.106.179902",
  "@article{Perdew2009_026403_err,\n  title = {Erratum: Workhorse Semilocal Density Functional for Condensed Matter Physics and Quantum Chemistry [Phys. Rev. Lett. 103, 026403 (2009)]},\n  author = {J. P. Perdew and A. Ruzsinszky and G. I. Csonka and L. A. Constantin and J. Sun},\n  journal = {Phys. Rev. Lett.},\n  volume = {106},\n  issue = {17},\n  pages = {179902},\n  year = {2011},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.106.179902},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.106.179902}\n}"
};


func_reference_type xc_ref_Perez2015_3844 = {
  "R. Sarmiento-Perez, S. Botti, and M. A. L. Marques, J. Chem. Theory Comput. 11, 3844 (2015)",
  "10.1021/acs.jctc.5b00529",
  "@article{Perez2015_3844,\n  author = {Rafael Sarmiento-P\\'{e}rez and Silvana Botti and Miguel A. L. Marques},\n  title = {Optimized Exchange and Correlation Semilocal Functional for the Calculation of Energies of Formation},\n  journal = {J. Chem. Theory Comput.},\n  volume = {11},\n  number = {8},\n  pages = {3844-3850},\n  year = {2015},\n  doi = {10.1021/acs.jctc.5b00529},\n  url = {http://doi.org/10.1021/acs.jctc.5b00529},\n}"
};


func_reference_type xc_ref_Pernal2009_263201 = {
  "K. Pernal, R. Podeszwa, K. Patkowski, and K. Szalewicz, Phys. Rev. Lett. 103, 263201 (2009)",
  "10.1103/PhysRevLett.103.263201",
  "@article{Pernal2009_263201,\n  title = {Dispersionless Density Functional Theory},\n  author = {Pernal, Katarzyna and Podeszwa, Rafa\\l{} and Patkowski, Konrad and Szalewicz, Krzysztof},\n  journal = {Phys. Rev. Lett.},\n  volume = {103},\n  issue = {26},\n  pages = {263201},\n  year = {2009},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.103.263201},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.103.263201}\n}"
};


func_reference_type xc_ref_Peverati2011_191102 = {
  "R. Peverati and D. G. Truhlar, J. Chem. Phys. 135, 191102 (2011)",
  "10.1063/1.3663871",
  "@article{Peverati2011_191102,\n  title = \"Communication: A global hybrid generalized gradient approximation to the exchange-correlation functional that satisfies the second-order density-gradient constraint and has broad applicability in chemistry\",\n  author = \"R. Peverati and D. G. Truhlar\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2011\",\n  volume = \"135\",\n  number = \"19\",\n  pages = 191102,\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/135/19/10.1063/1.3663871\",\n  doi = \"10.1063/1.3663871\"\n}"
};


func_reference_type xc_ref_Peverati2011_1991 = {
  "R. Peverati, Y. Zhao, and D. G. Truhlar, J. Phys. Chem. Lett. 2, 1991 (2011)",
  "10.1021/jz200616w",
  "@article{Peverati2011_1991,\n  title = {Generalized Gradient Approximation That Recovers the Second-Order Density-Gradient Expansion with Optimized Across-the-Board Performance},\n  author = {R. Peverati and Y. Zhao and D. G. Truhlar},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {2},\n  number = {16},\n  pages = {1991},\n  year = {2011},\n  doi = {10.1021/jz200616w},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jz200616w},\n}"
};


func_reference_type xc_ref_Peverati2011_2810 = {
  "R. Peverati and D. G. Truhlar, J. Phys. Chem. Lett. 2, 2810 (2011)",
  "10.1021/jz201170d",
  "@article{Peverati2011_2810,\n  title = {Improving the Accuracy of Hybrid Meta-GGA Density Functionals by Range Separation},\n  author = {R. Peverati and D. G. Truhlar},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {2},\n  number = {21},\n  pages = {2810},\n  year = {2011},\n  doi = {10.1021/jz201170d},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jz201170d}\n}"
};


func_reference_type xc_ref_Peverati2012_117 = {
  "R. Peverati and D. G. Truhlar, J. Phys. Chem. Lett. 3, 117 (2012)",
  "10.1021/jz201525m",
  "@article{Peverati2012_117,\n  title = {M11-L: A Local Density Functional That Provides Improved Accuracy for Electronic Structure Calculations in Chemistry and Physics},\n  author = {R. Peverati and D. G. Truhlar},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {3},\n  number = {1},\n  pages = {117},\n  year = {2012},\n  doi = {10.1021/jz201525m},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jz201525m}\n}"
};


func_reference_type xc_ref_Peverati2012_13171 = {
  "R. Peverati and D. G. Truhlar, Phys. Chem. Chem. Phys. 14, 13171 (2012)",
  "10.1039/C2CP42025B",
  "@article{Peverati2012_13171,\n  title = \"An improved and broadly accurate local approximation to the exchange-correlation density functional: The MN12-L functional for electronic structure calculations in chemistry and physics\",\n  author = \"R. Peverati and D. G. Truhlar\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"14\",\n  issue = \"38\",\n  pages = \"13171\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C2CP42025B\",\n  url = \"http://pubs.rsc.org/en/Content/ArticleLanding/2012/CP/c2cp42025b\"\n}"
};


func_reference_type xc_ref_Peverati2012_16187 = {
  "R. Peverati and D. G. Truhlar, Phys. Chem. Chem. Phys. 14, 16187 (2012)",
  "10.1039/C2CP42576A",
  "@article{Peverati2012_16187,\n  title = \"Screened-exchange density functionals with broad accuracy for chemistry and solid-state physics\",\n  author = \"R. Peverati and D. G. Truhlar\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"14\",\n  issue = \"47\",\n  pages = \"16187\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C2CP42576A\",\n  url = \"http://pubs.rsc.org/en/Content/ArticleLanding/2012/CP/c2cp42576a\"\n}"
};


func_reference_type xc_ref_Peverati2012_2310 = {
  "R. Peverati and D. G. Truhlar, J. Chem. Theory Comput. 8, 2310 (2012)",
  "10.1021/ct3002656",
  "@article{Peverati2012_2310,\n  title = {Exchange-Correlation Functional with Good Accuracy for Both Structural and Energetic Properties while Depending Only on the Density and Its Gradient},\n  author = {R. Peverati and D. G. Truhlar},\n  journal = {J. Chem. Theory Comput.},\n  volume = {8},\n  number = {7},\n  pages = {2310},\n  year = {2012},\n  doi = {10.1021/ct3002656},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct3002656}\n}"
};


func_reference_type xc_ref_Pittalis2007_235314 = {
  "S. Pittalis, E. Rasanen, N. Helbig, and E. K. U. Gross, Phys. Rev. B 76, 235314 (2007)",
  "10.1103/PhysRevB.76.235314",
  "@article{Pittalis2007_235314,\n  title = {Exchange-energy functionals for finite two-dimensional systems},\n  author = {S. Pittalis and E. R\\\"as\\\"anen and N. Helbig and E. K. U. Gross},\n  journal = {Phys. Rev. B},\n  volume = {76},\n  issue = {23},\n  pages = {235314},\n  year = {2007},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.76.235314},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.76.235314}\n}"
};


func_reference_type xc_ref_Pittalis2008_195322 = {
  "S. Pittalis, E. Rasanen, and M. A. L. Marques, Phys. Rev. B 78, 195322 (2008)",
  "10.1103/PhysRevB.78.195322",
  "@article{Pittalis2008_195322,\n  title = {Local correlation functional for electrons in two dimensions},\n  author = {S. Pittalis and E. R\\\"as\\\"anen and M. A. L. Marques},\n  journal = {Phys. Rev. B},\n  volume = {78},\n  issue = {19},\n  pages = {195322},\n  year = {2008},\n  month = {Nov},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.78.195322},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.78.195322}\n}"
};


func_reference_type xc_ref_Pittalis2009_012503 = {
  "S. Pittalis, E. Rasanen, J. G. Vilhena, and M. A. L. Marques, Phys. Rev. A 79, 012503 (2009)",
  "10.1103/PhysRevA.79.012503",
  "@article{Pittalis2009_012503,\n  title = {Density gradients for the exchange energy of electrons in two dimensions},\n  author = {S. Pittalis and E. R\\\"as\\\"anen and J. G. Vilhena and M. A. L. Marques},\n  journal = {Phys. Rev. A},\n  volume = {79},\n  issue = {1},\n  pages = {012503},\n  year = {2009},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.79.012503},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.79.012503}\n}"
};


func_reference_type xc_ref_Pittalis2010_115108 = {
  "S. Pittalis, E. Rasanen, and C. R. Proetto, Phys. Rev. B 81, 115108 (2010)",
  "10.1103/PhysRevB.81.115108",
  "@article{Pittalis2010_115108,\n  title = {Becke--Johnson-type exchange potential for two-dimensional systems},\n  author = {S. Pittalis and E. R\\\"as\\\"anen and C. R. Proetto},\n  journal = {Phys. Rev. B},\n  volume = {81},\n  issue = {11},\n  pages = {115108},\n  year = {2010},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.81.115108},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.81.115108}\n}"
};


func_reference_type xc_ref_Proynov1994_7874 = {
  "E. I. Proynov and D. R. Salahub, Phys. Rev. B 49, 7874 (1994)",
  "10.1103/PhysRevB.49.7874",
  "@article{Proynov1994_7874,\n  title = {Simple but efficient correlation functional from a model pair-correlation function},\n  author = {E. I. Proynov and D. R. Salahub},\n  journal = {Phys. Rev. B},\n  volume = {49},\n  issue = {12},\n  pages = {7874},\n  year = {1994},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.49.7874},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.49.7874}\n}"
};


func_reference_type xc_ref_Proynov1998_12616 = {
  "E. I. Proynov and D. R. Salahub, Phys. Rev. B 57, 12616 (1998)",
  "10.1103/PhysRevB.57.12616",
  "@article{Proynov1998_12616,\n  title = {Erratum: Simple but efficient correlation functional from a model pair-correlation function [Phys. Rev. B 49, 7874 (1994)]},\n  author = {Proynov, Emil I. and Salahub, Dennis R.},\n  journal = {Phys. Rev. B},\n  volume = {57},\n  issue = {19},\n  pages = {12616--12616},\n  numpages = {0},\n  year = {1998},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.57.12616},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.57.12616}\n}"
};


func_reference_type xc_ref_Proynov2000_10013 = {
  "E. Proynov, H. Chermette, and D. R. Salahub, J. Chem. Phys. 113, 10013 (2000)",
  "10.1063/1.1321309",
  "@article{Proynov2000_10013,\n  author = {E. Proynov and H. Chermette and D. R. Salahub},\n  title = {New $\\tau$-dependent correlation functional combined with a modified Becke exchange},\n  journal = {J. Chem. Phys.},\n  volume = {113},\n  number = {22},\n  pages = {10013-10027},\n  year = {2000},\n  doi = {10.1063/1.1321309},\n  url = {http://doi.org/10.1063/1.1321309},\n}"
};


func_reference_type xc_ref_Proynov2006_436 = {
  "E. I. Proynov and A. J. Thakkar, Int. J. Quantum Chem. 106, 436 (2006)",
  "10.1002/qua.20758",
  "@article{Proynov2006_436,\n  author = {Proynov, Emil I. and Thakkar, Ajit J.},\n  title = {Is combining meta-GGA correlation functionals with the OPTX exchange functional useful?},\n  journal = {Int. J. Quantum Chem.},\n  volume = {106},\n  number = {2},\n  publisher = {Wiley Subscription Services, Inc., A Wiley Company},\n  issn = {1097-461X},\n  url = {http://doi.org/10.1002/qua.20758},\n  doi = {10.1002/qua.20758},\n  pages = {436--446},\n  year = {2006},\n}"
};


func_reference_type xc_ref_Proynov2008_103 = {
  "E. Proynov, Z. Gan, and J. Kong, Chem. Phys. Lett. 455, 103 (2008)",
  "10.1016/j.cplett.2008.02.039",
  "@article{Proynov2008_103,\n  author = \"Emil Proynov and Zhenting Gan and Jing Kong\",\n  title = \"Analytical representation of the Becke--Roussel exchange functional\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"455\",\n  number = \"1\",\n  pages = \"103 - 109\",\n  year = \"2008\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2008.02.039\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261408002285\",\n}"
};


func_reference_type xc_ref_Proynov2009_014103 = {
  "E. Proynov and J. Kong, Phys. Rev. A 79, 014103 (2009)",
  "10.1103/PhysRevA.79.014103",
  "@article{Proynov2009_014103,\n  title = {Analytic form of the correlation energy of the uniform electron gas},\n  author = {Proynov, Emil and Kong, Jing},\n  journal = {Phys. Rev. A},\n  volume = {79},\n  issue = {1},\n  pages = {014103},\n  year = {2009},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.79.014103},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.79.014103}\n}"
};


func_reference_type xc_ref_Proynov2017_059904 = {
  "E. Proynov and J. Kong, Phys. Rev. A 95, 059904 (2017)",
  "10.1103/PhysRevA.95.059904",
  "@article{Proynov2017_059904,\n  title = {Erratum: Analytic form of the correlation energy of the uniform electron gas [Phys. Rev. A 79, 014103 (2009)]},\n  author = {Proynov, Emil and Kong, Jing},\n  journal = {Phys. Rev. A},\n  volume = {95},\n  issue = {5},\n  pages = {059904},\n  year = {2017},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.95.059904},\n  url = {https://link.aps.org/doi/10.1103/PhysRevA.95.059904}\n}"
};


func_reference_type xc_ref_Rae1973_574 = {
  "A. Rae, Chem. Phys. Lett. 18, 574 (1973)",
  "10.1016/0009-2614(73)80469-5",
  "@article{Rae1973_574,\n  title = \"A theory for the interactions between closed shell systems\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"18\",\n  number = \"4\",\n  pages = \"574 - 577\",\n  year = \"1973\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/0009-2614(73)80469-5\",\n  url = \"//www.sciencedirect.com/science/article/pii/0009261473804695\",\n  author = \"A.I.M. Rae\",\n}"
};


func_reference_type xc_ref_Ragot2004_7671 = {
  "S. Ragot and P. Cortona, J. Chem. Phys. 121, 7671 (2004)",
  "10.1063/1.1792153",
  "@article{Ragot2004_7671,\n  title = \"Correlation energy of many-electron systems: A modified Colle--Salvetti approach\",\n  author = \"S. Ragot and P. Cortona\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2004\",\n  volume = \"121\",\n  number = \"16\",\n  pages = \"7671\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/121/16/10.1063/1.1792153\",\n  doi = \"10.1063/1.1792153\"\n}"
};


func_reference_type xc_ref_Rajagopal1978_L943 = {
  "A. K. Rajagopal, J. Phys. C: Solid State Phys. 11, L943 (1978)",
  "10.1088/0022-3719/11/24/002",
  "@article{Rajagopal1978_L943,\n  author = {A. K. Rajagopal},\n  title = {Inhomogeneous relativistic electron gas},\n  journal = {J. Phys. C: Solid State Phys.},\n  volume = {11},\n  number = {24},\n  pages = {L943},\n  url = {http://stacks.iop.org/0022-3719/11/i = 24/a = 002},\n  year = {1978},\n  doi = {10.1088/0022-3719/11/24/002}\n}"
};


func_reference_type xc_ref_Rasanen2010_044112 = {
  "E. Rasanen, S. Pittalis, and C. R. Proetto, J. Chem. Phys. 132, 044112 (2010)",
  "10.1063/1.3300063",
  "@article{Rasanen2010_044112,\n  title = {Universal correction for the Becke--Johnson exchange potential},\n  author = {E. R\\\"as\\\"anen and S. Pittalis and C. R. Proetto},\n  journal = {J. Chem. Phys.},\n  year = {2010},\n  volume = {132},\n  number = {4},\n  pages = {044112},\n  doi = {10.1063/1.3300063},\n  url = {http://scitation.aip.org/content/aip/journal/jcp/132/4/10.1063/1.3300063}\n}"
};


func_reference_type xc_ref_Reiher2001_48 = {
  "M. Reiher, O. Salomon, and B. A. Hess, Theor. Chem. Acc. 107, 48 (2001)",
  "10.1007/s00214-001-0300-3",
  "@article{Reiher2001_48,\n  title = {Reparameterization of hybrid functionals based on energy differences of states of different multiplicity},\n  author = {M. Reiher and O. Salomon and B. A. Hess},\n  journal = {Theor. Chem. Acc.},\n  volume = {107},\n  number = {1},\n  pages = {48-55},\n  year = {2001},\n  issn = {1432-881X},\n  publisher = {Springer-Verlag},\n  language = {English},\n  doi = {10.1007/s00214-001-0300-3},\n  url = {http://doi.org/10.1007/s00214-001-0300-3}\n}"
};


func_reference_type xc_ref_Renz2009_16292 = {
  "M. Renz, K. Theilacker, C. Lambert, and M. Kaupp, J. Am. Chem. Soc. 131, 16292 (2009)",
  "10.1021/ja9070859",
  "@article{Renz2009_16292,\n  author = {Renz, Manuel and Theilacker, Kolja and Lambert, Christoph and Kaupp, Martin},\n  title = {A Reliable Quantum-Chemical Protocol for the Characterization of Organic Mixed-Valence Compounds},\n  journal = {J. Am. Chem. Soc.},\n  volume = {131},\n  number = {44},\n  pages = {16292-16302},\n  year = {2009},\n  doi = {10.1021/ja9070859},\n  url = {https://doi.org/10.1021/ja9070859}\n}"
};


func_reference_type xc_ref_Rey1998_581 = {
  "J. Rey and A. Savin, Int. J. Quantum Chem. 69, 581 (1998)",
  "10.1002/(SICI)1097-461X(1998)69:4<581::AID-QUA16>3.0.CO;2-2",
  "@article {Rey1998_581,\n  author = {Rey, J\\'{e}r\\^{o}me and Savin, Andreas},\n  title = {Virtual space level shifting and correlation energies},\n  journal = {Int. J. Quantum Chem.},\n  volume = {69},\n  number = {4},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  url = {http://doi.org/10.1002/(SICI)1097-461X(1998)69:4<581::AID-QUA16>3.0.CO;2-2},\n  doi = {10.1002/(SICI)1097-461X(1998)69:4<581::AID-QUA16>3.0.CO;2-2},\n  pages = {581--590},\n  year = {1998},\n}"
};


func_reference_type xc_ref_Rinke2012_126404 = {
  "P. Rinke, A. Schleife, E. Kioupakis, A. Janotti, C. Rodl, F. Bechstedt, M. Scheffler, and C. G. Van de Walle, Phys. Rev. Lett. 108, 126404 (2012)",
  "10.1103/PhysRevLett.108.126404",
  "@article{Rinke2012_126404,\n  title = {First-Principles Optical Spectra for $F$ Centers in MgO},\n  author = {Rinke, Patrick and Schleife, Andr\\'{e} and Kioupakis, Emmanouil and Janotti, Anderson and R\\\"odl, Claudia and Bechstedt, Friedhelm and Scheffler, Matthias and Van de Walle, Chris G.},\n  journal = {Phys. Rev. Lett.},\n  volume = {108},\n  issue = {12},\n  pages = {126404},\n  year = {2012},\n  month = {Mar},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.108.126404},\n  url = {https://link.aps.org/doi/10.1103/PhysRevLett.108.126404}\n}"
};


func_reference_type xc_ref_Rohrdanz2009_054112 = {
  "M. A. Rohrdanz, K. M. Martins, and J. M. Herbert, J. Chem. Phys. 130, 054112 (2009)",
  "10.1063/1.3073302",
  "@article{Rohrdanz2009_054112,\n  author = \"Rohrdanz, Mary A. and Martins, Katie M. and Herbert, John M.\",\n  title = \"A long-range-corrected density functional that performs well for both ground-state properties and time-dependent density functional theory excitation energies, including charge-transfer excited states\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2009\",\n  volume = \"130\",\n  number = \"5\",\n  eid = 054112,\n  pages = \"054112\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/130/5/10.1063/1.3073302\",\n  doi = \"10.1063/1.3073302\"\n}"
};


func_reference_type xc_ref_Ruggeri2018_161105 = {
  "M. Ruggeri, P. L. Rios, and A. Alavi, Phys. Rev. B 98, 161105 (2018)",
  "10.1103/PhysRevB.98.161105",
  "@article{Ruggeri2018_161105,\n  title = {Correlation energies of the high-density spin-polarized electron gas to meV accuracy},\n  author = {Ruggeri, Michele and R\\'{i}os, Pablo L\\'opez and Alavi, Ali},\n  journal = {Phys. Rev. B},\n  volume = {98},\n  issue = {16},\n  pages = {161105},\n  year = {2018},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.98.161105},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.98.161105}\n}"
};


func_reference_type xc_ref_Ruzsinszky2009_763 = {
  "A. Ruzsinszky, G. I. Csonka, and G. E. Scuseria, J. Chem. Theory Comput. 5, 763 (2009)",
  "10.1021/ct8005369",
  "@article{Ruzsinszky2009_763,\n  title = {Regularized Gradient Expansion for Atoms, Molecules, and Solids},\n  author = {A. Ruzsinszky and G. I. Csonka and G. E. Scuseria},\n  journal = {J. Chem. Theory Comput.},\n  volume = {5},\n  number = {4},\n  pages = {763},\n  year = {2009},\n  doi = {10.1021/ct8005369},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct8005369}\n}"
};


func_reference_type xc_ref_Ruzsinszky2012_2078 = {
  "A. Ruzsinszky, J. Sun, B. Xiao, and G. I. Csonka, J. Chem. Theory Comput. 8, 2078 (2012)",
  "10.1021/ct300269u",
  "@article{Ruzsinszky2012_2078,\n  author = {Ruzsinszky, Adrienn and Sun, Jianwei and Xiao, Bing and Csonka, G\\'{a}bor I.},\n  title = {A meta-GGA Made Free of the Order of Limits Anomaly},\n  journal = {J. Chem. Theory Comput.},\n  volume = {8},\n  number = {6},\n  pages = {2078-2087},\n  year = {2012},\n  doi = {10.1021/ct300269u},\n  url = {https://doi.org/10.1021/ct300269u},\n}"
};


func_reference_type xc_ref_Savin1995_327 = {
  "A. Savin and H.-J. Flad, Int. J. Quantum Chem. 56, 327 (1995)",
  "10.1002/qua.560560417",
  "@article{Savin1995_327,\n  title = {Density functionals for the Yukawa electron-electron interaction},\n  author = {A. Savin and H.-J. Flad},\n  journal = {Int. J. Quantum Chem.},\n  volume = {56},\n  number = {4},\n  pages = {327},\n  year = {1995},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  doi = {10.1002/qua.560560417},\n  url = {http://onlinelibrary.wiley.com/doi/10.1002/qua.560560417/abstract}\n}"
};


func_reference_type xc_ref_Schimka2011_024116 = {
  "L. Schimka, J. Harl, and G. Kresse, J. Chem. Phys. 134, 024116 (2011)",
  "10.1063/1.3524336",
  "@article{Schimka2011_024116,\n  author = \"Schimka, Laurids and Harl, Judith and Kresse, Georg\",\n  title = \"Improved hybrid functional for solids: The HSEsol functional\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2011\",\n  volume = \"134\",\n  number = \"2\",\n  pages = \"024116\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/134/2/10.1063/1.3524336\",\n  doi = \"10.1063/1.3524336\"\n}"
};


func_reference_type xc_ref_Schipper2000_1344 = {
  "P. R. T. Schipper, O. V. Gritsenko, S. J. A. van Gisbergen, and E. J. Baerends, J. Chem. Phys. 112, 1344 (2000)",
  "10.1063/1.480688",
  "@article{Schipper2000_1344,\n  title = \"Molecular calculations of excitation energies and (hyper)polarizabilities with a statistical average of orbital model exchange-correlation potentials\",\n  author = \"P. R. T. Schipper and O. V. Gritsenko and S. J. A. van Gisbergen and E. J. Baerends\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2000\",\n  volume = \"112\",\n  number = \"3\",\n  pages = \"1344\",\n  doi = \"10.1063/1.480688\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/112/3/10.1063/1.480688\"\n}"
};


func_reference_type xc_ref_Schmider1998_9624 = {
  "H. L. Schmider and A. D. Becke, J. Chem. Phys. 108, 9624 (1998)",
  "10.1063/1.476438",
  "@article{Schmider1998_9624,\n  title = \"Optimized density functionals from the extended G2 test set\",\n  author = \"H. L. Schmider and A. D. Becke\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1998\",\n  volume = \"108\",\n  number = \"23\",\n  pages = \"9624\",\n  doi = \"10.1063/1.476438\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/108/23/10.1063/1.476438\"\n}"
};


func_reference_type xc_ref_Schultz2005_11127 = {
  "N. E. Schultz, Y. Zhao, and D. G. Truhlar, J. Phys. Chem. A 109, 11127 (2005)",
  "10.1021/jp0539223",
  "@article{Schultz2005_11127,\n  title = {Density Functionals for Inorganometallic and Organometallic Chemistry},\n  author = {N. E. Schultz and Y. Zhao and D. G. Truhlar},\n  journal = {J. Phys. Chem. A},\n  volume = {109},\n  number = {49},\n  pages = {11127},\n  year = {2005},\n  doi = {10.1021/jp0539223},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp0539223}\n}"
};


func_reference_type xc_ref_Seth2012_901 = {
  "M. Seth and T. Ziegler, J. Chem. Theory Comput. 8, 901 (2012)",
  "10.1021/ct300006h",
  "@article{Seth2012_901,\n  author = {Seth, Michael and Ziegler, Tom},\n  title = {Range-Separated Exchange Functionals with Slater-Type Functions},\n  journal = {J. Chem. Theory Comput.},\n  volume = {8},\n  number = {3},\n  pages = {901-907},\n  year = {2012},\n  doi = {10.1021/ct300006h},\n  url = {http://doi.org/10.1021/ct300006h}\n}"
};


func_reference_type xc_ref_Seth2013_2286 = {
  "M. Seth, T. Ziegler, M. Steinmetz, and S. Grimme, J. Chem. Theory Comput. 9, 2286 (2013)",
  "10.1021/ct301112m",
  "@article{Seth2013_2286,\n  author = {Seth, Michael and Ziegler, Tom and Steinmetz, Marc and Grimme, Stefan},\n  title = {Modeling Transition Metal Reactions with Range-Separated Functionals},\n  journal = {J. Chem. Theory Comput.},\n  volume = {9},\n  number = {5},\n  pages = {2286-2299},\n  year = {2013},\n  doi = {10.1021/ct301112m},\n  url = {http://doi.org/10.1021/ct301112m}\n}"
};


func_reference_type xc_ref_Shao2003_4807 = {
  "Y. Shao, M. Head-Gordon, and A. I. Krylov, J. Chem. Phys. 118, 4807 (2003)",
  "10.1063/1.1545679",
  "@article{Shao2003_4807,\n  author = {Yihan Shao and Martin Head-Gordon and Anna I. Krylov},\n  title = {The spin--flip approach within time-dependent density functional theory: Theory and applications to diradicals},\n  journal = {J. Chem. Phys.},\n  volume = {118},\n  number = {11},\n  pages = {4807-4818},\n  year = {2003},\n  doi = {10.1063/1.1545679},\n  url = {http://doi.org/10.1063/1.1545679}\n}"
};


func_reference_type xc_ref_Shao2020_587 = {
  "Y. Shao, Y. Mei, D. Sundholm, and V. R. I. Kaila, J. Chem. Theory Comput. 16, 587 (2020)",
  "10.1021/acs.jctc.9b00823",
  "@article{Shao2020_587,\n  author = {Shao, Yihan and Mei, Ye and Sundholm, Dage and Kaila, Ville R. I.},\n  title = {Benchmarking the Performance of Time-Dependent Density Functional Theory Methods on Biochromophores},\n  journal = {J. Chem. Theory Comput.},\n  volume = {16},\n  number = {1},\n  pages = {587-600},\n  year = {2020},\n  doi = {10.1021/acs.jctc.9b00823},\n  url = {https://doi.org/10.1021/acs.jctc.9b00823}\n}"
};


func_reference_type xc_ref_Slater1951_385 = {
  "J. C. Slater, Phys. Rev. 81, 385 (1951)",
  "10.1103/PhysRev.81.385",
  "@article{Slater1951_385,\n  title = {A Simplification of the Hartree--Fock Method},\n  author = {J. C. Slater},\n  journal = {Phys. Rev.},\n  volume = {81},\n  issue = {3},\n  pages = {385},\n  year = {1951},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRev.81.385},\n  url = {http://link.aps.org/doi/10.1103/PhysRev.81.385}\n}"
};


func_reference_type xc_ref_Song2007_154105 = {
  "J.-W. Song, T. Hirosawa, T. Tsuneda, and K. Hirao, J. Chem. Phys. 126, 154105 (2007)",
  "10.1063/1.2721532",
  "@article{Song2007_154105,\n  author = {Song,Jong-Won  and Hirosawa,Tsuyoshi  and Tsuneda,Takao  and Hirao,Kimihiko },\n  title = {Long-range corrected density functional calculations of chemical reactions: Redetermination of parameter},\n  journal = {J. Chem. Phys.},\n  volume = {126},\n  number = {15},\n  pages = {154105},\n  year = {2007},\n  doi = {10.1063/1.2721532},\n}"
};


func_reference_type xc_ref_Song2007_154109 = {
  "J.-W. Song, S. Tokura, T. Sato, M. A. Watson, and K. Hirao, J. Chem. Phys. 127, 154109 (2007)",
  "10.1063/1.2790017",
  "@article{Song2007_154109,\n  author = {Song, Jong-Won and Tokura, Seiken and Sato, Takeshi and Watson, Mark A. and Hirao, Kimihiko},\n  title = {An improved long-range corrected hybrid exchange-correlation functional including a short-range Gaussian attenuation (LCgau-BOP)},\n  journal = {J. Chem. Phys.},\n  volume = {127},\n  number = {15},\n  pages = {154109},\n  year = {2007},\n  doi = {10.1063/1.2790017},\n  url = {https://doi.org/10.1063/1.2790017},\n}"
};


func_reference_type xc_ref_Song2008_184113 = {
  "J.-W. Song, M. A. Watson, A. Nakata, and K. Hirao, J. Chem. Phys. 129, 184113 (2008)",
  "10.1063/1.3010372",
  "@article{Song2008_184113,\n  author = {Song, Jong-Won and Watson, Mark A. and Nakata, Ayako and Hirao, Kimihiko},\n  title = {Core-excitation energy calculations with a long-range corrected hybrid exchange-correlation functional including a short-range Gaussian attenuation (LCgau-BOP)},\n  journal = {J. Chem. Phys.},\n  volume = {129},\n  number = {18},\n  pages = {184113},\n  year = {2008},\n  doi = {10.1063/1.3010372},\n  url = {https://doi.org/10.1063/1.3010372},\n}"
};


func_reference_type xc_ref_Song2009_144108 = {
  "J.-W. Song, M. A. Watson, and K. Hirao, J. Chem. Phys. 131, 144108 (2009)",
  "10.1063/1.3243819",
  "@article{Song2009_144108,\n  author = {Song, Jong-Won and Watson, Mark A. and Hirao, Kimihiko},\n  title = {An improved long-range corrected hybrid functional with vanishing Hartree--Fock exchange at zero interelectronic distance (LC2gau-BOP)},\n  journal = {J. Chem. Phys.},\n  volume = {131},\n  number = {14},\n  pages = {144108},\n  year = {2009},\n  doi = {10.1063/1.3243819},\n  url = {https://doi.org/10.1063/1.3243819},\n}"
};


func_reference_type xc_ref_Sparrow2021 = {
  "Z. M. Sparrow, B. G. Ernst, T. K. Quady, and R. A. D. Jr, Case21: Uniting non-empirical and semi-empirical density functional approximation strategies using constraint-based regularization, (2021), arXiv:2109.12560 [physics.chem-ph]",
  "",
  "@misc{Sparrow2021,\n  title={CASE21: Uniting Non-Empirical and Semi-Empirical Density Functional Approximation Strategies using Constraint-Based Regularization},\n  author={Zachary M. Sparrow and Brian G. Ernst and Trine K. Quady and Robert A. DiStasio Jr},\n  year={2021},\n  eprint={2109.12560},\n  archivePrefix={arXiv},\n  primaryClass={physics.chem-ph}\n}"
};


func_reference_type xc_ref_Staroverov2003_12129 = {
  "V. N. Staroverov, G. E. Scuseria, J. Tao, and J. P. Perdew, J. Chem. Phys. 119, 12129 (2003)",
  "10.1063/1.1626543",
  "@article{Staroverov2003_12129,\n  title = \"Comparative assessment of a new nonempirical density functional: Molecules and hydrogen-bonded complexes\",\n  author = \"V. N. Staroverov and G. E. Scuseria and J. Tao and J. P. Perdew\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2003\",\n  volume = \"119\",\n  number = \"23\",\n  pages = \"12129\",\n  doi = \"10.1063/1.1626543\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/119/23/10.1063/1.1626543\"\n}"
};


func_reference_type xc_ref_Stephens1994_11623 = {
  "P. J. Stephens, F. J. Devlin, C. F. Chabalowski, and M. J. Frisch, J. Phys. Chem. 98, 11623 (1994)",
  "10.1021/j100096a001",
  "@article{Stephens1994_11623,\n  title = {Ab Initio Calculation of Vibrational Absorption and Circular Dichroism Spectra Using Density Functional Force Fields},\n  author = {P. J. Stephens and F. J. Devlin and C. F. Chabalowski and M. J. Frisch},\n  journal = {J. Phys. Chem.},\n  volume = {98},\n  number = {45},\n  pages = {11623},\n  year = {1994},\n  doi = {10.1021/j100096a001},\n  url = {http://pubs.acs.org/doi/abs/10.1021/j100096a001}\n}"
};


func_reference_type xc_ref_Stewart1995_4337 = {
  "P. A. Stewart and P. M. W. Gill, J. Chem. Soc., Faraday Trans. 91, 4337 (1995)",
  "10.1039/FT9959104337",
  "@article{Stewart1995_4337,\n  author = \"Stewart, Phillip A. and Gill, Peter M. W.\",\n  title = \"Becke--Wigner: a simple but powerful density functional\",\n  journal = \"J. Chem. Soc., Faraday Trans.\",\n  year = \"1995\",\n  volume = \"91\",\n  issue = \"24\",\n  pages = \"4337-4341\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/FT9959104337\",\n  url = \"http://doi.org/10.1039/FT9959104337\",\n}"
};


func_reference_type xc_ref_Sun2012_051101 = {
  "J. Sun, B. Xiao, and A. Ruzsinszky, J. Chem. Phys. 137, 051101 (2012)",
  "10.1063/1.4742312",
  "@article{Sun2012_051101,\n  title = \"Communication: Effect of the orbital-overlap dependence in the meta generalized gradient approximation\",\n  author = \"J. Sun and B. Xiao and A. Ruzsinszky\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"137\",\n  number = \"5\",\n  pages = 051101,\n  doi = \"10.1063/1.4742312\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/137/5/10.1063/1.4742312\"\n}"
};


func_reference_type xc_ref_Sun2013_044113 = {
  "J. Sun, R. Haunschild, B. Xiao, I. W. Bulik, G. E. Scuseria, and J. P. Perdew, J. Chem. Phys. 138, 044113 (2013)",
  "10.1063/1.4789414",
  "@article{Sun2013_044113,\n  title = \"Semilocal and hybrid meta-generalized gradient approximations based on the understanding of the kinetic-energy-density dependence\",\n  author = \"J. Sun and R. Haunschild and B. Xiao and I. W. Bulik and G. E. Scuseria and J. P. Perdew\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2013\",\n  volume = \"138\",\n  number = \"4\",\n  pages = 044113,\n  doi = \"10.1063/1.4789414\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/138/4/10.1063/1.4789414\"\n}"
};


func_reference_type xc_ref_Sun2015_036402 = {
  "J. Sun, A. Ruzsinszky, and J. P. Perdew, Phys. Rev. Lett. 115, 036402 (2015)",
  "10.1103/PhysRevLett.115.036402",
  "@article{Sun2015_036402,\n  title = {Strongly Constrained and Appropriately Normed Semilocal Density Functional},\n  author = {Sun, Jianwei and Ruzsinszky, Adrienn and Perdew, John P.},\n  journal = {Phys. Rev. Lett.},\n  volume = {115},\n  issue = {3},\n  pages = {036402},\n  year = {2015},\n  month = {Jul},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.115.036402},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.115.036402}\n}"
};


func_reference_type xc_ref_Sun2015_685 = {
  "J. Sun, J. P. Perdew, and A. Ruzsinszky, Proc. Natl. Acad. Sci. U. S. A. 112, 685 (2015)",
  "10.1073/pnas.1423145112",
  "@article{Sun2015_685,\n  author = {Sun, Jianwei and Perdew, John P. and Ruzsinszky, Adrienn},\n  title = {Semilocal density functional obeying a strongly tightened bound for exchange},\n  volume = {112},\n  number = {3},\n  pages = {685-689},\n  year = {2015},\n  doi = {10.1073/pnas.1423145112},\n  url = {http://www.pnas.org/content/112/3/685.abstract},\n  journal = {Proc. Natl. Acad. Sci. U. S. A.}\n}"
};


func_reference_type xc_ref_Swart2009_094103 = {
  "M. Swart, M. Sola, and F. M. Bickelhaupt, J. Chem. Phys. 131, 094103 (2009)",
  "10.1063/1.3213193",
  "@article{Swart2009_094103,\n  title = \"A new all-round density functional based on spin states and SN2 barriers\",\n  author = \"M. Swart and M. Sol\\'{a} and F. M. Bickelhaupt\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2009\",\n  volume = \"131\",\n  number = \"9\",\n  pages = 094103,\n  doi = \"10.1063/1.3213193\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/131/9/10.1063/1.3213193\"\n}"
};


func_reference_type xc_ref_Swart2009_69 = {
  "M. Swart, M. Sola, and F. M. Bickelhaupt, J. Comput. Methods Sci. Eng. 9, 69 (2009)",
  "10.3233/JCM-2009-0230",
  "@article{Swart2009_69,\n  title = {Switching between OPTX and PBE exchange functionals},\n  author = {M. Swart and M. Sol\\'{a} and F. M. Bickelhaupt},\n  journal = {J. Comput. Methods Sci. Eng.},\n  volume = {9},\n  issue = {1},\n  pages = {69},\n  year = {2009},\n  doi = {10.3233/JCM-2009-0230},\n  url = {http://iospress.metapress.com/content/0736k00r11272hm7}\n}"
};


func_reference_type xc_ref_Swart2011_1117 = {
  "M. Swart, M. Sola, and F. M. Bickelhaupt, J. Comput. Chem. 32, 1117 (2011)",
  "10.1002/jcc.21693",
  "@article{Swart2011_1117,\n  author = {Swart, Marcel and Sol\\'{a}, Miquel and Bickelhaupt, F. Matthias},\n  title = {Inter- and intramolecular dispersion interactions},\n  journal = {J. Comput. Chem.},\n  volume = {32},\n  number = {6},\n  pages = {1117-1127},\n  doi = {10.1002/jcc.21693},\n  url = {https://onlinelibrary.wiley.com/doi/abs/10.1002/jcc.21693},\n  year = {2011}\n}"
};


func_reference_type xc_ref_Swart2013_166 = {
  "M. Swart, Chem. Phys. Lett. 580, 166 (2013)",
  "10.1016/j.cplett.2013.06.045",
  "@article{Swart2013_166,\n  title = \"A new family of hybrid density functionals\",\n  author = \"Marcel Swart\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"580\",\n  pages = \"166 - 171\",\n  year = \"2013\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2013.06.045\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261413008221\"\n}"
};


func_reference_type xc_ref_Tahchieva2018_4806 = {
  "D. N. Tahchieva, D. Bakowies, R. Ramakrishnan, and O. A. von Lilienfeld, J. Chem. Theory Comput. 14, 4806 (2018)",
  "10.1021/acs.jctc.8b00174",
  "@article{Tahchieva2018_4806,\n  author = {Tahchieva, Diana N. and Bakowies, Dirk and Ramakrishnan, Raghunathan and von Lilienfeld, O. Anatole},\n  title = {Torsional Potentials of Glyoxal, Oxalyl Halides, and Their Thiocarbonyl Derivatives: Challenges for Popular Density Functional Approximations},\n  journal = {J. Chem. Theory Comput.},\n  volume = {14},\n  number = {9},\n  pages = {4806-4817},\n  year = {2018},\n  doi = {10.1021/acs.jctc.8b00174},\n  url = {https://doi.org/10.1021/acs.jctc.8b00174}\n}"
};


func_reference_type xc_ref_Tao2001_3519 = {
  "J. Tao, J. Chem. Phys. 115, 3519 (2001)",
  "10.1063/1.1388047",
  "@article{Tao2001_3519,\n  author = {Tao, Jianmin},\n  title = {Exchange energy density of an atom as a functional of the electron density},\n  journal = {J. Chem. Phys.},\n  volume = {115},\n  number = {8},\n  pages = {3519-3530},\n  year = {2001},\n  doi = {10.1063/1.1388047},\n  url = {https://doi.org/10.1063/1.1388047},\n}"
};


func_reference_type xc_ref_Tao2002_2335 = {
  "J. Tao, J. Chem. Phys. 116, 2335 (2002)",
  "10.1063/1.1447882",
  "@article{Tao2002_2335,\n  author = {Tao, Jianmin},\n  title = {An accurate MGGA-based hybrid exchange-correlation functional},\n  journal = {J. Chem. Phys.},\n  volume = {116},\n  number = {6},\n  pages = {2335-2337},\n  year = {2002},\n  doi = {10.1063/1.1447882},\n  url = {https://doi.org/10.1063/1.1447882}\n}"
};


func_reference_type xc_ref_Tao2003_146401 = {
  "J. Tao, J. P. Perdew, V. N. Staroverov, and G. E. Scuseria, Phys. Rev. Lett. 91, 146401 (2003)",
  "10.1103/PhysRevLett.91.146401",
  "@article{Tao2003_146401,\n  title = {Climbing the Density Functional Ladder: Nonempirical Meta-Generalized Gradient Approximation Designed for Molecules and Solids},\n  author = {J. Tao and J. P. Perdew and V. N. Staroverov and G. E. Scuseria},\n  journal = {Phys. Rev. Lett.},\n  volume = {91},\n  issue = {14},\n  pages = {146401},\n  year = {2003},\n  month = {Sep},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.91.146401},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.91.146401}\n}"
};


func_reference_type xc_ref_Tao2016_073001 = {
  "J. Tao and Y. Mo, Phys. Rev. Lett. 117, 073001 (2016)",
  "10.1103/PhysRevLett.117.073001",
  "@article{Tao2016_073001,\n  title = {Accurate Semilocal Density Functional for Condensed-Matter Physics and Quantum Chemistry},\n  author = {Tao, Jianmin and Mo, Yuxiang},\n  journal = {Phys. Rev. Lett.},\n  volume = {117},\n  issue = {7},\n  pages = {073001},\n  year = {2016},\n  month = {Aug},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.117.073001},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.117.073001}\n}"
};


func_reference_type xc_ref_Tawada2004_8425 = {
  "Y. Tawada, T. Tsuneda, S. Yanagisawa, T. Yanai, and K. Hirao, J. Chem. Phys. 120, 8425 (2004)",
  "10.1063/1.1688752",
  "@article{Tawada2004_8425,\n  author = {Yoshihiro Tawada and Takao Tsuneda and Susumu Yanagisawa and Takeshi Yanai and Kimihiko Hirao},\n  title = {A long-range-corrected time-dependent density functional theory},\n  journal = {J. Chem. Phys.},\n  volume = {120},\n  number = {18},\n  pages = {8425-8433},\n  year = {2004},\n  doi = {10.1063/1.1688752},\n  url = {http://doi.org/10.1063/1.1688752},\n}"
};


func_reference_type xc_ref_Thakkar1992_6920 = {
  "A. J. Thakkar, Phys. Rev. A 46, 6920 (1992)",
  "10.1103/PhysRevA.46.6920",
  "@article{Thakkar1992_6920,\n  title = {Comparison of kinetic-energy density functionals},\n  author = {A. J. Thakkar},\n  journal = {Phys. Rev. A},\n  volume = {46},\n  issue = {11},\n  pages = {6920},\n  year = {1992},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.46.6920},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.46.6920}\n}"
};


func_reference_type xc_ref_Thakkar2009_134109 = {
  "A. J. Thakkar and S. P. McCarthy, J. Chem. Phys. 131, 134109 (2009)",
  "10.1063/1.3243845",
  "@article{Thakkar2009_134109,\n  author = {Ajit J. Thakkar and Shane P. McCarthy},\n  title = {Toward improved density functionals for the correlation energy},\n  journal = {J. Chem. Phys.},\n  volume = {131},\n  number = {13},\n  pages = {134109},\n  year = {2009},\n  doi = {10.1063/1.3243845},\n  url = {http://doi.org/10.1063/1.3243845},\n}"
};


func_reference_type xc_ref_Thomas1927_542 = {
  "L. H. Thomas, Math. Proc. Cambridge Philos. Soc. 23, 542 (1927)",
  "10.1017/S0305004100011683",
  "@article{Thomas1927_542,\n  title = {The calculation of atomic fields},\n  author = {L. H. Thomas},\n  journal = {Math. Proc. Cambridge Philos. Soc.},\n  volume = {23},\n  issue = {05},\n  month = {1},\n  year = {1927},\n  issn = {1469-8064},\n  pages = {542},\n  doi = {10.1017/S0305004100011683},\n  url = {http://journals.cambridge.org/article_S0305004100011683},\n}"
};


func_reference_type xc_ref_Tognetti2007_381 = {
  "V. Tognetti, P. Cortona, and C. Adamo, Chem. Phys. Lett. 439, 381 (2007)",
  "10.1016/j.cplett.2007.03.081",
  "@article{Tognetti2007_381,\n  title = \"The performances of a parameter-free local correlation functional: The Ragot--Cortona model\",\n  author = \"V. Tognetti and P. Cortona and C. Adamo\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"439\",\n  number = \"4--6\",\n  pages = \"381\",\n  year = \"2007\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2007.03.081\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261407003600\",\n}"
};


func_reference_type xc_ref_Tognetti2008_034101 = {
  "V. Tognetti, P. Cortona, and C. Adamo, J. Chem. Phys. 128, 034101 (2008)",
  "10.1063/1.2816137",
  "@article{Tognetti2008_034101,\n  title = \"A new parameter-free correlation functional based on an average atomic reduced density gradient analysis\",\n  author = \"V. Tognetti and P. Cortona and C. Adamo\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2008\",\n  volume = \"128\",\n  number = \"3\",\n  pages = 034101,\n  doi = \"10.1063/1.2816137\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/128/3/10.1063/1.2816137\"\n}"
};


func_reference_type xc_ref_Tognetti2008_536 = {
  "V. Tognetti, P. Cortona, and C. Adamo, Chem. Phys. Lett. 460, 536 (2008)",
  "10.1016/j.cplett.2008.06.032",
  "@article{Tognetti2008_536,\n  title = \"Increasing physical constraints and improving performances in a parameter-free GGA functional\",\n  author = \"V. Tognetti and P. Cortona and C. Adamo\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"460\",\n  number = \"4--6\",\n  pages = \"536\",\n  year = \"2008\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2008.06.032\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261408008464\"\n}"
};


func_reference_type xc_ref_Tognetti2009_14415 = {
  "V. Tognetti and C. Adamo, J. Phys. Chem. A 113, 14415 (2009)",
  "10.1021/jp903672e",
  "@article{Tognetti2009_14415,\n  title = {Optimized GGA Functional for Proton Transfer Reactions},\n  author = {V. Tognetti and C. Adamo},\n  journal = {J. Phys. Chem. A},\n  volume = {113},\n  number = {52},\n  pages = {14415},\n  year = {2009},\n  doi = {10.1021/jp903672e},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp903672e},\n}"
};


func_reference_type xc_ref_Toulouse2002_10465 = {
  "J. Toulouse, A. Savin, and C. Adamo, J. Chem. Phys. 117, 10465 (2002)",
  "10.1063/1.1521432",
  "@article{Toulouse2002_10465,\n  author = {Julien Toulouse and Andreas Savin and Carlo Adamo},\n  title = {Validation and assessment of an accurate approach to the correlation problem in density functional theory: The Kriger--Chen--Iafrate--Savin model},\n  journal = {J. Chem. Phys.},\n  volume = {117},\n  number = {23},\n  pages = {10465-10473},\n  year = {2002},\n  doi = {10.1063/1.1521432},\n  url = {http://doi.org/10.1063/1.1521432},\n}"
};


func_reference_type xc_ref_Toulouse2004_1047 = {
  "J. Toulouse, A. Savin, and H.-J. Flad, Int. J. Quantum Chem. 100, 1047 (2004)",
  "10.1002/qua.20259",
  "@article {Toulouse2004_1047,\n  author = {Toulouse, Julien and Savin, Andreas and Flad, Heinz-J\\\"urgen},\n  title = {Short-range exchange-correlation energy of a uniform electron gas with modified electron-electron interaction},\n  journal = {Int. J. Quantum Chem.},\n  volume = {100},\n  number = {6},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  url = {http://doi.org/10.1002/qua.20259},\n  doi = {10.1002/qua.20259},\n  pages = {1047--1056},\n  year = {2004}\n}"
};


func_reference_type xc_ref_Tozer1996_9200 = {
  "D. J. Tozer, V. E. Ingamells, and N. C. Handy, J. Chem. Phys. 105, 9200 (1996)",
  "10.1063/1.472753",
  "@article{Tozer1996_9200,\n  author = {Tozer, David J. and Ingamells, Victoria E. and Handy, Nicholas C.},\n  title = {Exchange-correlation potentials},\n  journal = {J. Chem. Phys.},\n  volume = {105},\n  number = {20},\n  pages = {9200-9213},\n  year = {1996},\n  doi = {10.1063/1.472753},\n  url = {https://doi.org/10.1063/1.472753},\n}"
};


func_reference_type xc_ref_Tozer1997_183 = {
  "D. J. Tozer, N. C. Handy, and W. H. Green, Chem. Phys. Lett. 273, 183 (1997)",
  "10.1016/S0009-2614(97)00586-1",
  "@article{Tozer1997_183,\n  title = \"Exchange-correlation functionals from ab initio electron densities\",\n  author = \"D. J. Tozer and N. C. Handy and W. H. Green\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"273\",\n  number = \"3--4\",\n  pages = \"183\",\n  year = \"1997\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/S0009-2614(97)00586-1\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261497005861\",\n}"
};


func_reference_type xc_ref_Tozer1998_2545 = {
  "D. J. Tozer and N. C. Handy, J. Chem. Phys. 108, 2545 (1998)",
  "10.1063/1.475638",
  "@article{Tozer1998_2545,\n  title = \"The development of new exchange-correlation functionals\",\n  author = \"D. J. Tozer and N. C. Handy\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1998\",\n  volume = \"108\",\n  number = \"6\",\n  pages = \"2545\",\n  doi = \"10.1063/1.475638\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/108/6/10.1063/1.475638\"\n}"
};


func_reference_type xc_ref_Tozer1998_3162 = {
  "D. J. Tozer and N. C. Handy, J. Phys. Chem. A 102, 3162 (1998)",
  "10.1021/jp980259s",
  "@article{Tozer1998_3162,\n  title = {Development of New Exchange-Correlation Functionals. 2},\n  author = {D. J. Tozer and N. C. Handy},\n  journal = {J. Phys. Chem. A},\n  volume = {102},\n  number = {18},\n  pages = {3162},\n  year = {1998},\n  doi = {10.1021/jp980259s},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp980259s}\n}"
};


func_reference_type xc_ref_Tran2002_441 = {
  "F. Tran and T. A. Wesolowski, Int. J. Quantum Chem. 89, 441 (2002)",
  "10.1002/qua.10306",
  "@article{Tran2002_441,\n  title = {Link between the kinetic- and exchange-energy functionals in the generalized gradient approximation},\n  author = {F. Tran and T. A. Weso\\l{}owski},\n  journal = {Int. J. Quantum Chem.},\n  volume = {89},\n  number = {5},\n  pages = {441},\n  year = {2002},\n  publisher = {Wiley Subscription Services, Inc., A Wiley Company},\n  issn = {1097-461X},\n  doi = {10.1002/qua.10306},\n  url = {http://onlinelibrary.wiley.com/doi/10.1002/qua.10306/abstract}\n}"
};


func_reference_type xc_ref_Tran2009_226401 = {
  "F. Tran and P. Blaha, Phys. Rev. Lett. 102, 226401 (2009)",
  "10.1103/PhysRevLett.102.226401",
  "@article{Tran2009_226401,\n  title = {Accurate Band Gaps of Semiconductors and Insulators with a Semilocal Exchange-Correlation Potential},\n  author = {F. Tran and P. Blaha},\n  journal = {Phys. Rev. Lett.},\n  volume = {102},\n  issue = {22},\n  pages = {226401},\n  year = {2009},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.102.226401},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.102.226401}\n}"
};


func_reference_type xc_ref_Trickey2011_075146 = {
  "S. B. Trickey, V. V. Karasiev, and A. Vela, Phys. Rev. B 84, 075146 (2011)",
  "10.1103/PhysRevB.84.075146",
  "@article{Trickey2011_075146,\n  title = {Positivity constraints and information-theoretical kinetic energy functionals},\n  author = {Trickey, S. B. and Karasiev, V. V. and Vela, A.},\n  journal = {Phys. Rev. B},\n  volume = {84},\n  issue = {7},\n  pages = {075146},\n  year = {2011},\n  month = {Aug},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.84.075146},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.84.075146}\n}"
};


func_reference_type xc_ref_Tsuneda1999_10664 = {
  "T. Tsuneda, T. Suzumura, and K. Hirao, J. Chem. Phys. 110, 10664 (1999)",
  "10.1063/1.479012",
  "@article{Tsuneda1999_10664,\n  title = \"A new one-parameter progressive Colle--Salvetti-type correlation functional\",\n  author = \"T. Tsuneda and T. Suzumura and K. Hirao\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1999\",\n  volume = \"110\",\n  number = \"22\",\n  pages = \"10664\",\n  doi = \"10.1063/1.479012\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/110/22/10.1063/1.479012\"\n}"
};


func_reference_type xc_ref_Tsuneda1999_5656 = {
  "T. Tsuneda, T. Suzumura, and K. Hirao, J. Chem. Phys. 111, 5656 (1999)",
  "10.1063/1.479954",
  "@article{Tsuneda1999_5656,\n  author = \"Tsuneda, Takao and Suzumura, Toshihisa and Hirao, Kimihiko\",\n  title = \"A reexamination of exchange energy functionals\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1999\",\n  volume = \"111\",\n  number = \"13\",\n  pages = \"5656-5667\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/111/13/10.1063/1.479954\",\n  doi = \"10.1063/1.479954\"\n}"
};


func_reference_type xc_ref_Tsuneda2000_15527 = {
  "T. Tsuneda and K. Hirao, Phys. Rev. B 62, 15527 (2000)",
  "10.1103/PhysRevB.62.15527",
  "@article{Tsuneda2000_15527,\n  title = {Parameter-free exchange functional},\n  author = {Tsuneda, Takao and Hirao, Kimihiko},\n  journal = {Phys. Rev. B},\n  volume = {62},\n  issue = {23},\n  pages = {15527--15531},\n  year = {2000},\n  month = {Dec},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.62.15527},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.62.15527}\n}"
};


func_reference_type xc_ref_Ugalde1994_423 = {
  "J. M. Ugalde, C. Sarasola, and M. Aguado, J. Phys. B: At., Mol. Opt. Phys. 27, 423 (1994)",
  "10.1088/0953-4075/27/3/009",
  "@article{Ugalde1994_423,\n  doi = {10.1088/0953-4075/27/3/009},\n  url = {https://doi.org/10.1088%2F0953-4075%2F27%2F3%2F009},\n  year = 1994,\n  month = {feb},\n  publisher = {{IOP} Publishing},\n  volume = {27},\n  number = {3},\n  pages = {423--427},\n  author = {J M Ugalde and C Sarasola and M Aguado},\n  title = {Exchange energy from Gaussian-type basis sets},\n  journal = {J. Phys. B: At., Mol. Opt. Phys.},\n}"
};


func_reference_type xc_ref_Urso2021_1 = {
  "V. Urso and L. A. Constantin, Eur. Phys. J. B 94, 1 (2021)",
  "",
  "@article{Urso2021_1,\n  title={Quasi-dimensional models applied to kinetic and exchange energy density functionals},\n  author={Urso, Vittoria and Constantin, Lucian A},\n  journal={Eur. Phys. J. B},\n  volume={94},\n  number={7},\n  pages={1--9},\n  year={2021},\n  publisher={Springer}\n}"
};


func_reference_type xc_ref_VanVoorhis1998_400 = {
  "T. V. Voorhis and G. E. Scuseria, J. Chem. Phys. 109, 400 (1998)",
  "10.1063/1.476577",
  "@article{VanVoorhis1998_400,\n  title = \"A novel form for the exchange-correlation energy functional\",\n  author = \"T. Van Voorhis and G. E. Scuseria\",\n  journal = \"J. Chem. Phys.\",\n  year = \"1998\",\n  volume = \"109\",\n  number = \"2\",\n  pages = \"400\",\n  doi = \"10.1063/1.476577\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/109/2/10.1063/1.476577\"\n}"
};


func_reference_type xc_ref_Vela2009_244103 = {
  "A. Vela, V. Medel, and S. B. Trickey, J. Chem. Phys. 130, 244103 (2009)",
  "10.1063/1.3152713",
  "@article{Vela2009_244103,\n  title = \"Variable Lieb--Oxford bound satisfaction in a generalized gradient exchange-correlation functional\",\n  author = \"A. Vela and V. Medel and S. B. Trickey\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2009\",\n  volume = \"130\",\n  number = \"24\",\n  pages = 244103,\n  doi = \"10.1063/1.3152713\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/130/24/10.1063/1.3152713\"\n}"
};


func_reference_type xc_ref_Vela2012_144115 = {
  "A. Vela, J. C. Pacheco-Kato, J. L. Gazquez, J. M. del Campo, and S. B. Trickey, J. Chem. Phys. 136, 144115 (2012)",
  "10.1063/1.3701132",
  "@article{Vela2012_144115,\n  title = \"Improved constraint satisfaction in a simple generalized gradient approximation exchange functional\",\n  author = \"A. Vela and J. C. Pacheco-Kato and J. L. G\\'{a}zquez and J. M. del Campo and S. B. Trickey\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"136\",\n  number = \"14\",\n  pages = 144115,\n  doi = \"10.1063/1.3701132\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/136/14/10.1063/1.3701132\"\n}"
};


func_reference_type xc_ref_Verma2014_18A534 = {
  "P. Verma and R. J. Bartlett, J. Chem. Phys. 140, 18A534 (2014)",
  "10.1063/1.4871409",
  "@article{Verma2014_18A534,\n  author = {Verma, Prakash and Bartlett, Rodney J.},\n  title = {Increasing the applicability of density functional theory. IV. Consequences of ionization-potential improved exchange-correlation potentials},\n  journal = {J. Chem. Phys.},\n  volume = {140},\n  number = {18},\n  pages = {18A534},\n  year = {2014},\n  doi = {10.1063/1.4871409},\n  url = {https://doi.org/10.1063/1.4871409}\n}"
};


func_reference_type xc_ref_Verma2017_380 = {
  "P. Verma and D. G. Truhlar, J. Phys. Chem. Lett. 8, 380 (2017)",
  "10.1021/acs.jpclett.6b02757",
  "@article{Verma2017_380,\n  author = {Verma, Pragya and Truhlar, Donald G.},\n  title = {HLE16: A Local Kohn--Sham Gradient Approximation with Good Performance for Semiconductor Band Gaps and Molecular Excitation Energies},\n  journal = {J. Phys. Chem. Lett.},\n  volume = {8},\n  number = {2},\n  pages = {380-387},\n  year = {2017},\n  doi = {10.1021/acs.jpclett.6b02757},\n  url = {http://doi.org/10.1021/acs.jpclett.6b02757}\n}"
};


func_reference_type xc_ref_Verma2017_7144 = {
  "P. Verma and D. G. Truhlar, J. Phys. Chem. C 121, 7144 (2017)",
  "10.1021/acs.jpcc.7b01066",
  "@article{Verma2017_7144,\n  author = {Verma, Pragya and Truhlar, Donald G.},\n  title = {HLE17: An Improved Local Exchange--Correlation Functional for Computing Semiconductor Band Gaps and Molecular Excitation Energies},\n  journal = {J. Phys. Chem. C},\n  volume = {121},\n  number = {13},\n  pages = {7144-7154},\n  year = {2017},\n  doi = {10.1021/acs.jpcc.7b01066},\n  url = {http://doi.org/10.1021/acs.jpcc.7b01066}\n}"
};


func_reference_type xc_ref_Verma2019_2966 = {
  "P. Verma, Y. Wang, S. Ghosh, X. He, and D. G. Truhlar, J. Phys. Chem. A 123, 2966 (2019)",
  "10.1021/acs.jpca.8b11499",
  "@article{Verma2019_2966,\n  author = {Verma, Pragya and Wang, Ying and Ghosh, Soumen and He, Xiao and Truhlar, Donald G.},\n  title = {Revised M11 Exchange-Correlation Functional for Electronic Excitation Energies and Ground-State Properties},\n  journal = {J. Phys. Chem. A},\n  volume = {123},\n  number = {13},\n  pages = {2966-2990},\n  year = {2019},\n  doi = {10.1021/acs.jpca.8b11499},\n  url = {https://doi.org/10.1021/acs.jpca.8b11499}\n}"
};


func_reference_type xc_ref_Vilhena2014 = {
  "J. G. Vilhena, E. Rasanen, M. A. L. Marques, and S. Pittalis, J. Chem. Theory Comput. 10, 1837 (2014)",
  "10.1021/ct4010728",
  "@article{Vilhena2014,\n  author = {Vilhena, J. G. and R\\\"as\\\"anen, E. and Marques, M. A. L. and Pittalis, S.},\n  title = {Construction of the B88 Exchange-Energy Functional in Two Dimensions},\n  journal = {J. Chem. Theory Comput.},\n  volume = {10},\n  number = {5},\n  pages = {1837-1842},\n  year = {2014},\n  doi = {10.1021/ct4010728},\n  url = {https://doi.org/10.1021/ct4010728},\n}"
};


func_reference_type xc_ref_Vitos1998_12611 = {
  "L. Vitos, H. L. Skriver, and J. Kollar, Phys. Rev. B 57, 12611 (1998)",
  "10.1103/PhysRevB.57.12611",
  "@article{Vitos1998_12611,\n  title = {Kinetic-energy functionals studied by surface calculations},\n  author = {L. Vitos and H. L. Skriver and J. Koll\\'{a}r},\n  journal = {Phys. Rev. B},\n  volume = {57},\n  issue = {19},\n  pages = {12611},\n  year = {1998},\n  month = {May},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.57.12611},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.57.12611}\n}"
};


func_reference_type xc_ref_Vitos2000_052511 = {
  "L. Vitos, B. Johansson, J. Kollar, and H. L. Skriver, Phys. Rev. A 61, 052511 (2000)",
  "10.1103/PhysRevA.61.052511",
  "@article{Vitos2000_052511,\n  title = {Local kinetic-energy density of the Airy gas},\n  author = {L. Vitos and B. Johansson and J. Koll\\'{a}r and H. L. Skriver},\n  journal = {Phys. Rev. A},\n  volume = {61},\n  issue = {5},\n  pages = {052511},\n  year = {2000},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.61.052511},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.61.052511}\n}"
};


func_reference_type xc_ref_Vitos2000_10046 = {
  "L. Vitos, B. Johansson, J. Kollar, and H. L. Skriver, Phys. Rev. B 62, 10046 (2000)",
  "10.1103/PhysRevB.62.10046",
  "@article{Vitos2000_10046,\n  title = {Exchange energy in the local Airy gas approximation},\n  author = {L. Vitos and B. Johansson and J. Koll\\'{a}r and H. L. Skriver},\n  journal = {Phys. Rev. B},\n  volume = {62},\n  issue = {15},\n  pages = {10046},\n  year = {2000},\n  month = {Oct},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.62.10046},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.62.10046}\n}"
};


func_reference_type xc_ref_Vosko1980_1200 = {
  "S. H. Vosko, L. Wilk, and M. Nusair, Can. J. Phys. 58, 1200 (1980)",
  "10.1139/p80-159",
  "@article{Vosko1980_1200,\n  title = {Accurate spin-dependent electron liquid correlation energies for local spin density calculations: a critical analysis},\n  author = {S. H. Vosko and L. Wilk and M. Nusair},\n  journal = {Can. J. Phys.},\n  volume = {58},\n  number = {8},\n  pages = {1200},\n  year = {1980},\n  doi = {10.1139/p80-159},\n  url = {http://www.nrcresearchpress.com/doi/abs/10.1139/p80-159},\n}"
};


func_reference_type xc_ref_Vydrov2006_234109 = {
  "O. A. Vydrov and G. E. Scuseria, J. Chem. Phys. 125, 234109 (2006)",
  "10.1063/1.2409292",
  "@article{Vydrov2006_234109,\n  author = {Vydrov, Oleg A. and Scuseria, Gustavo E.},\n  title = {Assessment of a long-range corrected hybrid functional},\n  journal = {J. Chem. Phys.},\n  volume = {125},\n  number = {23},\n  pages = {234109},\n  year = {2006},\n  doi = {10.1063/1.2409292},\n  url = {https://doi.org/10.1063/1.2409292}\n}"
};


func_reference_type xc_ref_Vydrov2010_244103 = {
  "O. A. Vydrov and T. Van Voorhis, J. Chem. Phys. 133, 244103 (2010)",
  "10.1063/1.3521275",
  "@article{Vydrov2010_244103,\n  author = {Vydrov, Oleg A. and Van Voorhis, Troy},\n  title = {Nonlocal van der Waals density functional: The simpler the better},\n  journal = {J. Chem. Phys.},\n  volume = {133},\n  number = {24},\n  pages = {244103},\n  year = {2010},\n  doi = {10.1063/1.3521275},\n  url = {https://doi.org/10.1063/1.3521275}\n}"
};


func_reference_type xc_ref_Wang2017_8487 = {
  "Y. Wang, X. Jin, H. S. Yu, D. G. Truhlar, and X. He, Proc. Natl. Acad. Sci. U. S. A. 114, 8487 (2017)",
  "10.1073/pnas.1705670114",
  "@article{Wang2017_8487,\n  author = {Wang, Ying and Jin, Xinsheng and Yu, Haoyu S. and Truhlar, Donald G. and He, Xiao},\n  title = {Revised M06-L functional for improved accuracy on chemical reaction barrier heights, noncovalent interactions, and solid-state physics},\n  volume = {114},\n  number = {32},\n  pages = {8487-8492},\n  year = {2017},\n  doi = {10.1073/pnas.1705670114},\n  url = {http://www.pnas.org/content/114/32/8487.abstract},\n  journal = {Proc. Natl. Acad. Sci. U. S. A.}\n}"
};


func_reference_type xc_ref_Wang2018_10257 = {
  "Y. Wang, P. Verma, X. Jin, D. G. Truhlar, and X. He, Proc. Natl. Acad. Sci. U. S. A. 115, 10257 (2018)",
  "10.1073/pnas.1810421115",
  "@article {Wang2018_10257,\n  author = {Wang, Ying and Verma, Pragya and Jin, Xinsheng and Truhlar, Donald G. and He, Xiao},\n  title = {Revised M06 density functional for main-group and transition-metal chemistry},\n  volume = {115},\n  number = {41},\n  pages = {10257--10262},\n  year = {2018},\n  doi = {10.1073/pnas.1810421115},\n  publisher = {National Academy of Sciences},\n  issn = {0027-8424},\n  url = {https://www.pnas.org/content/115/41/10257},\n  journal = {Proc. Natl. Acad. Sci. U. S. A.}\n}"
};


func_reference_type xc_ref_Wang2020_2294 = {
  "Y. Wang, P. Verma, L. Zhang, Y. Li, Z. Liu, D. G. Truhlar, and X. He, Proc. Natl. Acad. Sci. U. S. A. 117, 2294 (2020)",
  "10.1073/pnas.1913699117",
  "@article{Wang2020_2294,\n  author = {Wang, Ying and Verma, Pragya and Zhang, Lujia and Li, Yaqi and Liu, Zhonghua and Truhlar, Donald G. and He, Xiao},\n  title = {M06-SX screened-exchange density functional for chemistry and solid-state physics},\n  volume = {117},\n  number = {5},\n  pages = {2294--2301},\n  year = {2020},\n  doi = {10.1073/pnas.1913699117},\n  publisher = {National Academy of Sciences},\n  issn = {0027-8424},\n  url = {https://www.pnas.org/content/117/5/2294},\n  journal = {Proc. Natl. Acad. Sci. U. S. A.}\n}"
};


func_reference_type xc_ref_Weintraub2009_754 = {
  "E. Weintraub, T. M. Henderson, and G. E. Scuseria, J. Chem. Theory Comput. 5, 754 (2009)",
  "10.1021/ct800530u",
  "@article{Weintraub2009_754,\n  title = {Long-Range-Corrected Hybrids Based on a New Model Exchange Hole},\n  author = {E. Weintraub and T. M. Henderson and G. E. Scuseria},\n  journal = {J. Chem. Theory Comput.},\n  volume = {5},\n  number = {4},\n  pages = {754-762},\n  year = {2009},\n  doi = {10.1021/ct800530u},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct800530u}\n}"
};


func_reference_type xc_ref_Weizsacker1935_431 = {
  "C. F. von Weizsacker, Z. Phys. 96, 431 (1935)",
  "10.1007/BF01337700",
  "@article{Weizsacker1935_431,\n  title = {Zur Theorie der Kernmassen},\n  author = {C. F. von Weizs\\\"acker},\n  journal = {Z. Phys.},\n  volume = {96},\n  number = {7-8},\n  pages = {431},\n  year = {1935},\n  issn = {0044-3328},\n  publisher = {Springer-Verlag},\n  language = {German},\n  doi = {10.1007/BF01337700},\n  url = {http://link.springer.com/article/10.1007\\%2FBF01337700}\n}"
};


func_reference_type xc_ref_Wellendorff2012_235149 = {
  "J. Wellendorff, K. T. Lundgaard, A. Mogelhoj, V. Petzold, D. D. Landis, J. K. Norskov, T. Bligaard, and K. W. Jacobsen, Phys. Rev. B 85, 235149 (2012)",
  "10.1103/PhysRevB.85.235149",
  "@article{Wellendorff2012_235149,\n  title = {Density functionals for surface science: Exchange-correlation model development with Bayesian error estimation},\n  author = {Wellendorff, Jess and Lundgaard, Keld T. and M\\o{}gelh\\o{}j, Andreas and Petzold, Vivien and Landis, David D. and N\\o{}rskov, Jens K. and Bligaard, Thomas and Jacobsen, Karsten W.},\n  journal = {Phys. Rev. B},\n  volume = {85},\n  issue = {23},\n  pages = {235149},\n  year = {2012},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.85.235149},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.85.235149}\n}"
};


func_reference_type xc_ref_Wellendorff2014_144107 = {
  "J. Wellendorff, K. T. Lundgaard, K. W. Jacobsen, and T. Bligaard, J. Chem. Phys. 140, 144107 (2014)",
  "10.1063/1.4870397",
  "@article{Wellendorff2014_144107,\n  author = {Wellendorff, Jess and Lundgaard, Keld T. and Jacobsen, Karsten W. and Bligaard, Thomas},\n  title = {mBEEF: An accurate semi-local Bayesian error estimation density functional},\n  journal = {J. Chem. Phys.},\n  year = {2014},\n  volume = {140},\n  number = {14},\n  pages = {144107},\n  url = {http://scitation.aip.org/content/aip/journal/jcp/140/14/10.1063/1.4870397},\n  doi = {10.1063/1.4870397}\n}"
};


func_reference_type xc_ref_Wigner1938_678 = {
  "E. Wigner, Trans. Faraday Soc. 34, 678 (1938)",
  "10.1039/TF9383400678",
  "@article{Wigner1938_678,\n  title = {Effects of the electron interaction on the energy levels of electrons in metals},\n  author = {E. Wigner},\n  journal = {Trans. Faraday Soc.},\n  volume = {34},\n  pages = {678},\n  year = {1938},\n  publisher = {The Royal Society of Chemistry},\n  doi = {10.1039/TF9383400678},\n  url = {10.1039/TF9383400678},\n}"
};


func_reference_type xc_ref_Wiitala2006_1085 = {
  "K. W. Wiitala, T. R. Hoye, and C. J. Cramer, J. Chem. Theory Comput. 2, 1085 (2006)",
  "10.1021/ct6001016",
  "@article{Wiitala2006_1085,\n  author = {Wiitala, Keith W. and Hoye, Thomas R. and Cramer, Christopher J.},\n  title = {Hybrid Density Functional Methods Empirically Optimized for the Computation of 13C and 1H Chemical Shifts in Chloroform Solution},\n  journal = {J. Chem. Theory Comput.},\n  volume = {2},\n  number = {4},\n  pages = {1085-1092},\n  year = {2006},\n  doi = {10.1021/ct6001016},\n  url = {https://doi.org/10.1021/ct6001016},\n}"
};


func_reference_type xc_ref_Wilson1990_12930 = {
  "L. C. Wilson and M. Levy, Phys. Rev. B 41, 12930 (1990)",
  "10.1103/PhysRevB.41.12930",
  "@article{Wilson1990_12930,\n  title = {Nonlocal Wigner-like correlation-energy density functional through coordinate scaling},\n  author = {L. C. Wilson and M. Levy},\n  journal = {Phys. Rev. B},\n  volume = {41},\n  issue = {18},\n  pages = {12930},\n  year = {1990},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.41.12930},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.41.12930}\n}"
};


func_reference_type xc_ref_Wilson1994_337 = {
  "L. C. Wilson, Chem. Phys. 181, 337 (1994)",
  "10.1016/0301-0104(93)E0444-Z",
  "@article{Wilson1994_337,\n  author = \"Leslie C. Wilson\",\n  title = \"Development of non-local Wigner-like correlation-energy density functionals through coordinate scaling requirements and evaluation of contraint obedience in the construction of density functionals\",\n  journal = \"Chem. Phys.\",\n  volume = \"181\",\n  number = \"3\",\n  pages = \"337 - 353\",\n  year = \"1994\",\n  issn = \"0301-0104\",\n  doi = \"10.1016/0301-0104(93)E0444-Z\",\n  url = \"http://www.sciencedirect.com/science/article/pii/0301010493E0444Z\"\n}"
};


func_reference_type xc_ref_Wilson1998_523 = {
  "L. C. Wilson and S. Ivanov, Int. J. Quantum Chem. 69, 523 (1998)",
  "10.1002/(SICI)1097-461X(1998)69:4<523::AID-QUA9>3.0.CO;2-X",
  "@article{Wilson1998_523,\n  author = {L. C. Wilson and S. Ivanov},\n  title = {A new Wigner-like correlation-energy functional from coordinate scaling requirements},\n  journal = {Int. J. Quantum Chem.},\n  volume = {69},\n  number = {4},\n  pages = {523},\n  year = {1998},\n  publisher = {John Wiley & Sons, Inc.},\n  issn = {1097-461X},\n  doi = {10.1002/(SICI)1097-461X(1998)69:4<523::AID-QUA9>3.0.CO;2-X},\n  url = {http://onlinelibrary.wiley.com/doi/10.1002/(SICI)1097-461X(1998)69:4<523::AID-QUA9>3.0.CO;2-X/abstract}\n}"
};


func_reference_type xc_ref_Wilson2001_9233 = {
  "P. J. Wilson, T. J. Bradley, and D. J. Tozer, J. Chem. Phys. 115, 9233 (2001)",
  "10.1063/1.1412605",
  "@article{Wilson2001_9233,\n  title = \"Hybrid exchange-correlation functional determined from thermochemical data and ab initio potentials\",\n  author = \"P. J. Wilson and T. J. Bradley and D. J. Tozer\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2001\",\n  volume = \"115\",\n  number = \"20\",\n  pages = \"9233\",\n  doi = \"10.1063/1.1412605\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/115/20/10.1063/1.1412605\"\n}"
};


func_reference_type xc_ref_Wu2006_235116 = {
  "Z. Wu and R. E. Cohen, Phys. Rev. B 73, 235116 (2006)",
  "10.1103/PhysRevB.73.235116",
  "@article{Wu2006_235116,\n  title = {More accurate generalized gradient approximation for solids},\n  author = {Z. Wu and R. E. Cohen},\n  journal = {Phys. Rev. B},\n  volume = {73},\n  issue = {23},\n  pages = {235116},\n  year = {2006},\n  month = {Jun},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.73.235116},\n  url = {http://link.aps.org/doi/10.1103/PhysRevB.73.235116}\n}"
};


func_reference_type xc_ref_Xie2021_045130 = {
  "Q.-X. Xie, J. Wu, and Y. Zhao, Phys. Rev. B 103, 045130 (2021)",
  "10.1103/PhysRevB.103.045130",
  "@article{Xie2021_045130,\n  title = {Accurate correlation energy functional for uniform electron gas from an interpolation ansatz without fitting parameters},\n  author = {Xie, Qing-Xing and Wu, Jiashun and Zhao, Yan},\n  journal = {Phys. Rev. B},\n  volume = {103},\n  issue = {4},\n  pages = {045130},\n  numpages = {6},\n  year = {2021},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevB.103.045130},\n  url = {https://link.aps.org/doi/10.1103/PhysRevB.103.045130}\n}"
};


func_reference_type xc_ref_Xu2004_2673 = {
  "X. Xu and W. A. Goddard, Proc. Natl. Acad. Sci. U. S. A. 101, 2673 (2004)",
  "10.1073/pnas.0308730100",
  "@article{Xu2004_2673,\n  title = {The X3LYP extended density functional for accurate descriptions of nonbond interactions, spin states, and thermochemical properties},\n  author = {X. Xu and W. A. Goddard},\n  volume = {101},\n  number = {9},\n  pages = {2673},\n  year = {2004},\n  doi = {10.1073/pnas.0308730100},\n  url = {http://www.pnas.org/content/101/9/2673.abstract},\n  journal = {Proc. Natl. Acad. Sci. U. S. A.}\n}"
};


func_reference_type xc_ref_Xu2004_4068 = {
  "X. Xu and W. A. Goddard, J. Chem. Phys. 121, 4068 (2004)",
  "10.1063/1.1771632",
  "@article{Xu2004_4068,\n  title = \"The extended Perdew--Burke--Ernzerhof functional with improved accuracy for thermodynamic and electronic properties of molecular systems\",\n  author = \"X. Xu and W. A. Goddard\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2004\",\n  volume = \"121\",\n  number = \"9\",\n  pages = \"4068\",\n  doi = \"10.1063/1.1771632\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/121/9/10.1063/1.1771632\"\n}"
};


func_reference_type xc_ref_Yanai2004_51 = {
  "T. Yanai, D. P. Tew, and N. C. Handy, Chem. Phys. Lett. 393, 51 (2004)",
  "10.1016/j.cplett.2004.06.011",
  "@article{Yanai2004_51,\n  title = \"A new hybrid exchange-correlation functional using the Coulomb-attenuating method (CAM-B3LYP)\",\n  author = \"T. Yanai and D. P. Tew and N. C. Handy\",\n  journal = \"Chem. Phys. Lett.\",\n  volume = \"393\",\n  number = \"1--3\",\n  pages = \"51\",\n  year = \"2004\",\n  issn = \"0009-2614\",\n  doi = \"10.1016/j.cplett.2004.06.011\",\n  url = \"http://www.sciencedirect.com/science/article/pii/S0009261404008620\"\n}"
};


func_reference_type xc_ref_Yonei1965_1051 = {
  "K. Yonei and Y. Tomishima, J. Phys. Soc. Jpn. 20, 1051 (1965)",
  "10.1143/JPSJ.20.1051",
  "@article{Yonei1965_1051,\n  title = {On the Weizs\\\"acker Correction to the Thomas--Fermi Theory of the Atom},\n  author = {K. Yonei and Y. Tomishima},\n  journal = {J. Phys. Soc. Jpn.},\n  volume = {20},\n  number = {6},\n  pages = {1051},\n  year = {1965},\n  doi = {10.1143/JPSJ.20.1051},\n  url = {http://journals.jps.jp/doi/abs/10.1143/JPSJ.20.1051},\n}"
};


func_reference_type xc_ref_Yu2015_12146 = {
  "H. S. Yu, W. Zhang, P. Verma, X. He, and D. G. Truhlar, Phys. Chem. Chem. Phys. 17, 12146 (2015)",
  "10.1039/C5CP01425E",
  "@article{Yu2015_12146,\n  author = \"Yu, Haoyu S. and Zhang, Wenjing and Verma, Pragya and He, Xiao and Truhlar, Donald G.\",\n  title = \"Nonseparable exchange-correlation functional for molecules{,} including homogeneous catalysis involving transition metals\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2015\",\n  volume = \"17\",\n  issue = \"18\",\n  pages = \"12146-12160\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C5CP01425E\",\n  url = \"http://doi.org/10.1039/C5CP01425E\",\n}"
};


func_reference_type xc_ref_Yu2016_1280 = {
  "H. S. Yu, X. He, and D. G. Truhlar, J. Chem. Theory Comput. 12, 1280 (2016)",
  "10.1021/acs.jctc.5b01082",
  "@article{Yu2016_1280,\n  author = {Haoyu S. Yu and Xiao He and Donald G. Truhlar},\n  title = {MN15-L: A New Local Exchange-Correlation Functional for Kohn-Sham Density Functional Theory with Broad Accuracy for Atoms, Molecules, and Solids},\n  journal = {J. Chem. Theory Comput.},\n  volume = {12},\n  number = {3},\n  pages = {1280-1293},\n  year = {2016},\n  doi = {10.1021/acs.jctc.5b01082},\n  url = {http://doi.org/10.1021/acs.jctc.5b01082}\n}"
};


func_reference_type xc_ref_Yu2016_5032 = {
  "H. S. Yu, X. He, S. L. Li, and D. G. Truhlar, Chem. Sci. 7, 5032 (2016)",
  "10.1039/C6SC00705H",
  "@article{Yu2016_5032,\n  author = \"Yu, Haoyu S. and He, Xiao and Li, Shaohong L. and Truhlar, Donald G.\",\n  title = \"MN15: A Kohn-Sham global-hybrid exchange-correlation density functional with broad accuracy for multi-reference and single-reference systems and noncovalent interactions\",\n  journal = \"Chem. Sci.\",\n  year = \"2016\",\n  volume = \"7\",\n  issue = \"8\",\n  pages = \"5032-5051\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/C6SC00705H\",\n  url = \"http://doi.org/10.1039/C6SC00705H\"\n}"
};


func_reference_type xc_ref_Zhang1998_890 = {
  "Y. Zhang and W. Yang, Phys. Rev. Lett. 80, 890 (1998)",
  "10.1103/PhysRevLett.80.890",
  "@article{Zhang1998_890,\n  title = {Comment on \"Generalized Gradient Approximation Made Simple\"},\n  author = {Y. Zhang and W. Yang},\n  journal = {Phys. Rev. Lett.},\n  volume = {80},\n  issue = {4},\n  pages = {890},\n  year = {1998},\n  month = {Jan},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevLett.80.890},\n  url = {http://link.aps.org/doi/10.1103/PhysRevLett.80.890}\n}"
};


func_reference_type xc_ref_Zhao1993_918 = {
  "Q. Zhao, M. Levy, and R. G. Parr, Phys. Rev. A 47, 918 (1993)",
  "10.1103/PhysRevA.47.918",
  "@article{Zhao1993_918,\n  title = {Applications of coordinate-scaling procedures to the exchange-correlation energy},\n  author = {Zhao, Qingsheng and Levy, Mel and Parr, Robert G.},\n  journal = {Phys. Rev. A},\n  volume = {47},\n  issue = {2},\n  pages = {918--922},\n  year = {1993},\n  month = {Feb},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.47.918},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.47.918}\n}"
};


func_reference_type xc_ref_Zhao2004_2715 = {
  "Y. Zhao, B. J. Lynch, and D. G. Truhlar, J. Phys. Chem. A 108, 2715 (2004)",
  "10.1021/jp049908s",
  "@article{Zhao2004_2715,\n  title = {Development and Assessment of a New Hybrid Density Functional Model for Thermochemical Kinetics},\n  author = {Y. Zhao and B. J. Lynch and D. G. Truhlar},\n  journal = {J. Phys. Chem. A},\n  volume = {108},\n  number = {14},\n  pages = {2715},\n  year = {2004},\n  doi = {10.1021/jp049908s},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp049908s}\n}"
};


func_reference_type xc_ref_Zhao2004_6908 = {
  "Y. Zhao and D. G. Truhlar, J. Phys. Chem. A 108, 6908 (2004)",
  "10.1021/jp048147q",
  "@article{Zhao2004_6908,\n  title = {Hybrid Meta Density Functional Theory Methods for Thermochemistry, Thermochemical Kinetics, and Noncovalent Interactoins: The MPW1B95 and MPWB1K Models and Comparative Assessments for Hydrogen Bonding and van der Waals Interactions},\n  author = {Y. Zhao and D. G. Truhlar},\n  journal = {J. Phys. Chem. A},\n  volume = {108},\n  number = {33},\n  pages = {6908},\n  year = {2004},\n  doi = {10.1021/jp048147q},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp048147q}\n}"
};


func_reference_type xc_ref_Zhao2005_161103 = {
  "Y. Zhao, N. E. Schultz, and D. G. Truhlar, J. Chem. Phys. 123, 161103 (2005)",
  "10.1063/1.2126975",
  "@article{Zhao2005_161103,\n  title = \"Exchange-correlation functional with broad accuracy for metallic and nonmetallic compounds, kinetics, and noncovalent interactions\",\n  author = \"Y. Zhao and N. E. Schultz and D. G. Truhlar\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2005\",\n  volume = \"123\",\n  number = \"16\",\n  pages = 161103,\n  doi = \"10.1063/1.2126975\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/123/16/10.1063/1.2126975\"\n}"
};


func_reference_type xc_ref_Zhao2005_2012 = {
  "Y. Zhao, N. Gonzalez-Garcia, and D. G. Truhlar, J. Phys. Chem. A 109, 2012 (2005)",
  "10.1021/jp045141s",
  "@article{Zhao2005_2012,\n  author = {Zhao, Yan and Gonz\\'{a}lez-Garc\\'{i}a, N\\'{u}ria and Truhlar, Donald G.},\n  title = {Benchmark Database of Barrier Heights for Heavy Atom Transfer, Nucleophilic Substitution, Association, and Unimolecular Reactions and Its Use to Test Theoretical Methods},\n  journal = {J. Phys. Chem. A},\n  volume = {109},\n  number = {9},\n  pages = {2012-2018},\n  year = {2005},\n  doi = {10.1021/jp045141s},\n  url = {http://doi.org/10.1021/jp045141s}\n}"
};


func_reference_type xc_ref_Zhao2005_415 = {
  "Y. Zhao and D. G. Truhlar, J. Chem. Theory Comput. 1, 415 (2005)",
  "10.1021/ct049851d",
  "@article{Zhao2005_415,\n  author = {Zhao, Yan and Truhlar, Donald G.},\n  title = {Benchmark Databases for Nonbonded Interactions and Their Use To Test Density Functional Theory},\n  journal = {J. Chem. Theory Comput.},\n  volume = {1},\n  number = {3},\n  pages = {415-432},\n  year = {2005},\n  doi = {10.1021/ct049851d},\n  url = {http://doi.org/10.1021/ct049851d}\n}"
};


func_reference_type xc_ref_Zhao2005_43 = {
  "Y. Zhao, B. J. Lynch, and D. G. Truhlar, Phys. Chem. Chem. Phys. 7, 43 (2005)",
  "10.1039/B416937A",
  "@article{Zhao2005_43,\n  author = \"Zhao, Yan and Lynch, Benjamin J. and Truhlar, Donald G.\",\n  title = \"Multi-coefficient extrapolated density functional theory for thermochemistry and thermochemical kinetics\",\n  journal = \"Phys. Chem. Chem. Phys.\",\n  year = \"2005\",\n  volume = \"7\",\n  issue = \"1\",\n  pages = \"43-52\",\n  publisher = \"The Royal Society of Chemistry\",\n  doi = \"10.1039/B416937A\",\n  url = \"http://doi.org/10.1039/B416937A\"\n}"
};


func_reference_type xc_ref_Zhao2005_5656 = {
  "Y. Zhao and D. G. Truhlar, J. Phys. Chem. A 109, 5656 (2005)",
  "10.1021/jp050536c",
  "@article{Zhao2005_5656,\n  title = {Design of Density Functionals That Are Broadly Accurate for Thermochemistry, Thermochemical Kinetics, and Nonbonded Interactions},\n  author = {Y. Zhao and D. G. Truhlar},\n  journal = {J. Phys. Chem. A},\n  volume = {109},\n  number = {25},\n  pages = {5656},\n  year = {2005},\n  doi = {10.1021/jp050536c},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp050536c}\n}"
};


func_reference_type xc_ref_Zhao2006_13126 = {
  "Y. Zhao and D. G. Truhlar, J. Phys. Chem. A 110, 13126 (2006)",
  "10.1021/jp066479k",
  "@article{Zhao2006_13126,\n  title = {Density Functional for Spectroscopy: No Long-Range Self-Interaction Error, Good Performance for Rydberg and Charge-Transfer States, and Better Performance on Average than B3LYP for Ground States},\n  author = {Y. Zhao and D. G. Truhlar},\n  journal = {J. Phys. Chem. A},\n  volume = {110},\n  number = {49},\n  pages = {13126},\n  year = {2006},\n  doi = {10.1021/jp066479k},\n  url = {http://pubs.acs.org/doi/abs/10.1021/jp066479k}\n}"
};


func_reference_type xc_ref_Zhao2006_194101 = {
  "Y. Zhao and D. G. Truhlar, J. Chem. Phys. 125, 194101 (2006)",
  "10.1063/1.2370993",
  "@article{Zhao2006_194101,\n  title = \"A new local density functional for main-group thermochemistry, transition metal bonding, thermochemical kinetics, and noncovalent interactions\",\n  author = \"Y. Zhao and D. G. Truhlar\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2006\",\n  volume = \"125\",\n  number = \"19\",\n  pages = 194101,\n  doi = \"10.1063/1.2370993\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/125/19/10.1063/1.2370993\"\n}"
};


func_reference_type xc_ref_Zhao2006_364 = {
  "Y. Zhao, N. E. Schultz, and D. G. Truhlar, J. Chem. Theory Comput. 2, 364 (2006)",
  "10.1021/ct0502763",
  "@article{Zhao2006_364,\n  title = {Design of Density Functionals by Combining the Method of Constraint Satisfaction with Parametrization for Thermochemistry, Thermochemical Kinetics, and Noncovalent Interactions},\n  author = {Y. Zhao and N. E. Schultz and D. G. Truhlar},\n  journal = {J. Chem. Theory Comput.},\n  volume = {2},\n  number = {2},\n  pages = {364},\n  year = {2006},\n  doi = {10.1021/ct0502763},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct0502763}\n}"
};


func_reference_type xc_ref_Zhao2008_184109 = {
  "Y. Zhao and D. G. Truhlar, J. Chem. Phys. 128, 184109 (2008)",
  "10.1063/1.2912068",
  "@article{Zhao2008_184109,\n  title = \"Construction of a generalized gradient approximation by restoring the density-gradient expansion and enforcing a tight Lieb--Oxford bound\",\n  author = \"Y. Zhao and D. G. Truhlar\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2008\",\n  volume = \"128\",\n  number = \"18\",\n  pages = 184109,\n  doi = \"10.1063/1.2912068\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/128/18/10.1063/1.2912068\"\n}"
};


func_reference_type xc_ref_Zhao2008_1849 = {
  "Y. Zhao and D. G. Truhlar, J. Chem. Theory Comput. 4, 1849 (2008)",
  "10.1021/ct800246v",
  "@article{Zhao2008_1849,\n  title = {Exploring the Limit of Accuracy of the Global Hybrid Meta Density Functional for Main-Group Thermochemistry, Kinetics, and Noncovalent Interactions},\n  author = {Y. Zhao and D. G. Truhlar},\n  journal = {J. Chem. Theory Comput.},\n  volume = {4},\n  number = {11},\n  pages = {1849},\n  year = {2008},\n  doi = {10.1021/ct800246v},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct800246v}\n}"
};


func_reference_type xc_ref_Zhao2008_215 = {
  "Y. Zhao and D. G. Truhlar, Theor. Chem. Acc. 120, 215 (2008)",
  "10.1007/s00214-007-0310-x",
  "@article{Zhao2008_215,\n  title = {The M06 suite of density functionals for main group thermochemistry, thermochemical kinetics, noncovalent interactions, excited states, and transition elements: two new functionals and systematic testing of four M06-class functionals and 12 other functionals},\n  author = {Y. Zhao and D. G. Truhlar},\n  journal = {Theor. Chem. Acc.},\n  volume = {120},\n  number = {1-3},\n  pages = {215},\n  year = {2008},\n  issn = {1432-881X},\n  publisher = {Springer-Verlag},\n  language = {English},\n  doi = {10.1007/s00214-007-0310-x},\n  url = {http://link.springer.com/article/10.1007\\%2Fs00214-007-0310-x}\n}"
};


func_reference_type xc_ref_Zheng2009_808 = {
  "J. Zheng, Y. Zhao, and D. G. Truhlar, J. Chem. Theory Comput. 5, 808 (2009)",
  "10.1021/ct800568m",
  "@article{Zheng2009_808,\n  title = {The DBH24/08 Database and Its Use to Assess Electronic Structure Model Chemistries for Chemical Reaction Barrier Heights},\n  author = {J. Zheng and Y. Zhao and D. G. Truhlar},\n  journal = {J. Chem. Theory Comput.},\n  volume = {5},\n  number = {4},\n  pages = {808},\n  year = {2009},\n  doi = {10.1021/ct800568m},\n  url = {http://pubs.acs.org/doi/abs/10.1021/ct800568m},\n}"
};


func_reference_type xc_ref_delCampo2012_104108 = {
  "J. M. del Campo, J. L. Gazquez, S. B. Trickey, and A. Vela, J. Chem. Phys. 136, 104108 (2012)",
  "10.1063/1.3691197",
  "@article{delCampo2012_104108,\n  title = \"Non-empirical improvement of PBE and its hybrid PBE0 for general description of molecular properties\",\n  author = \"J. M. del Campo and J. L. G\\'{a}zquez and S. B. Trickey and A. Vela\",\n  journal = \"J. Chem. Phys.\",\n  year = \"2012\",\n  volume = \"136\",\n  number = \"10\",\n  pages = 104108,\n  doi = \"10.1063/1.3691197\",\n  url = \"http://scitation.aip.org/content/aip/journal/jcp/136/10/10.1063/1.3691197\"\n}"
};


func_reference_type xc_ref_gaussianimplementation = {
  "Defined through Gaussian implementation",
  "",
  "@misc{gaussianimplementation,\n  note = {Defined through Gaussian implementation},\n}"
};


func_reference_type xc_ref_lda_xc_corrksdt_note = {
  "",
  "",
  "@misc{lda_xc_corrksdt_note,\n  note = {Karasiev has stated that the functional would need reparameterisation for spin-polarized functionals, so it should in principle be only used for spin-unpolarized calculations; see discussion in https://gitlab.com/libxc/libxc/-/merge\\_requests/465.},\n}"
};


func_reference_type xc_ref_nwchemimplementation = {
  "Defined through NWChem implementation",
  "",
  "@misc{nwchemimplementation,\n  note = {Defined through NWChem implementation},\n}"
};


func_reference_type xc_ref_vanLeeuwen1994_2421 = {
  "R. van Leeuwen and E. J. Baerends, Phys. Rev. A 49, 2421 (1994)",
  "10.1103/PhysRevA.49.2421",
  "@article{vanLeeuwen1994_2421,\n  title = {Exchange-correlation potential with correct asymptotic behavior},\n  author = {R. van Leeuwen and E. J. Baerends},\n  journal = {Phys. Rev. A},\n  volume = {49},\n  issue = {4},\n  pages = {2421},\n  year = {1994},\n  month = {Apr},\n  publisher = {American Physical Society},\n  doi = {10.1103/PhysRevA.49.2421},\n  url = {http://link.aps.org/doi/10.1103/PhysRevA.49.2421}\n}"
};


func_reference_type xc_ref_vonBarth1972_1629 = {
  "U. von Barth and L. Hedin, J. Phys. C: Solid State Phys. 5, 1629 (1972)",
  "10.1088/0022-3719/5/13/012",
  "@article{vonBarth1972_1629,\n  title = {A local exchange-correlation potential for the spin polarized case. i},\n  author = {U. von Barth and L. Hedin},\n  journal = {J. Phys. C: Solid State Phys.},\n  volume = {5},\n  number = {13},\n  pages = {1629},\n  year = {1972},\n  doi = {10.1088/0022-3719/5/13/012},\n  url = {http://stacks.iop.org/0022-3719/5/i=13/a=012}\n}"
};


